// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: state.proto
// Protobuf C++ Version: 7.36.0

#include "state.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::ClipProto
        {&::_pbi::kDescriptorMethods, &::descriptor_table_state_2eproto, /* tracker*/ nullptr,},
        // ::ClipListProto
        {&::_pbi::kDescriptorMethods, &::descriptor_table_state_2eproto, /* tracker*/ nullptr,},
        // ::StateProto
        {&::_pbi::kDescriptorMethods, &::descriptor_table_state_2eproto, /* tracker*/ nullptr,},
        // ::SettingsProto
        {&::_pbi::kDescriptorMethods, &::descriptor_table_state_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
class SettingsProto::_Internal {
 public:
  using HasBits = decltype(::std::declval<SettingsProto>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(SettingsProto, _impl_._has_bits_);
};

constexpr SettingsProto::ParseTableT_ SettingsProto::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(SettingsProto, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::SettingsProto>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // int32 interpolation_quality = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(SettingsProto, _impl_.interpolation_quality_), 2>(),
       {8, 2, 0,
        PROTOBUF_FIELD_OFFSET(SettingsProto, _impl_.interpolation_quality_)}},
      // double cue_in_point_padding_seconds = 2;
      {::_pbi::TcParser::FastF64S1,
       {17, 0, 0,
        PROTOBUF_FIELD_OFFSET(SettingsProto, _impl_.cue_in_point_padding_seconds_)}},
      // double cue_out_point_padding_seconds = 3;
      {::_pbi::TcParser::FastF64S1,
       {25, 1, 0,
        PROTOBUF_FIELD_OFFSET(SettingsProto, _impl_.cue_out_point_padding_seconds_)}},
    }}, {{
      65535, 65535
    }}, {{
      // int32 interpolation_quality = 1;
      {PROTOBUF_FIELD_OFFSET(SettingsProto, _impl_.interpolation_quality_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // double cue_in_point_padding_seconds = 2;
      {PROTOBUF_FIELD_OFFSET(SettingsProto, _impl_.cue_in_point_padding_seconds_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
      // double cue_out_point_padding_seconds = 3;
      {PROTOBUF_FIELD_OFFSET(SettingsProto, _impl_.cue_out_point_padding_seconds_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr SettingsProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : cue_in_point_padding_seconds_{0},
        cue_out_point_padding_seconds_{0},
        interpolation_quality_{0} {}

template <typename>
constexpr SettingsProto::SettingsProto(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL SettingsProto::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) SettingsProto(arena);
}
constexpr auto SettingsProto::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(SettingsProto), alignof(SettingsProto));
}
constexpr auto SettingsProto::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &SettingsProto::MergeImpl,
          Super_::GetNewImpl<SettingsProto>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &SettingsProto::SharedDtor,
          Super_::GetClearImpl<SettingsProto>(), &SettingsProto::ByteSizeLong,
              &SettingsProto::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(SettingsProto, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[3],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_state_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SettingsProtoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SettingsProtoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 SettingsProto_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(SettingsProto::InternalGenerateClassData_(
            _default, &SettingsProto_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<SettingsProto>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SettingsProtoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) SettingsProto _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<SettingsProto>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SettingsProtoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SettingsProtoGlobalsTypeInternal SettingsProto_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* SettingsProto_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return SettingsProto_globals_.GetClassData();
#else
  return SettingsProto_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ClipProto::_Internal {
 public:
  using HasBits = decltype(::std::declval<ClipProto>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ClipProto, _impl_._has_bits_);
};

constexpr ClipProto::ParseTableT_ ClipProto::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ClipProto, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967232,  // skipmap
      offsetof(ParseTableT_, field_entries),
      6,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::ClipProto>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // int64 pts_in = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(ClipProto, _impl_.pts_in_), 1>(),
       {8, 1, 0,
        PROTOBUF_FIELD_OFFSET(ClipProto, _impl_.pts_in_)}},
      // int64 pts_out = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(ClipProto, _impl_.pts_out_), 2>(),
       {16, 2, 0,
        PROTOBUF_FIELD_OFFSET(ClipProto, _impl_.pts_out_)}},
      // repeated string description = 3;
      {::_pbi::TcParser::FastUR1,
       {26, 0, 0,
        PROTOBUF_FIELD_OFFSET(ClipProto, _impl_.description_)}},
      // int64 stream_idx = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(ClipProto, _impl_.stream_idx_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(ClipProto, _impl_.stream_idx_)}},
      // double fade_time_seconds = 5;
      {::_pbi::TcParser::FastF64S1,
       {41, 4, 0,
        PROTOBUF_FIELD_OFFSET(ClipProto, _impl_.fade_time_seconds_)}},
      // double speed = 6;
      {::_pbi::TcParser::FastF64S1,
       {49, 5, 0,
        PROTOBUF_FIELD_OFFSET(ClipProto, _impl_.speed_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // int64 pts_in = 1;
      {PROTOBUF_FIELD_OFFSET(ClipProto, _impl_.pts_in_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // int64 pts_out = 2;
      {PROTOBUF_FIELD_OFFSET(ClipProto, _impl_.pts_out_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // repeated string description = 3;
      {PROTOBUF_FIELD_OFFSET(ClipProto, _impl_.description_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // int64 stream_idx = 4;
      {PROTOBUF_FIELD_OFFSET(ClipProto, _impl_.stream_idx_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // double fade_time_seconds = 5;
      {PROTOBUF_FIELD_OFFSET(ClipProto, _impl_.fade_time_seconds_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
      // double speed = 6;
      {PROTOBUF_FIELD_OFFSET(ClipProto, _impl_.speed_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
    }},
    // no aux_entries
    {{
      "\11\0\0\13\0\0\0\0"
      "ClipProto"
      "description"
    }},
  };
}


inline constexpr ClipProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : description_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::ClipProto,
            PROTOBUF_FIELD_OFFSET(::ClipProto, _impl_.description_)>()
         }
        ,
        pts_in_{::int64_t{0}},
        pts_out_{::int64_t{0}},
        stream_idx_{::int64_t{0}},
        fade_time_seconds_{0},
        speed_{0} {}

template <typename>
constexpr ClipProto::ClipProto(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ClipProto::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ClipProto(arena);
}
constexpr auto ClipProto::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ClipProto), alignof(ClipProto));
}
constexpr auto ClipProto::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ClipProto::MergeImpl,
          Super_::GetNewImpl<ClipProto>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ClipProto::SharedDtor,
          Super_::GetClearImpl<ClipProto>(), &ClipProto::ByteSizeLong,
              &ClipProto::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ClipProto, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_state_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ClipProtoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ClipProtoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ClipProto_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ClipProto::InternalGenerateClassData_(
            _default, &ClipProto_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ClipProto>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ClipProtoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ClipProto _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ClipProto>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ClipProtoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ClipProtoGlobalsTypeInternal ClipProto_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ClipProto_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ClipProto_globals_.GetClassData();
#else
  return ClipProto_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ClipListProto::_Internal {
 public:
  using HasBits = decltype(::std::declval<ClipListProto>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ClipListProto, _impl_._has_bits_);
};

constexpr ClipListProto::ParseTableT_ ClipListProto::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ClipListProto, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::ClipListProto>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated .ClipProto clip = 1;
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ClipListProto, _impl_.clip_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .ClipProto clip = 1;
      {PROTOBUF_FIELD_OFFSET(ClipListProto, _impl_.clip_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::ClipProto>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::ClipProto_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr ClipListProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : clip_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::ClipListProto,
            PROTOBUF_FIELD_OFFSET(::ClipListProto, _impl_.clip_)>()
         }
     {}

template <typename>
constexpr ClipListProto::ClipListProto(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ClipListProto::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ClipListProto(arena);
}
constexpr auto ClipListProto::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ClipListProto), alignof(ClipListProto));
}
constexpr auto ClipListProto::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ClipListProto::MergeImpl,
          Super_::GetNewImpl<ClipListProto>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ClipListProto::SharedDtor,
          Super_::GetClearImpl<ClipListProto>(), &ClipListProto::ByteSizeLong,
              &ClipListProto::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ClipListProto, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_state_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ClipListProtoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ClipListProtoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ClipListProto_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ClipListProto::InternalGenerateClassData_(
            _default, &ClipListProto_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ClipListProto>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ClipListProtoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ClipListProto _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ClipListProto>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ClipListProtoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ClipListProtoGlobalsTypeInternal ClipListProto_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ClipListProto_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ClipListProto_globals_.GetClassData();
#else
  return ClipListProto_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class StateProto::_Internal {
 public:
  using HasBits = decltype(::std::declval<StateProto>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(StateProto, _impl_._has_bits_);
};

constexpr StateProto::ParseTableT_ StateProto::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(StateProto, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::StateProto>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .ClipListProto play_list = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(StateProto, _impl_.play_list_)}},
      // .ClipListProto clip_list = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(StateProto, _impl_.clip_list_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .ClipListProto clip_list = 1;
      {PROTOBUF_FIELD_OFFSET(StateProto, _impl_.clip_list_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .ClipListProto play_list = 2;
      {PROTOBUF_FIELD_OFFSET(StateProto, _impl_.play_list_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::ClipListProto>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::ClipListProto_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::ClipListProto>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::ClipListProto_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr StateProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : clip_list_{nullptr},
        play_list_{nullptr} {}

template <typename>
constexpr StateProto::StateProto(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL StateProto::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) StateProto(arena);
}
constexpr auto StateProto::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(StateProto), alignof(StateProto));
}
constexpr auto StateProto::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &StateProto::MergeImpl,
          Super_::GetNewImpl<StateProto>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &StateProto::SharedDtor,
          Super_::GetClearImpl<StateProto>(), &StateProto::ByteSizeLong,
              &StateProto::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(StateProto, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_state_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct StateProtoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr StateProtoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 StateProto_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(StateProto::InternalGenerateClassData_(
            _default, &StateProto_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<StateProto>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~StateProtoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) StateProto _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<StateProto>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(StateProtoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST StateProtoGlobalsTypeInternal StateProto_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* StateProto_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return StateProto_globals_.GetClassData();
#else
  return StateProto_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
static constexpr const ::_pb::EnumDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_enum_descriptors_state_2eproto = nullptr;
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_state_2eproto = nullptr;
const ::uint32_t
    TableStruct_state_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::ClipProto, _impl_._has_bits_),
        9, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::ClipProto, _impl_.pts_in_),
        PROTOBUF_FIELD_OFFSET(::ClipProto, _impl_.pts_out_),
        PROTOBUF_FIELD_OFFSET(::ClipProto, _impl_.description_),
        PROTOBUF_FIELD_OFFSET(::ClipProto, _impl_.stream_idx_),
        PROTOBUF_FIELD_OFFSET(::ClipProto, _impl_.fade_time_seconds_),
        PROTOBUF_FIELD_OFFSET(::ClipProto, _impl_.speed_),
        1,
        2,
        0,
        3,
        4,
        5,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::ClipListProto, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::ClipListProto, _impl_.clip_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::StateProto, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::StateProto, _impl_.clip_list_),
        PROTOBUF_FIELD_OFFSET(::StateProto, _impl_.play_list_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::SettingsProto, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::SettingsProto, _impl_.interpolation_quality_),
        PROTOBUF_FIELD_OFFSET(::SettingsProto, _impl_.cue_in_point_padding_seconds_),
        PROTOBUF_FIELD_OFFSET(::SettingsProto, _impl_.cue_out_point_padding_seconds_),
        2,
        0,
        1,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::ClipProto)},
        {15, sizeof(::ClipListProto)},
        {20, sizeof(::StateProto)},
        {27, sizeof(::SettingsProto)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::ClipProto_globals_,
        &::ClipListProto_globals_,
        &::StateProto_globals_,
        &::SettingsProto_globals_,
};
const char descriptor_table_protodef_state_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\013state.proto\"\177\n\tClipProto\022\016\n\006pts_in\030\001 \001"
    "(\003\022\017\n\007pts_out\030\002 \001(\003\022\023\n\013description\030\003 \003(\t"
    "\022\022\n\nstream_idx\030\004 \001(\003\022\031\n\021fade_time_second"
    "s\030\005 \001(\001\022\r\n\005speed\030\006 \001(\001\")\n\rClipListProto\022"
    "\030\n\004clip\030\001 \003(\0132\n.ClipProto\"R\n\nStateProto\022"
    "!\n\tclip_list\030\001 \001(\0132\016.ClipListProto\022!\n\tpl"
    "ay_list\030\002 \001(\0132\016.ClipListProto\"{\n\rSetting"
    "sProto\022\035\n\025interpolation_quality\030\001 \001(\005\022$\n"
    "\034cue_in_point_padding_seconds\030\002 \001(\001\022%\n\035c"
    "ue_out_point_padding_seconds\030\003 \001(\001b\006prot"
    "o3"
};
static ::absl::once_flag descriptor_table_state_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_state_2eproto = {
    false,
    false,
    402,
    descriptor_table_protodef_state_2eproto,
    "state.proto",
    &descriptor_table_state_2eproto_once,
    nullptr,
    0,
    4,
    schemas,
    file_message_globals,
    TableStruct_state_2eproto::offsets,
    file_level_enum_descriptors_state_2eproto,
    file_level_service_descriptors_state_2eproto,
};
// ===================================================================

ClipProto::ClipProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ClipProto_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:ClipProto)
}
PROTOBUF_NDEBUG_INLINE ClipProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::ClipProto& from_msg)
      : _has_bits_{from._has_bits_},
        description_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::ClipProto,
              PROTOBUF_FIELD_OFFSET(::ClipProto, _impl_.description_)>()
          , arena, from.description_
        }
     {}

ClipProto::ClipProto(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ClipProto& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ClipProto_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ClipProto* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, pts_in_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, pts_in_),
           offsetof(Impl_, speed_) -
               offsetof(Impl_, pts_in_) +
               sizeof(Impl_::speed_));

  // @@protoc_insertion_point(copy_constructor:ClipProto)
}
PROTOBUF_NDEBUG_INLINE ClipProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : description_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::ClipProto,
            PROTOBUF_FIELD_OFFSET(::ClipProto, _impl_.description_)>()
         }
     {}

inline void ClipProto::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, pts_in_),
           0,
           offsetof(Impl_, speed_) -
               offsetof(Impl_, pts_in_) +
               sizeof(Impl_::speed_));
}
ClipProto::~ClipProto() {
  // @@protoc_insertion_point(destructor:ClipProto)
  SharedDtor(*this);
}
inline void ClipProto::SharedDtor(MessageLite& self) {
  ClipProto& this_ = static_cast<ClipProto&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ClipProto_class_data_ =
        ClipProto::InternalGenerateClassData_(ClipProto_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ClipProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ClipProto_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ClipProto_class_data_.tc_table);
  return ClipProto_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ClipProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ClipProto_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ClipProto_globals_));
  return ClipProto_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ClipProto::ParseTableT_
    ClipProto::_table_ =
        ClipProto::InternalGenerateParseTable_(ClipProto_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ClipProto::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:ClipProto)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.description_.Clear();
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000003eU)) {
    ::memset(&this_._impl_.pts_in_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.speed_) -
                 reinterpret_cast<char*>(&this_._impl_.pts_in_)) +
                 sizeof(_impl_.speed_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ClipProto::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ClipProto& this_ = static_cast<const ClipProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ClipProto::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ClipProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:ClipProto)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int64 pts_in = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_pts_in() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<1>(
              stream, this_._internal_pts_in(), target);
    }
  }

  // int64 pts_out = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_pts_out() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<2>(
              stream, this_._internal_pts_out(), target);
    }
  }

  // repeated string description = 3;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_description_size(); i < n; ++i) {
      const auto& s = this_._internal_description().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "ClipProto.description");
      target = stream->WriteString(3, s, target);
    }
  }

  // int64 stream_idx = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_stream_idx() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<4>(
              stream, this_._internal_stream_idx(), target);
    }
  }

  // double fade_time_seconds = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (::absl::bit_cast<::uint64_t>(this_._internal_fade_time_seconds()) != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteDoubleToArray(
          5, this_._internal_fade_time_seconds(), target);
    }
  }

  // double speed = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (::absl::bit_cast<::uint64_t>(this_._internal_speed()) != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteDoubleToArray(
          6, this_._internal_speed(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:ClipProto)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ClipProto::ByteSizeLong(const MessageLite& base) {
  const ClipProto& this_ = static_cast<const ClipProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ClipProto::ByteSizeLong() const {
  const ClipProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:ClipProto)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    // repeated string description = 3;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_description().size());
      for (int i = 0, n = this_._internal_description().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_description().Get(i));
      }
    }
    // int64 pts_in = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_pts_in() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_pts_in());
      }
    }
    // int64 pts_out = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_pts_out() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_pts_out());
      }
    }
    // int64 stream_idx = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_stream_idx() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_stream_idx());
      }
    }
    // double fade_time_seconds = 5;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (::absl::bit_cast<::uint64_t>(this_._internal_fade_time_seconds()) != 0) {
        total_size += 9;
      }
    }
    // double speed = 6;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (::absl::bit_cast<::uint64_t>(this_._internal_speed()) != 0) {
        total_size += 9;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ClipProto::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ClipProto*>(&to_msg);
  auto& from = static_cast<const ClipProto&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:ClipProto)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_description()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_description());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_pts_in() != 0) {
        _this->_impl_.pts_in_ = from._impl_.pts_in_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_pts_out() != 0) {
        _this->_impl_.pts_out_ = from._impl_.pts_out_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_stream_idx() != 0) {
        _this->_impl_.stream_idx_ = from._impl_.stream_idx_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (::absl::bit_cast<::uint64_t>(from._internal_fade_time_seconds()) != 0) {
        _this->_impl_.fade_time_seconds_ = from._impl_.fade_time_seconds_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (::absl::bit_cast<::uint64_t>(from._internal_speed()) != 0) {
        _this->_impl_.speed_ = from._impl_.speed_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ClipProto::CopyFrom(const ClipProto& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:ClipProto)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ClipProto::InternalSwap(ClipProto* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.description_.InternalSwap(&other->_impl_.description_);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ClipProto, _impl_.speed_)
      + sizeof(ClipProto::_impl_.speed_)
      - PROTOBUF_FIELD_OFFSET(ClipProto, _impl_.pts_in_)>(
          reinterpret_cast<char*>(&_impl_.pts_in_),
          reinterpret_cast<char*>(&other->_impl_.pts_in_));
}

::google::protobuf::Metadata ClipProto::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ClipListProto::ClipListProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ClipListProto_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:ClipListProto)
}
PROTOBUF_NDEBUG_INLINE ClipListProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::ClipListProto& from_msg)
      : _has_bits_{from._has_bits_},
        clip_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::ClipListProto,
              PROTOBUF_FIELD_OFFSET(::ClipListProto, _impl_.clip_)>()
          , arena, from.clip_
        }
     {}

ClipListProto::ClipListProto(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ClipListProto& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ClipListProto_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ClipListProto* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:ClipListProto)
}
PROTOBUF_NDEBUG_INLINE ClipListProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : clip_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::ClipListProto,
            PROTOBUF_FIELD_OFFSET(::ClipListProto, _impl_.clip_)>()
         }
     {}

inline void ClipListProto::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ClipListProto::~ClipListProto() {
  // @@protoc_insertion_point(destructor:ClipListProto)
  SharedDtor(*this);
}
inline void ClipListProto::SharedDtor(MessageLite& self) {
  ClipListProto& this_ = static_cast<ClipListProto&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ClipListProto_class_data_ =
        ClipListProto::InternalGenerateClassData_(ClipListProto_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ClipListProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ClipListProto_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ClipListProto_class_data_.tc_table);
  return ClipListProto_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ClipListProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ClipListProto_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ClipListProto_globals_));
  return ClipListProto_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ClipListProto::ParseTableT_
    ClipListProto::_table_ =
        ClipListProto::InternalGenerateParseTable_(ClipListProto_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ClipListProto::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:ClipListProto)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.clip_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ClipListProto::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ClipListProto& this_ = static_cast<const ClipListProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ClipListProto::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ClipListProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:ClipListProto)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .ClipProto clip = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_clip_size());
         i < n; i++) {
      const auto& repfield = this_._internal_clip().Get(i);
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          1, repfield, repfield.GetCachedSize(), target,
          stream);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:ClipListProto)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ClipListProto::ByteSizeLong(const MessageLite& base) {
  const ClipListProto& this_ = static_cast<const ClipListProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ClipListProto::ByteSizeLong() const {
  const ClipListProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:ClipListProto)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated .ClipProto clip = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_clip_size();
      for (const auto& msg : this_._internal_clip()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ClipListProto::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ClipListProto*>(&to_msg);
  auto& from = static_cast<const ClipListProto&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:ClipListProto)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _this->_internal_mutable_clip()->InternalMergeFromWithArena(
        ::google::protobuf::MessageLite::internal_visibility(), arena,
        from._internal_clip());
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ClipListProto::CopyFrom(const ClipListProto& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:ClipListProto)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ClipListProto::InternalSwap(ClipListProto* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.clip_.InternalSwap(&other->_impl_.clip_);
}

::google::protobuf::Metadata ClipListProto::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

StateProto::StateProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, StateProto_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:StateProto)
}
PROTOBUF_NDEBUG_INLINE StateProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::StateProto& from_msg)
      : _has_bits_{from._has_bits_} {}

StateProto::StateProto(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const StateProto& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, StateProto_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  StateProto* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.clip_list_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.clip_list_)
                 : nullptr;
  _impl_.play_list_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.play_list_)
                 : nullptr;

  // @@protoc_insertion_point(copy_constructor:StateProto)
}
PROTOBUF_NDEBUG_INLINE StateProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void StateProto::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, clip_list_),
           0,
           offsetof(Impl_, play_list_) -
               offsetof(Impl_, clip_list_) +
               sizeof(Impl_::play_list_));
}
StateProto::~StateProto() {
  // @@protoc_insertion_point(destructor:StateProto)
  SharedDtor(*this);
}
inline void StateProto::SharedDtor(MessageLite& self) {
  StateProto& this_ = static_cast<StateProto&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.clip_list_;
  delete this_._impl_.play_list_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull StateProto_class_data_ =
        StateProto::InternalGenerateClassData_(StateProto_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StateProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StateProto_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(StateProto_class_data_.tc_table);
  return StateProto_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StateProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StateProto_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&StateProto_globals_));
  return StateProto_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const StateProto::ParseTableT_
    StateProto::_table_ =
        StateProto::InternalGenerateParseTable_(StateProto_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void StateProto::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:StateProto)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.clip_list_ != nullptr);
      this_._impl_.clip_list_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.play_list_ != nullptr);
      this_._impl_.play_list_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL StateProto::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const StateProto& this_ = static_cast<const StateProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL StateProto::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const StateProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:StateProto)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .ClipListProto clip_list = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.clip_list_, this_._impl_.clip_list_->GetCachedSize(), target,
        stream);
  }

  // .ClipListProto play_list = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.play_list_, this_._impl_.play_list_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:StateProto)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t StateProto::ByteSizeLong(const MessageLite& base) {
  const StateProto& this_ = static_cast<const StateProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t StateProto::ByteSizeLong() const {
  const StateProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:StateProto)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // .ClipListProto clip_list = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.clip_list_);
    }
    // .ClipListProto play_list = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.play_list_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void StateProto::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<StateProto*>(&to_msg);
  auto& from = static_cast<const StateProto&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:StateProto)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.clip_list_ != nullptr);
      if (_this->_impl_.clip_list_ == nullptr) {
        _this->_impl_.clip_list_ = Super_::CopyConstruct(arena, *from._impl_.clip_list_);
      } else {
        _this->_impl_.clip_list_->MergeFrom(*from._impl_.clip_list_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.play_list_ != nullptr);
      if (_this->_impl_.play_list_ == nullptr) {
        _this->_impl_.play_list_ = Super_::CopyConstruct(arena, *from._impl_.play_list_);
      } else {
        _this->_impl_.play_list_->MergeFrom(*from._impl_.play_list_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void StateProto::CopyFrom(const StateProto& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:StateProto)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void StateProto::InternalSwap(StateProto* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(StateProto, _impl_.play_list_)
      + sizeof(StateProto::_impl_.play_list_)
      - PROTOBUF_FIELD_OFFSET(StateProto, _impl_.clip_list_)>(
          reinterpret_cast<char*>(&_impl_.clip_list_),
          reinterpret_cast<char*>(&other->_impl_.clip_list_));
}

::google::protobuf::Metadata StateProto::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

SettingsProto::SettingsProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, SettingsProto_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:SettingsProto)
}
SettingsProto::SettingsProto(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SettingsProto& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, SettingsProto_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE SettingsProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void SettingsProto::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, cue_in_point_padding_seconds_),
           0,
           offsetof(Impl_, interpolation_quality_) -
               offsetof(Impl_, cue_in_point_padding_seconds_) +
               sizeof(Impl_::interpolation_quality_));
}
SettingsProto::~SettingsProto() {
  // @@protoc_insertion_point(destructor:SettingsProto)
  SharedDtor(*this);
}
inline void SettingsProto::SharedDtor(MessageLite& self) {
  SettingsProto& this_ = static_cast<SettingsProto&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull SettingsProto_class_data_ =
        SettingsProto::InternalGenerateClassData_(SettingsProto_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SettingsProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SettingsProto_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(SettingsProto_class_data_.tc_table);
  return SettingsProto_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SettingsProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SettingsProto_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&SettingsProto_globals_));
  return SettingsProto_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const SettingsProto::ParseTableT_
    SettingsProto::_table_ =
        SettingsProto::InternalGenerateParseTable_(SettingsProto_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void SettingsProto::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:SettingsProto)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    ::memset(&this_._impl_.cue_in_point_padding_seconds_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.interpolation_quality_) -
                 reinterpret_cast<char*>(&this_._impl_.cue_in_point_padding_seconds_)) +
                 sizeof(_impl_.interpolation_quality_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL SettingsProto::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const SettingsProto& this_ = static_cast<const SettingsProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL SettingsProto::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const SettingsProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:SettingsProto)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int32 interpolation_quality = 1;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_interpolation_quality() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<1>(
              stream, this_._internal_interpolation_quality(), target);
    }
  }

  // double cue_in_point_padding_seconds = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (::absl::bit_cast<::uint64_t>(this_._internal_cue_in_point_padding_seconds()) != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteDoubleToArray(
          2, this_._internal_cue_in_point_padding_seconds(), target);
    }
  }

  // double cue_out_point_padding_seconds = 3;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (::absl::bit_cast<::uint64_t>(this_._internal_cue_out_point_padding_seconds()) != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteDoubleToArray(
          3, this_._internal_cue_out_point_padding_seconds(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:SettingsProto)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t SettingsProto::ByteSizeLong(const MessageLite& base) {
  const SettingsProto& this_ = static_cast<const SettingsProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t SettingsProto::ByteSizeLong() const {
  const SettingsProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:SettingsProto)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // double cue_in_point_padding_seconds = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (::absl::bit_cast<::uint64_t>(this_._internal_cue_in_point_padding_seconds()) != 0) {
        total_size += 9;
      }
    }
    // double cue_out_point_padding_seconds = 3;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (::absl::bit_cast<::uint64_t>(this_._internal_cue_out_point_padding_seconds()) != 0) {
        total_size += 9;
      }
    }
    // int32 interpolation_quality = 1;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_interpolation_quality() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_interpolation_quality());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void SettingsProto::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<SettingsProto*>(&to_msg);
  auto& from = static_cast<const SettingsProto&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:SettingsProto)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (::absl::bit_cast<::uint64_t>(from._internal_cue_in_point_padding_seconds()) != 0) {
        _this->_impl_.cue_in_point_padding_seconds_ = from._impl_.cue_in_point_padding_seconds_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (::absl::bit_cast<::uint64_t>(from._internal_cue_out_point_padding_seconds()) != 0) {
        _this->_impl_.cue_out_point_padding_seconds_ = from._impl_.cue_out_point_padding_seconds_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_interpolation_quality() != 0) {
        _this->_impl_.interpolation_quality_ = from._impl_.interpolation_quality_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void SettingsProto::CopyFrom(const SettingsProto& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:SettingsProto)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void SettingsProto::InternalSwap(SettingsProto* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(SettingsProto, _impl_.interpolation_quality_)
      + sizeof(SettingsProto::_impl_.interpolation_quality_)
      - PROTOBUF_FIELD_OFFSET(SettingsProto, _impl_.cue_in_point_padding_seconds_)>(
          reinterpret_cast<char*>(&_impl_.cue_in_point_padding_seconds_),
          reinterpret_cast<char*>(&other->_impl_.cue_in_point_padding_seconds_));
}

::google::protobuf::Metadata SettingsProto::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_state_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
