/********************************************************************************
** Form generated from reading UI file 'aboutdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUTDIALOG_H
#define UI_ABOUTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_AboutDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *header;
    QTextEdit *textEdit;
    QDialogButtonBox *button_box;

    void setupUi(QDialog *AboutDialog)
    {
        if (AboutDialog->objectName().isEmpty())
            AboutDialog->setObjectName(QString::fromUtf8("AboutDialog"));
        AboutDialog->resize(684, 544);
        verticalLayout = new QVBoxLayout(AboutDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        header = new QLabel(AboutDialog);
        header->setObjectName(QString::fromUtf8("header"));

        verticalLayout->addWidget(header);

        textEdit = new QTextEdit(AboutDialog);
        textEdit->setObjectName(QString::fromUtf8("textEdit"));
        textEdit->setReadOnly(true);

        verticalLayout->addWidget(textEdit);

        button_box = new QDialogButtonBox(AboutDialog);
        button_box->setObjectName(QString::fromUtf8("button_box"));
        button_box->setOrientation(Qt::Horizontal);
        button_box->setStandardButtons(QDialogButtonBox::Ok);

        verticalLayout->addWidget(button_box);


        retranslateUi(AboutDialog);
        QObject::connect(button_box, SIGNAL(accepted()), AboutDialog, SLOT(accept()));
        QObject::connect(button_box, SIGNAL(rejected()), AboutDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(AboutDialog);
    } // setupUi

    void retranslateUi(QDialog *AboutDialog)
    {
        AboutDialog->setWindowTitle(QCoreApplication::translate("AboutDialog", "About Nageru", nullptr));
        header->setText(QCoreApplication::translate("AboutDialog", "<p><b>@PROGRAM@ @NAGERU_VERSION@</b></p>\n"
"\n"
"<p>@SUBHEADING@</p>", nullptr));
        textEdit->setHtml(QCoreApplication::translate("AboutDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /></head><body>\n"
"<p>\n"
"Nageru is Copyright (C) 2015 Steinar H. Gunderson &lt;steinar+nageru@gunderson.no&gt;<br />\n"
"Portions Copyright (C) 2003 Rune Holm.<br />\n"
"Portions Copyright (C) 2010-2011 Fons Adriaensen &lt;fons@linuxaudio.org&gt;<br />\n"
"Portions Copyright (C) 2012-2015 Fons Adriaensen &lt;fons@linuxaudio.org&gt;<br />\n"
"Portions Copyright (C) 2008-2015 Fons Adriaensen &lt;fons@linuxaudio.org&gt;<br />\n"
"Portions Copyright (c) 2007-2013 Intel Corporation. All Rights Reserved.</p>\n"
"\n"
"<p>This program is free software: you can redistribute it and/or modify<br />\n"
"it under the terms of the GNU General Public License as published by<br />\n"
"the Free Software Foundation, either version 3 of the License, or<br />\n"
"(at your option) any later version.</p>\n"
"\n"
"<p>This program is distributed in the hope that it will be useful"
                        ",<br />\n"
"but WITHOUT ANY WARRANTY; without even the implied warranty of<br />\n"
"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the<br />\n"
"GNU General Public License for more details.</p>\n"
"\n"
"<p>You should have received a copy of the GNU General Public License<br />\n"
"along with this program. If not, see &lt;<a href=\"http://www.gnu.org/licenses/\"><span style=\" text-decoration: underline; color:#0000ff;\">http://www.gnu.org/licenses/</span></a>&gt;.</p>\n"
"\n"
"<p><br />Portions of h264encode.h and h264encode.cpp:</p>\n"
"\n"
"<p>Copyright (c) 2007-2013 Intel Corporation. All Rights Reserved.</p>\n"
"\n"
"<p>Permission is hereby granted, free of charge, to any person obtaining a<br />\n"
"copy of this software and associated documentation files (the<br />\n"
"&quot;Software&quot;), to deal in the Software without restriction, including<br />\n"
"without limitation the rights to use, copy, modify, merge, publish,<br />\n"
"distribute, sub license, and/or sell copies of the Software, a"
                        "nd to<br />\n"
"permit persons to whom the Software is furnished to do so, subject to<br />\n"
"the following conditions:</p>\n"
"\n"
"<p>The above copyright notice and this permission notice (including the<br />\n"
"next paragraph) shall be included in all copies or substantial portions<br />\n"
"of the Software.</p>\n"
"\n"
"<p>THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS<br />\n"
"OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF<br />\n"
"MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.<br />\n"
"IN NO EVENT SHALL PRECISION INSIGHT AND/OR ITS SUPPLIERS BE LIABLE FOR<br />\n"
"ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,<br />\n"
"TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE<br />\n"
"SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.</p>\n"
"\n"
"<p><br />All files in decklink/:</p>\n"
"\n"
"<p>Copyright (c) 2009 Blackmagic Design<br />\n"
"Copyright (c) 2015 Blackmagic Design</p>\n"
"\n"
""
                        "<p>Permission is hereby granted, free of charge, to any person or organization\n"
"obtaining a copy of the software and accompanying documentation covered by\n"
"this license (the \"Software\") to use, reproduce, display, distribute,\n"
"execute, and transmit the Software, and to prepare derivative works of the\n"
"Software, and to permit third-parties to whom the Software is furnished to\n"
"do so, all subject to the following:</p>\n"
"\n"
"<p>The copyright notices in the Software and this entire statement, including\n"
"the above license grant, this restriction and the following disclaimer,\n"
"must be included in all copies of the Software, in whole or in part, and\n"
"all derivative works of the Software, unless such copies or derivative\n"
"works are solely in the form of machine-executable object code generated by\n"
"a source language processor.</p>\n"
"\n"
"<p>THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n"
"IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHAN"
                        "TABILITY,\n"
"FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT\n"
"SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE\n"
"FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,\n"
"ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\n"
"DEALINGS IN THE SOFTWARE.</p>\n"
"\n"
"<p>Marked parts of theme.cpp (Lua shims):</p>\n"
"\n"
"<p>The MIT License (MIT)</p>\n"
"\n"
"<p>Copyright (c) 2013 Hisham Muhammad</p>\n"
"\n"
"<p>Permission is hereby granted, free of charge, to any person obtaining a copy of\n"
"this software and associated documentation files (the \"Software\"), to deal in\n"
"the Software without restriction, including without limitation the rights to\n"
"use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of\n"
"the Software, and to permit persons to whom the Software is furnished to do so,\n"
"subject to the following conditions:</p>\n"
"\n"
"<p>The above copyright notice and this pe"
                        "rmission notice shall be included in all\n"
"copies or substantial portions of the Software.</p>\n"
"\n"
"<p>THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n"
"IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS\n"
"FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR\n"
"COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER\n"
"IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN\n"
"CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.</p>\n"
"\n"
"</body></html>\n"
"       ", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AboutDialog: public Ui_AboutDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ABOUTDIALOG_H
