/********************************************************************************
** Form generated from reading UI file 'analyzer.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ANALYZER_H
#define UI_ANALYZER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "glwidget.h"
#include "rgb_histogram.h"

QT_BEGIN_NAMESPACE

class Ui_Analyzer
{
public:
    QWidget *centralwidget;
    QHBoxLayout *horizontalLayout_3;
    QVBoxLayout *left_pane;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *display_centerer;
    QSpacerItem *display_left_spacer;
    GLWidget *display;
    QSpacerItem *display_right_spacer;
    QHBoxLayout *grab_horizontal_layout;
    QComboBox *input_box;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label;
    QComboBox *grab_frequency_box;
    QPushButton *grab_btn;
    RGBHistogram *histogram;
    QLabel *histogram_label;
    QSpacerItem *verticalSpacer_2;
    QVBoxLayout *right_pane;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *grabbed_frame_enterer;
    QSpacerItem *grabbed_frame_left_spacer;
    QLabel *grabbed_frame_label;
    QSpacerItem *grabbed_frame_right_spacer;
    QLabel *grabbed_frame_sublabel;
    QLabel *coord_label;
    QHBoxLayout *color_hbox;
    QVBoxLayout *verticalLayout_4;
    QLabel *label_3;
    QVBoxLayout *verticalLayout_3;
    QLabel *label_12;
    QGridLayout *gridLayout;
    QLabel *label_6;
    QLabel *green_label;
    QLabel *blue_label;
    QLabel *label_2;
    QLabel *red_label;
    QLabel *label_7;
    QLabel *hex_label;
    QLabel *label_5;
    QSpacerItem *verticalSpacer_4;

    void setupUi(QMainWindow *Analyzer)
    {
        if (Analyzer->objectName().isEmpty())
            Analyzer->setObjectName(QString::fromUtf8("Analyzer"));
        Analyzer->resize(845, 472);
        centralwidget = new QWidget(Analyzer);
        centralwidget->setObjectName(QString::fromUtf8("centralwidget"));
        horizontalLayout_3 = new QHBoxLayout(centralwidget);
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        left_pane = new QVBoxLayout();
        left_pane->setObjectName(QString::fromUtf8("left_pane"));
        verticalSpacer = new QSpacerItem(20, 5, QSizePolicy::Minimum, QSizePolicy::Expanding);

        left_pane->addItem(verticalSpacer);

        display_centerer = new QHBoxLayout();
        display_centerer->setSpacing(0);
        display_centerer->setObjectName(QString::fromUtf8("display_centerer"));
        display_left_spacer = new QSpacerItem(5, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        display_centerer->addItem(display_left_spacer);

        display = new GLWidget(centralwidget);
        display->setObjectName(QString::fromUtf8("display"));
        display->setEnabled(true);
        display->setAutoFillBackground(false);
        display->setStyleSheet(QString::fromUtf8("background: rgb(233, 185, 110)"));

        display_centerer->addWidget(display);

        display_right_spacer = new QSpacerItem(5, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        display_centerer->addItem(display_right_spacer);

        display_centerer->setStretch(1, 1);

        left_pane->addLayout(display_centerer);

        grab_horizontal_layout = new QHBoxLayout();
        grab_horizontal_layout->setObjectName(QString::fromUtf8("grab_horizontal_layout"));
        input_box = new QComboBox(centralwidget);
        input_box->setObjectName(QString::fromUtf8("input_box"));

        grab_horizontal_layout->addWidget(input_box);

        grab_horizontal_layout->setStretch(0, 1);

        left_pane->addLayout(grab_horizontal_layout);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        label = new QLabel(centralwidget);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout_4->addWidget(label);

        grab_frequency_box = new QComboBox(centralwidget);
        grab_frequency_box->setObjectName(QString::fromUtf8("grab_frequency_box"));

        horizontalLayout_4->addWidget(grab_frequency_box);

        grab_btn = new QPushButton(centralwidget);
        grab_btn->setObjectName(QString::fromUtf8("grab_btn"));

        horizontalLayout_4->addWidget(grab_btn);

        horizontalLayout_4->setStretch(1, 1);

        left_pane->addLayout(horizontalLayout_4);

        histogram = new RGBHistogram(centralwidget);
        histogram->setObjectName(QString::fromUtf8("histogram"));
        histogram->setAutoFillBackground(false);
        histogram->setStyleSheet(QString::fromUtf8("background: rgb(173, 127, 168)"));

        left_pane->addWidget(histogram);

        histogram_label = new QLabel(centralwidget);
        histogram_label->setObjectName(QString::fromUtf8("histogram_label"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(histogram_label->sizePolicy().hasHeightForWidth());
        histogram_label->setSizePolicy(sizePolicy);
        histogram_label->setAlignment(Qt::AlignCenter);

        left_pane->addWidget(histogram_label);

        verticalSpacer_2 = new QSpacerItem(20, 5, QSizePolicy::Minimum, QSizePolicy::Expanding);

        left_pane->addItem(verticalSpacer_2);

        left_pane->setStretch(1, 1);
        left_pane->setStretch(4, 1);

        horizontalLayout_3->addLayout(left_pane);

        right_pane = new QVBoxLayout();
        right_pane->setObjectName(QString::fromUtf8("right_pane"));
        verticalSpacer_3 = new QSpacerItem(20, 5, QSizePolicy::Minimum, QSizePolicy::Expanding);

        right_pane->addItem(verticalSpacer_3);

        grabbed_frame_enterer = new QHBoxLayout();
        grabbed_frame_enterer->setSpacing(0);
        grabbed_frame_enterer->setObjectName(QString::fromUtf8("grabbed_frame_enterer"));
        grabbed_frame_left_spacer = new QSpacerItem(5, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        grabbed_frame_enterer->addItem(grabbed_frame_left_spacer);

        grabbed_frame_label = new QLabel(centralwidget);
        grabbed_frame_label->setObjectName(QString::fromUtf8("grabbed_frame_label"));
        QSizePolicy sizePolicy1(QSizePolicy::Ignored, QSizePolicy::Ignored);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(1);
        sizePolicy1.setHeightForWidth(grabbed_frame_label->sizePolicy().hasHeightForWidth());
        grabbed_frame_label->setSizePolicy(sizePolicy1);
        grabbed_frame_label->setCursor(QCursor(Qt::CrossCursor));
        grabbed_frame_label->setMouseTracking(true);
        grabbed_frame_label->setAutoFillBackground(false);
        grabbed_frame_label->setStyleSheet(QString::fromUtf8("background: color(0,0,0)"));
        grabbed_frame_label->setScaledContents(true);

        grabbed_frame_enterer->addWidget(grabbed_frame_label);

        grabbed_frame_right_spacer = new QSpacerItem(5, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        grabbed_frame_enterer->addItem(grabbed_frame_right_spacer);

        grabbed_frame_enterer->setStretch(1, 1);

        right_pane->addLayout(grabbed_frame_enterer);

        grabbed_frame_sublabel = new QLabel(centralwidget);
        grabbed_frame_sublabel->setObjectName(QString::fromUtf8("grabbed_frame_sublabel"));
        grabbed_frame_sublabel->setAlignment(Qt::AlignCenter);

        right_pane->addWidget(grabbed_frame_sublabel);

        coord_label = new QLabel(centralwidget);
        coord_label->setObjectName(QString::fromUtf8("coord_label"));
        coord_label->setAlignment(Qt::AlignCenter);

        right_pane->addWidget(coord_label);

        color_hbox = new QHBoxLayout();
        color_hbox->setObjectName(QString::fromUtf8("color_hbox"));
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        label_3 = new QLabel(centralwidget);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        verticalLayout_4->addWidget(label_3);


        color_hbox->addLayout(verticalLayout_4);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        label_12 = new QLabel(centralwidget);
        label_12->setObjectName(QString::fromUtf8("label_12"));
        label_12->setAlignment(Qt::AlignCenter);

        verticalLayout_3->addWidget(label_12);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label_6 = new QLabel(centralwidget);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        gridLayout->addWidget(label_6, 1, 0, 1, 1);

        green_label = new QLabel(centralwidget);
        green_label->setObjectName(QString::fromUtf8("green_label"));

        gridLayout->addWidget(green_label, 1, 1, 1, 1);

        blue_label = new QLabel(centralwidget);
        blue_label->setObjectName(QString::fromUtf8("blue_label"));

        gridLayout->addWidget(blue_label, 2, 1, 1, 1);

        label_2 = new QLabel(centralwidget);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 3, 0, 1, 1);

        red_label = new QLabel(centralwidget);
        red_label->setObjectName(QString::fromUtf8("red_label"));

        gridLayout->addWidget(red_label, 0, 1, 1, 1);

        label_7 = new QLabel(centralwidget);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        gridLayout->addWidget(label_7, 2, 0, 1, 1);

        hex_label = new QLabel(centralwidget);
        hex_label->setObjectName(QString::fromUtf8("hex_label"));

        gridLayout->addWidget(hex_label, 3, 1, 1, 1);

        label_5 = new QLabel(centralwidget);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        gridLayout->addWidget(label_5, 0, 0, 1, 1);


        verticalLayout_3->addLayout(gridLayout);


        color_hbox->addLayout(verticalLayout_3);


        right_pane->addLayout(color_hbox);

        verticalSpacer_4 = new QSpacerItem(20, 5, QSizePolicy::Minimum, QSizePolicy::Expanding);

        right_pane->addItem(verticalSpacer_4);

        right_pane->setStretch(1, 1);

        horizontalLayout_3->addLayout(right_pane);

        horizontalLayout_3->setStretch(0, 2);
        horizontalLayout_3->setStretch(1, 3);
        Analyzer->setCentralWidget(centralwidget);

        retranslateUi(Analyzer);

        QMetaObject::connectSlotsByName(Analyzer);
    } // setupUi

    void retranslateUi(QMainWindow *Analyzer)
    {
        Analyzer->setWindowTitle(QCoreApplication::translate("Analyzer", "Analyzer", nullptr));
        label->setText(QCoreApplication::translate("Analyzer", "Grab every:", nullptr));
        grab_btn->setText(QCoreApplication::translate("Analyzer", "Grab", nullptr));
        histogram_label->setText(QCoreApplication::translate("Analyzer", "RGB histogram", nullptr));
        grabbed_frame_label->setText(QString());
        grabbed_frame_sublabel->setText(QCoreApplication::translate("Analyzer", "Grabbed frame", nullptr));
        coord_label->setText(QCoreApplication::translate("Analyzer", "Selected coordinate (x,y): (none)", nullptr));
        label_3->setText(QString());
        label_12->setText(QCoreApplication::translate("Analyzer", "Color (8-bit sRGB):", nullptr));
        label_6->setText(QCoreApplication::translate("Analyzer", "Green:", nullptr));
        green_label->setText(QCoreApplication::translate("Analyzer", "\342\200\224", nullptr));
        blue_label->setText(QCoreApplication::translate("Analyzer", "\342\200\224", nullptr));
        label_2->setText(QCoreApplication::translate("Analyzer", "Hex:", nullptr));
        red_label->setText(QCoreApplication::translate("Analyzer", "\342\200\224", nullptr));
        label_7->setText(QCoreApplication::translate("Analyzer", "Blue:", nullptr));
        hex_label->setText(QCoreApplication::translate("Analyzer", "#\342\200\224", nullptr));
        label_5->setText(QCoreApplication::translate("Analyzer", "Red:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Analyzer: public Ui_Analyzer {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ANALYZER_H
