// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: frame.proto
// Protobuf C++ Version: 7.36.0

#ifndef frame_2eproto_2epb_2eh
#define frame_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7036000
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_frame_2eproto

// Internal implementation detail -- do not use these members.
struct TableStruct_frame_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_frame_2eproto;
}  // extern "C"
class FileContentsProto;
struct FileContentsProtoGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern FileContentsProtoGlobalsTypeInternal FileContentsProto_globals_;
extern const ::google::protobuf::internal::ClassDataFull FileContentsProto_class_data_;
#else
extern const FileContentsProtoGlobalsTypeInternal FileContentsProto_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class FrameHeaderProto;
struct FrameHeaderProtoGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern FrameHeaderProtoGlobalsTypeInternal FrameHeaderProto_globals_;
extern const ::google::protobuf::internal::ClassDataFull FrameHeaderProto_class_data_;
#else
extern const FrameHeaderProtoGlobalsTypeInternal FrameHeaderProto_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class StreamContentsProto;
struct StreamContentsProtoGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern StreamContentsProtoGlobalsTypeInternal StreamContentsProto_globals_;
extern const ::google::protobuf::internal::ClassDataFull StreamContentsProto_class_data_;
#else
extern const StreamContentsProtoGlobalsTypeInternal StreamContentsProto_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google


// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED StreamContentsProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:StreamContentsProto) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline StreamContentsProto() : StreamContentsProto(nullptr) {}
  ~StreamContentsProto() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(StreamContentsProto* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(StreamContentsProto));
  }
#endif

  template <typename = void>
  explicit constexpr StreamContentsProto(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline StreamContentsProto(const StreamContentsProto& from) : StreamContentsProto(nullptr, from) {}
  inline StreamContentsProto(StreamContentsProto&& from) noexcept : StreamContentsProto(nullptr, ::std::move(from)) {}
  inline StreamContentsProto& operator=(const StreamContentsProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline StreamContentsProto& operator=(StreamContentsProto&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const StreamContentsProto& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<StreamContentsProto>(&StreamContentsProto_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(StreamContentsProto& a, StreamContentsProto& b) { a.Swap(&b); }
  inline void Swap(StreamContentsProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(StreamContentsProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] StreamContentsProto* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<StreamContentsProto>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const StreamContentsProto& from);
  using Super_::MergeFrom;
  void MergeFrom(const StreamContentsProto& from) { StreamContentsProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(StreamContentsProto* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "StreamContentsProto"; }

  explicit StreamContentsProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  StreamContentsProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const StreamContentsProto& from);
  StreamContentsProto(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, StreamContentsProto&& from) noexcept
      : StreamContentsProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPtsFieldNumber = 2,
    kFileSizeFieldNumber = 3,
    kOffsetFieldNumber = 4,
    kAudioSizeFieldNumber = 5,
    kStreamIdxFieldNumber = 1,
  };
  // repeated int64 pts = 2 [packed = true];
  [[nodiscard]] int pts_size() const;
  private:
  int _internal_pts_size() const;

  public:
  void clear_pts() ;
  [[nodiscard]] ::int64_t pts(int index) const;
  void set_pts(int index, ::int64_t value);
  void add_pts(::int64_t value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<::int64_t>& pts()
      const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL mutable_pts();

  private:
  const ::google::protobuf::RepeatedField<::int64_t>& _internal_pts() const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL _internal_mutable_pts();

  public:
  // repeated int64 file_size = 3 [packed = true];
  [[nodiscard]] int file_size_size() const;
  private:
  int _internal_file_size_size() const;

  public:
  void clear_file_size() ;
  [[nodiscard]] ::int64_t file_size(int index) const;
  void set_file_size(int index, ::int64_t value);
  void add_file_size(::int64_t value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<::int64_t>& file_size()
      const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL mutable_file_size();

  private:
  const ::google::protobuf::RepeatedField<::int64_t>& _internal_file_size() const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL _internal_mutable_file_size();

  public:
  // repeated int64 offset = 4 [packed = true];
  [[nodiscard]] int offset_size() const;
  private:
  int _internal_offset_size() const;

  public:
  void clear_offset() ;
  [[nodiscard]] ::int64_t offset(int index) const;
  void set_offset(int index, ::int64_t value);
  void add_offset(::int64_t value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<::int64_t>& offset()
      const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL mutable_offset();

  private:
  const ::google::protobuf::RepeatedField<::int64_t>& _internal_offset() const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL _internal_mutable_offset();

  public:
  // repeated int32 audio_size = 5 [packed = true];
  [[nodiscard]] int audio_size_size() const;
  private:
  int _internal_audio_size_size() const;

  public:
  void clear_audio_size() ;
  [[nodiscard]] ::int32_t audio_size(int index) const;
  void set_audio_size(int index, ::int32_t value);
  void add_audio_size(::int32_t value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<::int32_t>& audio_size()
      const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL mutable_audio_size();

  private:
  const ::google::protobuf::RepeatedField<::int32_t>& _internal_audio_size() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL _internal_mutable_audio_size();

  public:
  // int32 stream_idx = 1;
  void clear_stream_idx() ;
  [[nodiscard]] ::int32_t stream_idx() const;
  void set_stream_idx(::int32_t value);

  private:
  ::int32_t _internal_stream_idx() const;
  void _internal_set_stream_idx(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:StreamContentsProto)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 5,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const StreamContentsProto& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedField<::int64_t> pts_;
    ::google::protobuf::RepeatedField<::int64_t> file_size_;
    ::google::protobuf::RepeatedField<::int64_t> offset_;
    ::google::protobuf::RepeatedField<::int32_t> audio_size_;
    ::int32_t stream_idx_;
    ::google::protobuf::internal::CachedSize _pts_cached_byte_size_;
    ::google::protobuf::internal::CachedSize _file_size_cached_byte_size_;
    ::google::protobuf::internal::CachedSize _offset_cached_byte_size_;
    ::google::protobuf::internal::CachedSize _audio_size_cached_byte_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_frame_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED FrameHeaderProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:FrameHeaderProto) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline FrameHeaderProto() : FrameHeaderProto(nullptr) {}
  ~FrameHeaderProto() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(FrameHeaderProto* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(FrameHeaderProto));
  }
#endif

  template <typename = void>
  explicit constexpr FrameHeaderProto(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline FrameHeaderProto(const FrameHeaderProto& from) : FrameHeaderProto(nullptr, from) {}
  inline FrameHeaderProto(FrameHeaderProto&& from) noexcept : FrameHeaderProto(nullptr, ::std::move(from)) {}
  inline FrameHeaderProto& operator=(const FrameHeaderProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline FrameHeaderProto& operator=(FrameHeaderProto&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const FrameHeaderProto& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<FrameHeaderProto>(&FrameHeaderProto_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(FrameHeaderProto& a, FrameHeaderProto& b) { a.Swap(&b); }
  inline void Swap(FrameHeaderProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(FrameHeaderProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] FrameHeaderProto* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<FrameHeaderProto>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const FrameHeaderProto& from);
  using Super_::MergeFrom;
  void MergeFrom(const FrameHeaderProto& from) { FrameHeaderProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(FrameHeaderProto* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "FrameHeaderProto"; }

  explicit FrameHeaderProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  FrameHeaderProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const FrameHeaderProto& from);
  FrameHeaderProto(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, FrameHeaderProto&& from) noexcept
      : FrameHeaderProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPtsFieldNumber = 2,
    kStreamIdxFieldNumber = 1,
    kAudioSizeFieldNumber = 4,
    kFileSizeFieldNumber = 3,
  };
  // int64 pts = 2;
  void clear_pts() ;
  [[nodiscard]] ::int64_t pts() const;
  void set_pts(::int64_t value);

  private:
  ::int64_t _internal_pts() const;
  void _internal_set_pts(::int64_t value);

  public:
  // int32 stream_idx = 1;
  void clear_stream_idx() ;
  [[nodiscard]] ::int32_t stream_idx() const;
  void set_stream_idx(::int32_t value);

  private:
  ::int32_t _internal_stream_idx() const;
  void _internal_set_stream_idx(::int32_t value);

  public:
  // int32 audio_size = 4;
  void clear_audio_size() ;
  [[nodiscard]] ::int32_t audio_size() const;
  void set_audio_size(::int32_t value);

  private:
  ::int32_t _internal_audio_size() const;
  void _internal_set_audio_size(::int32_t value);

  public:
  // int64 file_size = 3;
  void clear_file_size() ;
  [[nodiscard]] ::int64_t file_size() const;
  void set_file_size(::int64_t value);

  private:
  ::int64_t _internal_file_size() const;
  void _internal_set_file_size(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:FrameHeaderProto)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 4,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const FrameHeaderProto& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::int64_t pts_;
    ::int32_t stream_idx_;
    ::int32_t audio_size_;
    ::int64_t file_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_frame_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED FileContentsProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:FileContentsProto) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline FileContentsProto() : FileContentsProto(nullptr) {}
  ~FileContentsProto() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(FileContentsProto* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(FileContentsProto));
  }
#endif

  template <typename = void>
  explicit constexpr FileContentsProto(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline FileContentsProto(const FileContentsProto& from) : FileContentsProto(nullptr, from) {}
  inline FileContentsProto(FileContentsProto&& from) noexcept : FileContentsProto(nullptr, ::std::move(from)) {}
  inline FileContentsProto& operator=(const FileContentsProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline FileContentsProto& operator=(FileContentsProto&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const FileContentsProto& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<FileContentsProto>(&FileContentsProto_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(FileContentsProto& a, FileContentsProto& b) { a.Swap(&b); }
  inline void Swap(FileContentsProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(FileContentsProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] FileContentsProto* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<FileContentsProto>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const FileContentsProto& from);
  using Super_::MergeFrom;
  void MergeFrom(const FileContentsProto& from) { FileContentsProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(FileContentsProto* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "FileContentsProto"; }

  explicit FileContentsProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  FileContentsProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const FileContentsProto& from);
  FileContentsProto(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, FileContentsProto&& from) noexcept
      : FileContentsProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kStreamFieldNumber = 1,
  };
  // repeated .StreamContentsProto stream = 1;
  [[nodiscard]] int stream_size() const;
  private:
  int _internal_stream_size() const;

  public:
  void clear_stream() ;
  [[nodiscard]] const ::StreamContentsProto& stream(int index) const;
  [[nodiscard]] ::StreamContentsProto* PROTOBUF_NONNULL mutable_stream(int index);
  ::StreamContentsProto* PROTOBUF_NONNULL add_stream();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::StreamContentsProto>&
  stream() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::StreamContentsProto>* PROTOBUF_NONNULL
  mutable_stream();

  private:
  const ::google::protobuf::RepeatedPtrField<::StreamContentsProto>& _internal_stream() const;
  ::google::protobuf::RepeatedPtrField<::StreamContentsProto>* PROTOBUF_NONNULL _internal_mutable_stream();

  public:
  // @@protoc_insertion_point(class_scope:FileContentsProto)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const FileContentsProto& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::StreamContentsProto > stream_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_frame_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// FrameHeaderProto

// int32 stream_idx = 1;
inline void FrameHeaderProto::clear_stream_idx() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.stream_idx_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::int32_t FrameHeaderProto::stream_idx() const {
  // @@protoc_insertion_point(field_get:FrameHeaderProto.stream_idx)
  return _internal_stream_idx();
}
inline void FrameHeaderProto::set_stream_idx(::int32_t value) {
  _internal_set_stream_idx(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:FrameHeaderProto.stream_idx)
}
inline ::int32_t FrameHeaderProto::_internal_stream_idx() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.stream_idx_;
}
inline void FrameHeaderProto::_internal_set_stream_idx(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.stream_idx_ = value;
}

// int64 pts = 2;
inline void FrameHeaderProto::clear_pts() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pts_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::int64_t FrameHeaderProto::pts() const {
  // @@protoc_insertion_point(field_get:FrameHeaderProto.pts)
  return _internal_pts();
}
inline void FrameHeaderProto::set_pts(::int64_t value) {
  _internal_set_pts(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:FrameHeaderProto.pts)
}
inline ::int64_t FrameHeaderProto::_internal_pts() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pts_;
}
inline void FrameHeaderProto::_internal_set_pts(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pts_ = value;
}

// int64 file_size = 3;
inline void FrameHeaderProto::clear_file_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.file_size_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::int64_t FrameHeaderProto::file_size() const {
  // @@protoc_insertion_point(field_get:FrameHeaderProto.file_size)
  return _internal_file_size();
}
inline void FrameHeaderProto::set_file_size(::int64_t value) {
  _internal_set_file_size(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:FrameHeaderProto.file_size)
}
inline ::int64_t FrameHeaderProto::_internal_file_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.file_size_;
}
inline void FrameHeaderProto::_internal_set_file_size(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.file_size_ = value;
}

// int32 audio_size = 4;
inline void FrameHeaderProto::clear_audio_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.audio_size_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::int32_t FrameHeaderProto::audio_size() const {
  // @@protoc_insertion_point(field_get:FrameHeaderProto.audio_size)
  return _internal_audio_size();
}
inline void FrameHeaderProto::set_audio_size(::int32_t value) {
  _internal_set_audio_size(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:FrameHeaderProto.audio_size)
}
inline ::int32_t FrameHeaderProto::_internal_audio_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.audio_size_;
}
inline void FrameHeaderProto::_internal_set_audio_size(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.audio_size_ = value;
}

// -------------------------------------------------------------------

// StreamContentsProto

// int32 stream_idx = 1;
inline void StreamContentsProto::clear_stream_idx() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.stream_idx_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int32_t StreamContentsProto::stream_idx() const {
  // @@protoc_insertion_point(field_get:StreamContentsProto.stream_idx)
  return _internal_stream_idx();
}
inline void StreamContentsProto::set_stream_idx(::int32_t value) {
  _internal_set_stream_idx(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:StreamContentsProto.stream_idx)
}
inline ::int32_t StreamContentsProto::_internal_stream_idx() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.stream_idx_;
}
inline void StreamContentsProto::_internal_set_stream_idx(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.stream_idx_ = value;
}

// repeated int64 pts = 2 [packed = true];
inline int StreamContentsProto::_internal_pts_size() const {
  return _internal_pts().size();
}
inline int StreamContentsProto::pts_size() const {
  return _internal_pts_size();
}
inline void StreamContentsProto::clear_pts() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pts_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::int64_t StreamContentsProto::pts(int index) const {
  // @@protoc_insertion_point(field_get:StreamContentsProto.pts)
  return _internal_pts().Get(index);
}
inline void StreamContentsProto::set_pts(int index, ::int64_t value) {
  _internal_mutable_pts()->Set(index, value);
  // @@protoc_insertion_point(field_set:StreamContentsProto.pts)
}
inline void StreamContentsProto::add_pts(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_pts()
      ->InternalAddWithArena<const ::google::protobuf::MessageLite*>(
          internal_visibility(), this, value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:StreamContentsProto.pts)
}
inline const ::google::protobuf::RepeatedField<::int64_t>& StreamContentsProto::pts() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:StreamContentsProto.pts)
  return _internal_pts();
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL StreamContentsProto::mutable_pts()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:StreamContentsProto.pts)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_pts();
}
inline const ::google::protobuf::RepeatedField<::int64_t>&
StreamContentsProto::_internal_pts() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pts_;
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL
StreamContentsProto::_internal_mutable_pts() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.pts_;
}

// repeated int64 file_size = 3 [packed = true];
inline int StreamContentsProto::_internal_file_size_size() const {
  return _internal_file_size().size();
}
inline int StreamContentsProto::file_size_size() const {
  return _internal_file_size_size();
}
inline void StreamContentsProto::clear_file_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.file_size_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::int64_t StreamContentsProto::file_size(int index) const {
  // @@protoc_insertion_point(field_get:StreamContentsProto.file_size)
  return _internal_file_size().Get(index);
}
inline void StreamContentsProto::set_file_size(int index, ::int64_t value) {
  _internal_mutable_file_size()->Set(index, value);
  // @@protoc_insertion_point(field_set:StreamContentsProto.file_size)
}
inline void StreamContentsProto::add_file_size(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_file_size()
      ->InternalAddWithArena<const ::google::protobuf::MessageLite*>(
          internal_visibility(), this, value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:StreamContentsProto.file_size)
}
inline const ::google::protobuf::RepeatedField<::int64_t>& StreamContentsProto::file_size() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:StreamContentsProto.file_size)
  return _internal_file_size();
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL StreamContentsProto::mutable_file_size()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:StreamContentsProto.file_size)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_file_size();
}
inline const ::google::protobuf::RepeatedField<::int64_t>&
StreamContentsProto::_internal_file_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.file_size_;
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL
StreamContentsProto::_internal_mutable_file_size() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.file_size_;
}

// repeated int64 offset = 4 [packed = true];
inline int StreamContentsProto::_internal_offset_size() const {
  return _internal_offset().size();
}
inline int StreamContentsProto::offset_size() const {
  return _internal_offset_size();
}
inline void StreamContentsProto::clear_offset() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.offset_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::int64_t StreamContentsProto::offset(int index) const {
  // @@protoc_insertion_point(field_get:StreamContentsProto.offset)
  return _internal_offset().Get(index);
}
inline void StreamContentsProto::set_offset(int index, ::int64_t value) {
  _internal_mutable_offset()->Set(index, value);
  // @@protoc_insertion_point(field_set:StreamContentsProto.offset)
}
inline void StreamContentsProto::add_offset(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_offset()
      ->InternalAddWithArena<const ::google::protobuf::MessageLite*>(
          internal_visibility(), this, value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_add:StreamContentsProto.offset)
}
inline const ::google::protobuf::RepeatedField<::int64_t>& StreamContentsProto::offset() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:StreamContentsProto.offset)
  return _internal_offset();
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL StreamContentsProto::mutable_offset()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_mutable_list:StreamContentsProto.offset)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_offset();
}
inline const ::google::protobuf::RepeatedField<::int64_t>&
StreamContentsProto::_internal_offset() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.offset_;
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL
StreamContentsProto::_internal_mutable_offset() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.offset_;
}

// repeated int32 audio_size = 5 [packed = true];
inline int StreamContentsProto::_internal_audio_size_size() const {
  return _internal_audio_size().size();
}
inline int StreamContentsProto::audio_size_size() const {
  return _internal_audio_size_size();
}
inline void StreamContentsProto::clear_audio_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.audio_size_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::int32_t StreamContentsProto::audio_size(int index) const {
  // @@protoc_insertion_point(field_get:StreamContentsProto.audio_size)
  return _internal_audio_size().Get(index);
}
inline void StreamContentsProto::set_audio_size(int index, ::int32_t value) {
  _internal_mutable_audio_size()->Set(index, value);
  // @@protoc_insertion_point(field_set:StreamContentsProto.audio_size)
}
inline void StreamContentsProto::add_audio_size(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_audio_size()
      ->InternalAddWithArena<const ::google::protobuf::MessageLite*>(
          internal_visibility(), this, value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_add:StreamContentsProto.audio_size)
}
inline const ::google::protobuf::RepeatedField<::int32_t>& StreamContentsProto::audio_size() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:StreamContentsProto.audio_size)
  return _internal_audio_size();
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL StreamContentsProto::mutable_audio_size()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_mutable_list:StreamContentsProto.audio_size)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_audio_size();
}
inline const ::google::protobuf::RepeatedField<::int32_t>&
StreamContentsProto::_internal_audio_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.audio_size_;
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL
StreamContentsProto::_internal_mutable_audio_size() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.audio_size_;
}

// -------------------------------------------------------------------

// FileContentsProto

// repeated .StreamContentsProto stream = 1;
inline int FileContentsProto::_internal_stream_size() const {
  return _internal_stream().size();
}
inline int FileContentsProto::stream_size() const {
  return _internal_stream_size();
}
inline void FileContentsProto::clear_stream() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.stream_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::StreamContentsProto& FileContentsProto::stream(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:FileContentsProto.stream)
  return _internal_stream().Get(index);
}
inline ::StreamContentsProto* PROTOBUF_NONNULL FileContentsProto::mutable_stream(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:FileContentsProto.stream)
  return _internal_mutable_stream()->Mutable(index);
}
inline ::StreamContentsProto* PROTOBUF_NONNULL FileContentsProto::add_stream()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::StreamContentsProto* _add =
      _internal_mutable_stream()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:FileContentsProto.stream)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::StreamContentsProto>& FileContentsProto::stream() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:FileContentsProto.stream)
  return _internal_stream();
}
inline ::google::protobuf::RepeatedPtrField<::StreamContentsProto>* PROTOBUF_NONNULL
FileContentsProto::mutable_stream() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:FileContentsProto.stream)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_stream();
}
inline const ::google::protobuf::RepeatedPtrField<::StreamContentsProto>&
FileContentsProto::_internal_stream() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.stream_;
}
inline ::google::protobuf::RepeatedPtrField<::StreamContentsProto>* PROTOBUF_NONNULL
FileContentsProto::_internal_mutable_stream() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.stream_;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // frame_2eproto_2epb_2eh
