/********************************************************************************
** Form generated from reading UI file 'midi_mapping.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MIDI_MAPPING_H
#define UI_MIDI_MAPPING_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_MIDIMappingDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTreeWidget *treeWidget;
    QLabel *label_3;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *save_button;
    QPushButton *load_button;
    QSpacerItem *horizontalSpacer;
    QDialogButtonBox *ok_cancel_buttons;

    void setupUi(QDialog *MIDIMappingDialog)
    {
        if (MIDIMappingDialog->objectName().isEmpty())
            MIDIMappingDialog->setObjectName(QString::fromUtf8("MIDIMappingDialog"));
        MIDIMappingDialog->resize(879, 583);
        verticalLayout = new QVBoxLayout(MIDIMappingDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        treeWidget = new QTreeWidget(MIDIMappingDialog);
        QTreeWidgetItem *__qtreewidgetitem = new QTreeWidgetItem();
        __qtreewidgetitem->setText(0, QString::fromUtf8("1"));
        treeWidget->setHeaderItem(__qtreewidgetitem);
        treeWidget->setObjectName(QString::fromUtf8("treeWidget"));

        verticalLayout->addWidget(treeWidget);

        label_3 = new QLabel(MIDIMappingDialog);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        verticalLayout->addWidget(label_3);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        save_button = new QPushButton(MIDIMappingDialog);
        save_button->setObjectName(QString::fromUtf8("save_button"));

        horizontalLayout_2->addWidget(save_button);

        load_button = new QPushButton(MIDIMappingDialog);
        load_button->setObjectName(QString::fromUtf8("load_button"));

        horizontalLayout_2->addWidget(load_button);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        ok_cancel_buttons = new QDialogButtonBox(MIDIMappingDialog);
        ok_cancel_buttons->setObjectName(QString::fromUtf8("ok_cancel_buttons"));
        ok_cancel_buttons->setOrientation(Qt::Horizontal);
        ok_cancel_buttons->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        horizontalLayout_2->addWidget(ok_cancel_buttons);

        horizontalLayout_2->setStretch(2, 1);

        verticalLayout->addLayout(horizontalLayout_2);


        retranslateUi(MIDIMappingDialog);

        QMetaObject::connectSlotsByName(MIDIMappingDialog);
    } // setupUi

    void retranslateUi(QDialog *MIDIMappingDialog)
    {
        MIDIMappingDialog->setWindowTitle(QCoreApplication::translate("MIDIMappingDialog", "MIDI controller setup", nullptr));
        label_3->setText(QCoreApplication::translate("MIDIMappingDialog", "Add or change a mapping by clicking in the cell, then moving the corresponding control on your MIDI device.", nullptr));
        save_button->setText(QCoreApplication::translate("MIDIMappingDialog", "&Save\342\200\246", nullptr));
        load_button->setText(QCoreApplication::translate("MIDIMappingDialog", "&Load\342\200\246", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MIDIMappingDialog: public Ui_MIDIMappingDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MIDI_MAPPING_H
