// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: state.proto
// Protobuf C++ Version: 7.36.0

#include "state.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::DeviceSpecProto
        {&::_pbi::kDescriptorMethods, &::descriptor_table_state_2eproto, /* tracker*/ nullptr,},
        // ::BusProto
        {&::_pbi::kDescriptorMethods, &::descriptor_table_state_2eproto, /* tracker*/ nullptr,},
        // ::InputMappingProto
        {&::_pbi::kDescriptorMethods, &::descriptor_table_state_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
class DeviceSpecProto::_Internal {
 public:
  using HasBits = decltype(::std::declval<DeviceSpecProto>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_._has_bits_);
};

constexpr DeviceSpecProto::ParseTableT_ DeviceSpecProto::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_._has_bits_),
      0, // no _extensions_
      7, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967168,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::DeviceSpecProto>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // optional .DeviceSpecProto.InputSourceType type = 1;
      {::_pbi::TcParser::FastEr0S1,
       {8, 4, 2,
        PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.type_)}},
      // optional int32 index = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(DeviceSpecProto, _impl_.index_), 5>(),
       {16, 5, 0,
        PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.index_)}},
      // optional string display_name = 3;
      {::_pbi::TcParser::FastBS1,
       {26, 0, 0,
        PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.display_name_)}},
      // optional string alsa_name = 4;
      {::_pbi::TcParser::FastBS1,
       {34, 1, 0,
        PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.alsa_name_)}},
      // optional string alsa_info = 5;
      {::_pbi::TcParser::FastBS1,
       {42, 2, 0,
        PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.alsa_info_)}},
      // optional int32 num_channels = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(DeviceSpecProto, _impl_.num_channels_), 6>(),
       {48, 6, 0,
        PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.num_channels_)}},
      // optional string address = 7;
      {::_pbi::TcParser::FastBS1,
       {58, 3, 0,
        PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.address_)}},
    }}, {{
      65535, 65535
    }}, {{
      // optional .DeviceSpecProto.InputSourceType type = 1;
      {PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.type_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kEnumRange)},
      // optional int32 index = 2;
      {PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.index_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // optional string display_name = 3;
      {PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.display_name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // optional string alsa_name = 4;
      {PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.alsa_name_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // optional string alsa_info = 5;
      {PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.alsa_info_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // optional int32 num_channels = 6;
      {PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.num_channels_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // optional string address = 7;
      {PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.address_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
    }},
    {{
        {0, 2},
    }},
    {{
    }},
  };
}


inline constexpr DeviceSpecProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : display_name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        alsa_name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        alsa_info_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        address_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        type_{static_cast< ::DeviceSpecProto_InputSourceType >(0)},
        index_{0},
        num_channels_{0} {}

template <typename>
constexpr DeviceSpecProto::DeviceSpecProto(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL DeviceSpecProto::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DeviceSpecProto(arena);
}
constexpr auto DeviceSpecProto::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(DeviceSpecProto), alignof(DeviceSpecProto));
}
constexpr auto DeviceSpecProto::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DeviceSpecProto::MergeImpl,
          Super_::GetNewImpl<DeviceSpecProto>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DeviceSpecProto::SharedDtor,
          Super_::GetClearImpl<DeviceSpecProto>(), &DeviceSpecProto::ByteSizeLong,
              &DeviceSpecProto::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_state_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DeviceSpecProtoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DeviceSpecProtoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DeviceSpecProto_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DeviceSpecProto::InternalGenerateClassData_(
            _default, &DeviceSpecProto_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DeviceSpecProto>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DeviceSpecProtoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DeviceSpecProto _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DeviceSpecProto>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DeviceSpecProtoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DeviceSpecProtoGlobalsTypeInternal DeviceSpecProto_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DeviceSpecProto_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DeviceSpecProto_globals_.GetClassData();
#else
  return DeviceSpecProto_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class BusProto::_Internal {
 public:
  using HasBits = decltype(::std::declval<BusProto>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(BusProto, _impl_._has_bits_);
};

constexpr BusProto::ParseTableT_ BusProto::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(BusProto, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::BusProto>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // optional int32 source_channel_right = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(BusProto, _impl_.source_channel_right_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(BusProto, _impl_.source_channel_right_)}},
      // optional string name = 1;
      {::_pbi::TcParser::FastBS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(BusProto, _impl_.name_)}},
      // optional int32 device_index = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(BusProto, _impl_.device_index_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(BusProto, _impl_.device_index_)}},
      // optional int32 source_channel_left = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(BusProto, _impl_.source_channel_left_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(BusProto, _impl_.source_channel_left_)}},
    }}, {{
      65535, 65535
    }}, {{
      // optional string name = 1;
      {PROTOBUF_FIELD_OFFSET(BusProto, _impl_.name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // optional int32 device_index = 2;
      {PROTOBUF_FIELD_OFFSET(BusProto, _impl_.device_index_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // optional int32 source_channel_left = 3;
      {PROTOBUF_FIELD_OFFSET(BusProto, _impl_.source_channel_left_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // optional int32 source_channel_right = 4;
      {PROTOBUF_FIELD_OFFSET(BusProto, _impl_.source_channel_right_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr BusProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        device_index_{0},
        source_channel_left_{0},
        source_channel_right_{0} {}

template <typename>
constexpr BusProto::BusProto(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL BusProto::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) BusProto(arena);
}
constexpr auto BusProto::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(BusProto), alignof(BusProto));
}
constexpr auto BusProto::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &BusProto::MergeImpl,
          Super_::GetNewImpl<BusProto>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &BusProto::SharedDtor,
          Super_::GetClearImpl<BusProto>(), &BusProto::ByteSizeLong,
              &BusProto::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(BusProto, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_state_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct BusProtoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr BusProtoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 BusProto_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(BusProto::InternalGenerateClassData_(
            _default, &BusProto_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<BusProto>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~BusProtoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) BusProto _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<BusProto>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(BusProtoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST BusProtoGlobalsTypeInternal BusProto_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* BusProto_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return BusProto_globals_.GetClassData();
#else
  return BusProto_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class InputMappingProto::_Internal {
 public:
  using HasBits = decltype(::std::declval<InputMappingProto>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(InputMappingProto, _impl_._has_bits_);
};

constexpr InputMappingProto::ParseTableT_ InputMappingProto::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(InputMappingProto, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::InputMappingProto>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated .BusProto bus = 2;
      {::_pbi::TcParser::FastMtR1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(InputMappingProto, _impl_.bus_)}},
      // repeated .DeviceSpecProto device = 1;
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(InputMappingProto, _impl_.device_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .DeviceSpecProto device = 1;
      {PROTOBUF_FIELD_OFFSET(InputMappingProto, _impl_.device_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // repeated .BusProto bus = 2;
      {PROTOBUF_FIELD_OFFSET(InputMappingProto, _impl_.bus_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::DeviceSpecProto>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::DeviceSpecProto_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::BusProto>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::BusProto_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr InputMappingProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : device_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::InputMappingProto,
            PROTOBUF_FIELD_OFFSET(::InputMappingProto, _impl_.device_)>()
         }
        ,
        bus_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::InputMappingProto,
            PROTOBUF_FIELD_OFFSET(::InputMappingProto, _impl_.bus_)>()
         }
     {}

template <typename>
constexpr InputMappingProto::InputMappingProto(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL InputMappingProto::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) InputMappingProto(arena);
}
constexpr auto InputMappingProto::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(InputMappingProto), alignof(InputMappingProto));
}
constexpr auto InputMappingProto::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &InputMappingProto::MergeImpl,
          Super_::GetNewImpl<InputMappingProto>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &InputMappingProto::SharedDtor,
          Super_::GetClearImpl<InputMappingProto>(), &InputMappingProto::ByteSizeLong,
              &InputMappingProto::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(InputMappingProto, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_state_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct InputMappingProtoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr InputMappingProtoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 InputMappingProto_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(InputMappingProto::InternalGenerateClassData_(
            _default, &InputMappingProto_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<InputMappingProto>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~InputMappingProtoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) InputMappingProto _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<InputMappingProto>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(InputMappingProtoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST InputMappingProtoGlobalsTypeInternal InputMappingProto_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* InputMappingProto_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return InputMappingProto_globals_.GetClassData();
#else
  return InputMappingProto_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_state_2eproto[1];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_state_2eproto = nullptr;
const ::uint32_t
    TableStruct_state_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::DeviceSpecProto, _impl_._has_bits_),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::DeviceSpecProto, _impl_.type_),
        PROTOBUF_FIELD_OFFSET(::DeviceSpecProto, _impl_.index_),
        PROTOBUF_FIELD_OFFSET(::DeviceSpecProto, _impl_.display_name_),
        PROTOBUF_FIELD_OFFSET(::DeviceSpecProto, _impl_.alsa_name_),
        PROTOBUF_FIELD_OFFSET(::DeviceSpecProto, _impl_.alsa_info_),
        PROTOBUF_FIELD_OFFSET(::DeviceSpecProto, _impl_.num_channels_),
        PROTOBUF_FIELD_OFFSET(::DeviceSpecProto, _impl_.address_),
        4,
        5,
        0,
        1,
        2,
        6,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::BusProto, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::BusProto, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::BusProto, _impl_.device_index_),
        PROTOBUF_FIELD_OFFSET(::BusProto, _impl_.source_channel_left_),
        PROTOBUF_FIELD_OFFSET(::BusProto, _impl_.source_channel_right_),
        0,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::InputMappingProto, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::InputMappingProto, _impl_.device_),
        PROTOBUF_FIELD_OFFSET(::InputMappingProto, _impl_.bus_),
        0,
        1,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::DeviceSpecProto)},
        {17, sizeof(::BusProto)},
        {28, sizeof(::InputMappingProto)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::DeviceSpecProto_globals_,
        &::BusProto_globals_,
        &::InputMappingProto_globals_,
};
const char descriptor_table_protodef_state_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\013state.proto\"\365\001\n\017DeviceSpecProto\022.\n\004typ"
    "e\030\001 \001(\0162 .DeviceSpecProto.InputSourceTyp"
    "e\022\r\n\005index\030\002 \001(\005\022\024\n\014display_name\030\003 \001(\t\022\021"
    "\n\talsa_name\030\004 \001(\t\022\021\n\talsa_info\030\005 \001(\t\022\024\n\014"
    "num_channels\030\006 \001(\005\022\017\n\007address\030\007 \001(\t\"@\n\017I"
    "nputSourceType\022\013\n\007SILENCE\020\000\022\020\n\014CAPTURE_C"
    "ARD\020\001\022\016\n\nALSA_INPUT\020\002\"i\n\010BusProto\022\014\n\004nam"
    "e\030\001 \001(\t\022\024\n\014device_index\030\002 \001(\005\022\033\n\023source_"
    "channel_left\030\003 \001(\005\022\034\n\024source_channel_rig"
    "ht\030\004 \001(\005\"M\n\021InputMappingProto\022 \n\006device\030"
    "\001 \003(\0132\020.DeviceSpecProto\022\026\n\003bus\030\002 \003(\0132\t.B"
    "usProto"
};
static ::absl::once_flag descriptor_table_state_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_state_2eproto = {
    false,
    false,
    447,
    descriptor_table_protodef_state_2eproto,
    "state.proto",
    &descriptor_table_state_2eproto_once,
    nullptr,
    0,
    3,
    schemas,
    file_message_globals,
    TableStruct_state_2eproto::offsets,
    file_level_enum_descriptors_state_2eproto,
    file_level_service_descriptors_state_2eproto,
};
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
DeviceSpecProto_InputSourceType_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_state_2eproto);
  return file_level_enum_descriptors_state_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t DeviceSpecProto_InputSourceType_internal_data_[] = {
    196608u, 0u, };
// ===================================================================

DeviceSpecProto::DeviceSpecProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, DeviceSpecProto_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:DeviceSpecProto)
}
PROTOBUF_NDEBUG_INLINE DeviceSpecProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::DeviceSpecProto& from_msg)
      : _has_bits_{from._has_bits_},
        display_name_(arena, from.display_name_),
        alsa_name_(arena, from.alsa_name_),
        alsa_info_(arena, from.alsa_info_),
        address_(arena, from.address_) {}

DeviceSpecProto::DeviceSpecProto(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const DeviceSpecProto& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, DeviceSpecProto_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  DeviceSpecProto* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, type_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, type_),
           offsetof(Impl_, num_channels_) -
               offsetof(Impl_, type_) +
               sizeof(Impl_::num_channels_));

  // @@protoc_insertion_point(copy_constructor:DeviceSpecProto)
}
PROTOBUF_NDEBUG_INLINE DeviceSpecProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : display_name_(arena),
        alsa_name_(arena),
        alsa_info_(arena),
        address_(arena) {}

inline void DeviceSpecProto::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, type_),
           0,
           offsetof(Impl_, num_channels_) -
               offsetof(Impl_, type_) +
               sizeof(Impl_::num_channels_));
}
DeviceSpecProto::~DeviceSpecProto() {
  // @@protoc_insertion_point(destructor:DeviceSpecProto)
  SharedDtor(*this);
}
inline void DeviceSpecProto::SharedDtor(MessageLite& self) {
  DeviceSpecProto& this_ = static_cast<DeviceSpecProto&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.display_name_.Destroy();
  this_._impl_.alsa_name_.Destroy();
  this_._impl_.alsa_info_.Destroy();
  this_._impl_.address_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DeviceSpecProto_class_data_ =
        DeviceSpecProto::InternalGenerateClassData_(DeviceSpecProto_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DeviceSpecProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DeviceSpecProto_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DeviceSpecProto_class_data_.tc_table);
  return DeviceSpecProto_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DeviceSpecProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DeviceSpecProto_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DeviceSpecProto_globals_));
  return DeviceSpecProto_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DeviceSpecProto::ParseTableT_
    DeviceSpecProto::_table_ =
        DeviceSpecProto::InternalGenerateParseTable_(DeviceSpecProto_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void DeviceSpecProto::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:DeviceSpecProto)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.display_name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      this_._impl_.alsa_name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      this_._impl_.alsa_info_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      this_._impl_.address_.ClearNonDefaultToEmpty();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000070U)) {
    ::memset(&this_._impl_.type_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.num_channels_) -
                 reinterpret_cast<char*>(&this_._impl_.type_)) +
                 sizeof(_impl_.num_channels_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL DeviceSpecProto::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const DeviceSpecProto& this_ = static_cast<const DeviceSpecProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL DeviceSpecProto::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const DeviceSpecProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:DeviceSpecProto)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // optional .DeviceSpecProto.InputSourceType type = 1;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteEnumToArray(
        1, this_._internal_type(), target);
  }

  // optional int32 index = 2;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
            stream, this_._internal_index(), target);
  }

  // optional string display_name = 3;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    const ::std::string& _s = this_._internal_display_name();
    target = stream->WriteStringMaybeAliased(3, _s, target);
  }

  // optional string alsa_name = 4;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    const ::std::string& _s = this_._internal_alsa_name();
    target = stream->WriteStringMaybeAliased(4, _s, target);
  }

  // optional string alsa_info = 5;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    const ::std::string& _s = this_._internal_alsa_info();
    target = stream->WriteStringMaybeAliased(5, _s, target);
  }

  // optional int32 num_channels = 6;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<6>(
            stream, this_._internal_num_channels(), target);
  }

  // optional string address = 7;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    const ::std::string& _s = this_._internal_address();
    target = stream->WriteStringMaybeAliased(7, _s, target);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:DeviceSpecProto)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t DeviceSpecProto::ByteSizeLong(const MessageLite& base) {
  const DeviceSpecProto& this_ = static_cast<const DeviceSpecProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t DeviceSpecProto::ByteSizeLong() const {
  const DeviceSpecProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:DeviceSpecProto)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    // optional string display_name = 3;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                      this_._internal_display_name());
    }
    // optional string alsa_name = 4;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                      this_._internal_alsa_name());
    }
    // optional string alsa_info = 5;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                      this_._internal_alsa_info());
    }
    // optional string address = 7;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                      this_._internal_address());
    }
    // optional .DeviceSpecProto.InputSourceType type = 1;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      total_size += 1 +
                    ::_pbi::WireFormatLite::EnumSize(this_._internal_type());
    }
    // optional int32 index = 2;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
          this_._internal_index());
    }
    // optional int32 num_channels = 6;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
          this_._internal_num_channels());
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void DeviceSpecProto::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<DeviceSpecProto*>(&to_msg);
  auto& from = static_cast<const DeviceSpecProto&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:DeviceSpecProto)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_set_display_name(from._internal_display_name());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_internal_set_alsa_name(from._internal_alsa_name());
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_internal_set_alsa_info(from._internal_alsa_info());
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _this->_internal_set_address(from._internal_address());
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _this->_impl_.type_ = from._impl_.type_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      _this->_impl_.index_ = from._impl_.index_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      _this->_impl_.num_channels_ = from._impl_.num_channels_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void DeviceSpecProto::CopyFrom(const DeviceSpecProto& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:DeviceSpecProto)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void DeviceSpecProto::InternalSwap(DeviceSpecProto* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.display_name_, &other->_impl_.display_name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.alsa_name_, &other->_impl_.alsa_name_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.alsa_info_, &other->_impl_.alsa_info_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.address_, &other->_impl_.address_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.num_channels_)
      + sizeof(DeviceSpecProto::_impl_.num_channels_)
      - PROTOBUF_FIELD_OFFSET(DeviceSpecProto, _impl_.type_)>(
          reinterpret_cast<char*>(&_impl_.type_),
          reinterpret_cast<char*>(&other->_impl_.type_));
}

::google::protobuf::Metadata DeviceSpecProto::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

BusProto::BusProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, BusProto_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:BusProto)
}
PROTOBUF_NDEBUG_INLINE BusProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::BusProto& from_msg)
      : _has_bits_{from._has_bits_},
        name_(arena, from.name_) {}

BusProto::BusProto(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const BusProto& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, BusProto_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  BusProto* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, device_index_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, device_index_),
           offsetof(Impl_, source_channel_right_) -
               offsetof(Impl_, device_index_) +
               sizeof(Impl_::source_channel_right_));

  // @@protoc_insertion_point(copy_constructor:BusProto)
}
PROTOBUF_NDEBUG_INLINE BusProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : name_(arena) {}

inline void BusProto::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, device_index_),
           0,
           offsetof(Impl_, source_channel_right_) -
               offsetof(Impl_, device_index_) +
               sizeof(Impl_::source_channel_right_));
}
BusProto::~BusProto() {
  // @@protoc_insertion_point(destructor:BusProto)
  SharedDtor(*this);
}
inline void BusProto::SharedDtor(MessageLite& self) {
  BusProto& this_ = static_cast<BusProto&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.name_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull BusProto_class_data_ =
        BusProto::InternalGenerateClassData_(BusProto_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
BusProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&BusProto_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(BusProto_class_data_.tc_table);
  return BusProto_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
BusProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&BusProto_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&BusProto_globals_));
  return BusProto_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const BusProto::ParseTableT_
    BusProto::_table_ =
        BusProto::InternalGenerateParseTable_(BusProto_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void BusProto::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:BusProto)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    this_._impl_.name_.ClearNonDefaultToEmpty();
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000000eU)) {
    ::memset(&this_._impl_.device_index_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.source_channel_right_) -
                 reinterpret_cast<char*>(&this_._impl_.device_index_)) +
                 sizeof(_impl_.source_channel_right_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL BusProto::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const BusProto& this_ = static_cast<const BusProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL BusProto::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const BusProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:BusProto)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // optional string name = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    const ::std::string& _s = this_._internal_name();
    target = stream->WriteStringMaybeAliased(1, _s, target);
  }

  // optional int32 device_index = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
            stream, this_._internal_device_index(), target);
  }

  // optional int32 source_channel_left = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<3>(
            stream, this_._internal_source_channel_left(), target);
  }

  // optional int32 source_channel_right = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
            stream, this_._internal_source_channel_right(), target);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:BusProto)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t BusProto::ByteSizeLong(const MessageLite& base) {
  const BusProto& this_ = static_cast<const BusProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t BusProto::ByteSizeLong() const {
  const BusProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:BusProto)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // optional string name = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                      this_._internal_name());
    }
    // optional int32 device_index = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
          this_._internal_device_index());
    }
    // optional int32 source_channel_left = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
          this_._internal_source_channel_left());
    }
    // optional int32 source_channel_right = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
          this_._internal_source_channel_right());
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void BusProto::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<BusProto*>(&to_msg);
  auto& from = static_cast<const BusProto&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:BusProto)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_set_name(from._internal_name());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_impl_.device_index_ = from._impl_.device_index_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_impl_.source_channel_left_ = from._impl_.source_channel_left_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _this->_impl_.source_channel_right_ = from._impl_.source_channel_right_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void BusProto::CopyFrom(const BusProto& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:BusProto)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void BusProto::InternalSwap(BusProto* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(BusProto, _impl_.source_channel_right_)
      + sizeof(BusProto::_impl_.source_channel_right_)
      - PROTOBUF_FIELD_OFFSET(BusProto, _impl_.device_index_)>(
          reinterpret_cast<char*>(&_impl_.device_index_),
          reinterpret_cast<char*>(&other->_impl_.device_index_));
}

::google::protobuf::Metadata BusProto::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

InputMappingProto::InputMappingProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, InputMappingProto_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:InputMappingProto)
}
PROTOBUF_NDEBUG_INLINE InputMappingProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::InputMappingProto& from_msg)
      : _has_bits_{from._has_bits_},
        device_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::InputMappingProto,
              PROTOBUF_FIELD_OFFSET(::InputMappingProto, _impl_.device_)>()
          , arena, from.device_
        }
        ,
        bus_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::InputMappingProto,
              PROTOBUF_FIELD_OFFSET(::InputMappingProto, _impl_.bus_)>()
          , arena, from.bus_
        }
     {}

InputMappingProto::InputMappingProto(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const InputMappingProto& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, InputMappingProto_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  InputMappingProto* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:InputMappingProto)
}
PROTOBUF_NDEBUG_INLINE InputMappingProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : device_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::InputMappingProto,
            PROTOBUF_FIELD_OFFSET(::InputMappingProto, _impl_.device_)>()
         }
        ,
        bus_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::InputMappingProto,
            PROTOBUF_FIELD_OFFSET(::InputMappingProto, _impl_.bus_)>()
         }
     {}

inline void InputMappingProto::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
InputMappingProto::~InputMappingProto() {
  // @@protoc_insertion_point(destructor:InputMappingProto)
  SharedDtor(*this);
}
inline void InputMappingProto::SharedDtor(MessageLite& self) {
  InputMappingProto& this_ = static_cast<InputMappingProto&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull InputMappingProto_class_data_ =
        InputMappingProto::InternalGenerateClassData_(InputMappingProto_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
InputMappingProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&InputMappingProto_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(InputMappingProto_class_data_.tc_table);
  return InputMappingProto_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
InputMappingProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&InputMappingProto_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&InputMappingProto_globals_));
  return InputMappingProto_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const InputMappingProto::ParseTableT_
    InputMappingProto::_table_ =
        InputMappingProto::InternalGenerateParseTable_(InputMappingProto_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void InputMappingProto::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:InputMappingProto)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.device_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.bus_.Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL InputMappingProto::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const InputMappingProto& this_ = static_cast<const InputMappingProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL InputMappingProto::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const InputMappingProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:InputMappingProto)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .DeviceSpecProto device = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_device_size());
         i < n; i++) {
      const auto& repfield = this_._internal_device().Get(i);
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          1, repfield, repfield.GetCachedSize(), target,
          stream);
    }
  }

  // repeated .BusProto bus = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_bus_size());
         i < n; i++) {
      const auto& repfield = this_._internal_bus().Get(i);
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          2, repfield, repfield.GetCachedSize(), target,
          stream);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:InputMappingProto)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t InputMappingProto::ByteSizeLong(const MessageLite& base) {
  const InputMappingProto& this_ = static_cast<const InputMappingProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t InputMappingProto::ByteSizeLong() const {
  const InputMappingProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:InputMappingProto)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated .DeviceSpecProto device = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_device_size();
      for (const auto& msg : this_._internal_device()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // repeated .BusProto bus = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1UL * this_._internal_bus_size();
      for (const auto& msg : this_._internal_bus()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void InputMappingProto::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<InputMappingProto*>(&to_msg);
  auto& from = static_cast<const InputMappingProto&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:InputMappingProto)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_device()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_device());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_internal_mutable_bus()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_bus());
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void InputMappingProto::CopyFrom(const InputMappingProto& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:InputMappingProto)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void InputMappingProto::InternalSwap(InputMappingProto* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.device_.InternalSwap(&other->_impl_.device_);
  _impl_.bus_.InternalSwap(&other->_impl_.bus_);
}

::google::protobuf::Metadata InputMappingProto::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_state_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
