/********************************************************************************
** Form generated from reading UI file 'audio_miniview.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_AUDIO_MINIVIEW_H
#define UI_AUDIO_MINIVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "clickable_label.h"
#include "ellipsis_label.h"
#include "nonlinear_fader.h"
#include "vumeter.h"

QT_BEGIN_NAMESPACE

class Ui_AudioMiniView
{
public:
    QVBoxLayout *main_vertical_layout;
    QFrame *frame;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_2;
    EllipsisLabel *bus_desc_label;
    QHBoxLayout *bottom_layout;
    QVBoxLayout *vu_meter_layout;
    QHBoxLayout *vu_meter_centerer;
    VUMeter *peak_meter;
    ClickableLabel *peak_display_label;
    QVBoxLayout *fader_layout;
    QHBoxLayout *fader_centerer;
    NonLinearFader *fader;
    QLabel *fader_label;
    QFrame *line;

    void setupUi(QWidget *AudioMiniView)
    {
        if (AudioMiniView->objectName().isEmpty())
            AudioMiniView->setObjectName(QString::fromUtf8("AudioMiniView"));
        AudioMiniView->resize(139, 300);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(AudioMiniView->sizePolicy().hasHeightForWidth());
        AudioMiniView->setSizePolicy(sizePolicy);
        AudioMiniView->setMinimumSize(QSize(139, 0));
        AudioMiniView->setMaximumSize(QSize(139, 16777215));
        main_vertical_layout = new QVBoxLayout(AudioMiniView);
        main_vertical_layout->setObjectName(QString::fromUtf8("main_vertical_layout"));
        main_vertical_layout->setContentsMargins(0, 0, 0, 0);
        frame = new QFrame(AudioMiniView);
        frame->setObjectName(QString::fromUtf8("frame"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(1);
        sizePolicy1.setHeightForWidth(frame->sizePolicy().hasHeightForWidth());
        frame->setSizePolicy(sizePolicy1);
        frame->setAutoFillBackground(true);
        frame->setFrameShape(QFrame::Panel);
        frame->setFrameShadow(QFrame::Plain);
        frame->setLineWidth(0);
        verticalLayout = new QVBoxLayout(frame);
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        bus_desc_label = new EllipsisLabel(frame);
        bus_desc_label->setObjectName(QString::fromUtf8("bus_desc_label"));
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(bus_desc_label->sizePolicy().hasHeightForWidth());
        bus_desc_label->setSizePolicy(sizePolicy2);
        bus_desc_label->setMinimumSize(QSize(0, 0));
        bus_desc_label->setAlignment(Qt::AlignCenter);

        verticalLayout_2->addWidget(bus_desc_label);

        bottom_layout = new QHBoxLayout();
        bottom_layout->setSpacing(0);
        bottom_layout->setObjectName(QString::fromUtf8("bottom_layout"));
        vu_meter_layout = new QVBoxLayout();
        vu_meter_layout->setObjectName(QString::fromUtf8("vu_meter_layout"));
        vu_meter_layout->setContentsMargins(0, -1, -1, 4);
        vu_meter_centerer = new QHBoxLayout();
        vu_meter_centerer->setSpacing(0);
        vu_meter_centerer->setObjectName(QString::fromUtf8("vu_meter_centerer"));
        vu_meter_centerer->setContentsMargins(-1, -1, -1, 0);
        peak_meter = new VUMeter(frame);
        peak_meter->setObjectName(QString::fromUtf8("peak_meter"));
        QSizePolicy sizePolicy3(QSizePolicy::Fixed, QSizePolicy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(1);
        sizePolicy3.setHeightForWidth(peak_meter->sizePolicy().hasHeightForWidth());
        peak_meter->setSizePolicy(sizePolicy3);
        peak_meter->setMinimumSize(QSize(16, 0));
        peak_meter->setSizeIncrement(QSize(1, 0));
        peak_meter->setBaseSize(QSize(0, 0));
        QPalette palette;
        QBrush brush(QColor(255, 255, 255, 255));
        brush.setStyle(Qt::SolidPattern);
        palette.setBrush(QPalette::Active, QPalette::Base, brush);
        QBrush brush1(QColor(5, 239, 111, 255));
        brush1.setStyle(Qt::SolidPattern);
        palette.setBrush(QPalette::Active, QPalette::Window, brush1);
        palette.setBrush(QPalette::Inactive, QPalette::Base, brush);
        palette.setBrush(QPalette::Inactive, QPalette::Window, brush1);
        palette.setBrush(QPalette::Disabled, QPalette::Base, brush1);
        palette.setBrush(QPalette::Disabled, QPalette::Window, brush1);
        peak_meter->setPalette(palette);
        peak_meter->setAutoFillBackground(true);

        vu_meter_centerer->addWidget(peak_meter);


        vu_meter_layout->addLayout(vu_meter_centerer);

        peak_display_label = new ClickableLabel(frame);
        peak_display_label->setObjectName(QString::fromUtf8("peak_display_label"));
        peak_display_label->setMinimumSize(QSize(30, 0));
        peak_display_label->setAlignment(Qt::AlignCenter);

        vu_meter_layout->addWidget(peak_display_label);

        vu_meter_layout->setStretch(0, 1);

        bottom_layout->addLayout(vu_meter_layout);

        fader_layout = new QVBoxLayout();
        fader_layout->setObjectName(QString::fromUtf8("fader_layout"));
        fader_layout->setContentsMargins(-1, -1, -1, 4);
        fader_centerer = new QHBoxLayout();
        fader_centerer->setSpacing(0);
        fader_centerer->setObjectName(QString::fromUtf8("fader_centerer"));
        fader = new NonLinearFader(frame);
        fader->setObjectName(QString::fromUtf8("fader"));
        fader->setMinimum(0);
        fader->setMaximum(1000);
        fader->setSingleStep(10);
        fader->setPageStep(100);
        fader->setOrientation(Qt::Vertical);
        fader->setTickPosition(QSlider::NoTicks);
        fader->setTickInterval(30);

        fader_centerer->addWidget(fader);


        fader_layout->addLayout(fader_centerer);

        fader_label = new QLabel(frame);
        fader_label->setObjectName(QString::fromUtf8("fader_label"));
        QSizePolicy sizePolicy4(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(fader_label->sizePolicy().hasHeightForWidth());
        fader_label->setSizePolicy(sizePolicy4);
        fader_label->setMinimumSize(QSize(30, 0));
        fader_label->setLineWidth(0);
        fader_label->setAlignment(Qt::AlignCenter);

        fader_layout->addWidget(fader_label);


        bottom_layout->addLayout(fader_layout);

        bottom_layout->setStretch(0, 1);
        bottom_layout->setStretch(1, 1);

        verticalLayout_2->addLayout(bottom_layout);


        horizontalLayout->addLayout(verticalLayout_2);

        line = new QFrame(frame);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::VLine);
        line->setFrameShadow(QFrame::Sunken);

        horizontalLayout->addWidget(line);


        verticalLayout->addLayout(horizontalLayout);


        main_vertical_layout->addWidget(frame);


        retranslateUi(AudioMiniView);

        QMetaObject::connectSlotsByName(AudioMiniView);
    } // setupUi

    void retranslateUi(QWidget *AudioMiniView)
    {
        AudioMiniView->setWindowTitle(QCoreApplication::translate("AudioMiniView", "Form", nullptr));
        bus_desc_label->setText(QCoreApplication::translate("AudioMiniView", "Channel description", nullptr));
        peak_display_label->setText(QCoreApplication::translate("AudioMiniView", "-0.0", nullptr));
        fader_label->setText(QCoreApplication::translate("AudioMiniView", "+0.0 dB", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AudioMiniView: public Ui_AudioMiniView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_AUDIO_MINIVIEW_H
