// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: frame.proto
// Protobuf C++ Version: 7.36.0

#include "frame.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::FrameHeaderProto
        {&::_pbi::kDescriptorMethods, &::descriptor_table_frame_2eproto, /* tracker*/ nullptr,},
        // ::StreamContentsProto
        {&::_pbi::kDescriptorMethods, &::descriptor_table_frame_2eproto, /* tracker*/ nullptr,},
        // ::FileContentsProto
        {&::_pbi::kDescriptorMethods, &::descriptor_table_frame_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
class StreamContentsProto::_Internal {
 public:
  using HasBits = decltype(::std::declval<StreamContentsProto>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(StreamContentsProto, _impl_._has_bits_);
};

constexpr StreamContentsProto::ParseTableT_ StreamContentsProto::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(StreamContentsProto, _impl_._has_bits_),
      0, // no _extensions_
      5, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967264,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::StreamContentsProto>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // int32 stream_idx = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(StreamContentsProto, _impl_.stream_idx_), 4>(),
       {8, 4, 0,
        PROTOBUF_FIELD_OFFSET(StreamContentsProto, _impl_.stream_idx_)}},
      // repeated int64 pts = 2 [packed = true];
      {::_pbi::TcParser::FastV64P1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(StreamContentsProto, _impl_.pts_)}},
      // repeated int64 file_size = 3 [packed = true];
      {::_pbi::TcParser::FastV64P1,
       {26, 1, 0,
        PROTOBUF_FIELD_OFFSET(StreamContentsProto, _impl_.file_size_)}},
      // repeated int64 offset = 4 [packed = true];
      {::_pbi::TcParser::FastV64P1,
       {34, 2, 0,
        PROTOBUF_FIELD_OFFSET(StreamContentsProto, _impl_.offset_)}},
      // repeated int32 audio_size = 5 [packed = true];
      {::_pbi::TcParser::FastV32P1,
       {42, 3, 0,
        PROTOBUF_FIELD_OFFSET(StreamContentsProto, _impl_.audio_size_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // int32 stream_idx = 1;
      {PROTOBUF_FIELD_OFFSET(StreamContentsProto, _impl_.stream_idx_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // repeated int64 pts = 2 [packed = true];
      {PROTOBUF_FIELD_OFFSET(StreamContentsProto, _impl_.pts_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedInt64)},
      // repeated int64 file_size = 3 [packed = true];
      {PROTOBUF_FIELD_OFFSET(StreamContentsProto, _impl_.file_size_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedInt64)},
      // repeated int64 offset = 4 [packed = true];
      {PROTOBUF_FIELD_OFFSET(StreamContentsProto, _impl_.offset_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedInt64)},
      // repeated int32 audio_size = 5 [packed = true];
      {PROTOBUF_FIELD_OFFSET(StreamContentsProto, _impl_.audio_size_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedInt32)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr StreamContentsProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : pts_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::StreamContentsProto,
            PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.pts_)>()
         }
        ,
        file_size_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::StreamContentsProto,
            PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.file_size_)>()
         }
        ,
        offset_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::StreamContentsProto,
            PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.offset_)>()
         }
        ,
        audio_size_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::StreamContentsProto,
            PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.audio_size_)>()
         }
        ,
        stream_idx_{0} {}

template <typename>
constexpr StreamContentsProto::StreamContentsProto(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL StreamContentsProto::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) StreamContentsProto(arena);
}
constexpr auto StreamContentsProto::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(StreamContentsProto), alignof(StreamContentsProto));
}
constexpr auto StreamContentsProto::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &StreamContentsProto::MergeImpl,
          Super_::GetNewImpl<StreamContentsProto>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &StreamContentsProto::SharedDtor,
          Super_::GetClearImpl<StreamContentsProto>(), &StreamContentsProto::ByteSizeLong,
              &StreamContentsProto::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(StreamContentsProto, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_frame_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct StreamContentsProtoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr StreamContentsProtoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 StreamContentsProto_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(StreamContentsProto::InternalGenerateClassData_(
            _default, &StreamContentsProto_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<StreamContentsProto>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~StreamContentsProtoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) StreamContentsProto _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<StreamContentsProto>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(StreamContentsProtoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST StreamContentsProtoGlobalsTypeInternal StreamContentsProto_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* StreamContentsProto_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return StreamContentsProto_globals_.GetClassData();
#else
  return StreamContentsProto_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class FrameHeaderProto::_Internal {
 public:
  using HasBits = decltype(::std::declval<FrameHeaderProto>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(FrameHeaderProto, _impl_._has_bits_);
};

constexpr FrameHeaderProto::ParseTableT_ FrameHeaderProto::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(FrameHeaderProto, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::FrameHeaderProto>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int32 audio_size = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(FrameHeaderProto, _impl_.audio_size_), 2>(),
       {32, 2, 0,
        PROTOBUF_FIELD_OFFSET(FrameHeaderProto, _impl_.audio_size_)}},
      // int32 stream_idx = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(FrameHeaderProto, _impl_.stream_idx_), 1>(),
       {8, 1, 0,
        PROTOBUF_FIELD_OFFSET(FrameHeaderProto, _impl_.stream_idx_)}},
      // int64 pts = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(FrameHeaderProto, _impl_.pts_), 0>(),
       {16, 0, 0,
        PROTOBUF_FIELD_OFFSET(FrameHeaderProto, _impl_.pts_)}},
      // int64 file_size = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(FrameHeaderProto, _impl_.file_size_), 3>(),
       {24, 3, 0,
        PROTOBUF_FIELD_OFFSET(FrameHeaderProto, _impl_.file_size_)}},
    }}, {{
      65535, 65535
    }}, {{
      // int32 stream_idx = 1;
      {PROTOBUF_FIELD_OFFSET(FrameHeaderProto, _impl_.stream_idx_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int64 pts = 2;
      {PROTOBUF_FIELD_OFFSET(FrameHeaderProto, _impl_.pts_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // int64 file_size = 3;
      {PROTOBUF_FIELD_OFFSET(FrameHeaderProto, _impl_.file_size_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // int32 audio_size = 4;
      {PROTOBUF_FIELD_OFFSET(FrameHeaderProto, _impl_.audio_size_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr FrameHeaderProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : pts_{::int64_t{0}},
        stream_idx_{0},
        audio_size_{0},
        file_size_{::int64_t{0}} {}

template <typename>
constexpr FrameHeaderProto::FrameHeaderProto(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL FrameHeaderProto::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) FrameHeaderProto(arena);
}
constexpr auto FrameHeaderProto::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(FrameHeaderProto), alignof(FrameHeaderProto));
}
constexpr auto FrameHeaderProto::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &FrameHeaderProto::MergeImpl,
          Super_::GetNewImpl<FrameHeaderProto>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &FrameHeaderProto::SharedDtor,
          Super_::GetClearImpl<FrameHeaderProto>(), &FrameHeaderProto::ByteSizeLong,
              &FrameHeaderProto::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(FrameHeaderProto, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_frame_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct FrameHeaderProtoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr FrameHeaderProtoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 FrameHeaderProto_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(FrameHeaderProto::InternalGenerateClassData_(
            _default, &FrameHeaderProto_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<FrameHeaderProto>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~FrameHeaderProtoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) FrameHeaderProto _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<FrameHeaderProto>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(FrameHeaderProtoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST FrameHeaderProtoGlobalsTypeInternal FrameHeaderProto_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* FrameHeaderProto_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return FrameHeaderProto_globals_.GetClassData();
#else
  return FrameHeaderProto_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class FileContentsProto::_Internal {
 public:
  using HasBits = decltype(::std::declval<FileContentsProto>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(FileContentsProto, _impl_._has_bits_);
};

constexpr FileContentsProto::ParseTableT_ FileContentsProto::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(FileContentsProto, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::FileContentsProto>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated .StreamContentsProto stream = 1;
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(FileContentsProto, _impl_.stream_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .StreamContentsProto stream = 1;
      {PROTOBUF_FIELD_OFFSET(FileContentsProto, _impl_.stream_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::StreamContentsProto>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::StreamContentsProto_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr FileContentsProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : stream_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::FileContentsProto,
            PROTOBUF_FIELD_OFFSET(::FileContentsProto, _impl_.stream_)>()
         }
     {}

template <typename>
constexpr FileContentsProto::FileContentsProto(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL FileContentsProto::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) FileContentsProto(arena);
}
constexpr auto FileContentsProto::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(FileContentsProto), alignof(FileContentsProto));
}
constexpr auto FileContentsProto::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &FileContentsProto::MergeImpl,
          Super_::GetNewImpl<FileContentsProto>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &FileContentsProto::SharedDtor,
          Super_::GetClearImpl<FileContentsProto>(), &FileContentsProto::ByteSizeLong,
              &FileContentsProto::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(FileContentsProto, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_frame_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct FileContentsProtoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr FileContentsProtoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 FileContentsProto_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(FileContentsProto::InternalGenerateClassData_(
            _default, &FileContentsProto_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<FileContentsProto>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~FileContentsProtoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) FileContentsProto _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<FileContentsProto>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(FileContentsProtoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST FileContentsProtoGlobalsTypeInternal FileContentsProto_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* FileContentsProto_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return FileContentsProto_globals_.GetClassData();
#else
  return FileContentsProto_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
static constexpr const ::_pb::EnumDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_enum_descriptors_frame_2eproto = nullptr;
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_frame_2eproto = nullptr;
const ::uint32_t
    TableStruct_frame_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::FrameHeaderProto, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::FrameHeaderProto, _impl_.stream_idx_),
        PROTOBUF_FIELD_OFFSET(::FrameHeaderProto, _impl_.pts_),
        PROTOBUF_FIELD_OFFSET(::FrameHeaderProto, _impl_.file_size_),
        PROTOBUF_FIELD_OFFSET(::FrameHeaderProto, _impl_.audio_size_),
        1,
        0,
        3,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_._has_bits_),
        8, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.stream_idx_),
        PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.pts_),
        PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.file_size_),
        PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.offset_),
        PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.audio_size_),
        4,
        0,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::FileContentsProto, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::FileContentsProto, _impl_.stream_),
        0,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::FrameHeaderProto)},
        {11, sizeof(::StreamContentsProto)},
        {24, sizeof(::FileContentsProto)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::FrameHeaderProto_globals_,
        &::StreamContentsProto_globals_,
        &::FileContentsProto_globals_,
};
const char descriptor_table_protodef_frame_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\013frame.proto\"Z\n\020FrameHeaderProto\022\022\n\nstr"
    "eam_idx\030\001 \001(\005\022\013\n\003pts\030\002 \001(\003\022\021\n\tfile_size\030"
    "\003 \001(\003\022\022\n\naudio_size\030\004 \001(\005\"}\n\023StreamConte"
    "ntsProto\022\022\n\nstream_idx\030\001 \001(\005\022\017\n\003pts\030\002 \003("
    "\003B\002\020\001\022\025\n\tfile_size\030\003 \003(\003B\002\020\001\022\022\n\006offset\030\004"
    " \003(\003B\002\020\001\022\026\n\naudio_size\030\005 \003(\005B\002\020\001\"9\n\021File"
    "ContentsProto\022$\n\006stream\030\001 \003(\0132\024.StreamCo"
    "ntentsProtob\006proto3"
};
static ::absl::once_flag descriptor_table_frame_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_frame_2eproto = {
    false,
    false,
    299,
    descriptor_table_protodef_frame_2eproto,
    "frame.proto",
    &descriptor_table_frame_2eproto_once,
    nullptr,
    0,
    3,
    schemas,
    file_message_globals,
    TableStruct_frame_2eproto::offsets,
    file_level_enum_descriptors_frame_2eproto,
    file_level_service_descriptors_frame_2eproto,
};
// ===================================================================

FrameHeaderProto::FrameHeaderProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, FrameHeaderProto_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:FrameHeaderProto)
}
FrameHeaderProto::FrameHeaderProto(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const FrameHeaderProto& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, FrameHeaderProto_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE FrameHeaderProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void FrameHeaderProto::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, pts_),
           0,
           offsetof(Impl_, file_size_) -
               offsetof(Impl_, pts_) +
               sizeof(Impl_::file_size_));
}
FrameHeaderProto::~FrameHeaderProto() {
  // @@protoc_insertion_point(destructor:FrameHeaderProto)
  SharedDtor(*this);
}
inline void FrameHeaderProto::SharedDtor(MessageLite& self) {
  FrameHeaderProto& this_ = static_cast<FrameHeaderProto&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull FrameHeaderProto_class_data_ =
        FrameHeaderProto::InternalGenerateClassData_(FrameHeaderProto_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
FrameHeaderProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&FrameHeaderProto_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(FrameHeaderProto_class_data_.tc_table);
  return FrameHeaderProto_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
FrameHeaderProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&FrameHeaderProto_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&FrameHeaderProto_globals_));
  return FrameHeaderProto_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const FrameHeaderProto::ParseTableT_
    FrameHeaderProto::_table_ =
        FrameHeaderProto::InternalGenerateParseTable_(FrameHeaderProto_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void FrameHeaderProto::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:FrameHeaderProto)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    ::memset(&this_._impl_.pts_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.file_size_) -
                 reinterpret_cast<char*>(&this_._impl_.pts_)) +
                 sizeof(_impl_.file_size_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL FrameHeaderProto::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const FrameHeaderProto& this_ = static_cast<const FrameHeaderProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL FrameHeaderProto::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const FrameHeaderProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:FrameHeaderProto)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int32 stream_idx = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_stream_idx() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<1>(
              stream, this_._internal_stream_idx(), target);
    }
  }

  // int64 pts = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_pts() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<2>(
              stream, this_._internal_pts(), target);
    }
  }

  // int64 file_size = 3;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_file_size() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<3>(
              stream, this_._internal_file_size(), target);
    }
  }

  // int32 audio_size = 4;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_audio_size() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_audio_size(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:FrameHeaderProto)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t FrameHeaderProto::ByteSizeLong(const MessageLite& base) {
  const FrameHeaderProto& this_ = static_cast<const FrameHeaderProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t FrameHeaderProto::ByteSizeLong() const {
  const FrameHeaderProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:FrameHeaderProto)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // int64 pts = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_pts() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_pts());
      }
    }
    // int32 stream_idx = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_stream_idx() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_stream_idx());
      }
    }
    // int32 audio_size = 4;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_audio_size() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_audio_size());
      }
    }
    // int64 file_size = 3;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_file_size() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_file_size());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void FrameHeaderProto::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<FrameHeaderProto*>(&to_msg);
  auto& from = static_cast<const FrameHeaderProto&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:FrameHeaderProto)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_pts() != 0) {
        _this->_impl_.pts_ = from._impl_.pts_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_stream_idx() != 0) {
        _this->_impl_.stream_idx_ = from._impl_.stream_idx_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_audio_size() != 0) {
        _this->_impl_.audio_size_ = from._impl_.audio_size_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_file_size() != 0) {
        _this->_impl_.file_size_ = from._impl_.file_size_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void FrameHeaderProto::CopyFrom(const FrameHeaderProto& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:FrameHeaderProto)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void FrameHeaderProto::InternalSwap(FrameHeaderProto* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(FrameHeaderProto, _impl_.file_size_)
      + sizeof(FrameHeaderProto::_impl_.file_size_)
      - PROTOBUF_FIELD_OFFSET(FrameHeaderProto, _impl_.pts_)>(
          reinterpret_cast<char*>(&_impl_.pts_),
          reinterpret_cast<char*>(&other->_impl_.pts_));
}

::google::protobuf::Metadata FrameHeaderProto::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

StreamContentsProto::StreamContentsProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, StreamContentsProto_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:StreamContentsProto)
}
PROTOBUF_NDEBUG_INLINE StreamContentsProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::StreamContentsProto& from_msg)
      : _has_bits_{from._has_bits_},
        pts_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::StreamContentsProto,
              PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.pts_)>()
          , arena, from.pts_
        }
        ,
        file_size_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::StreamContentsProto,
              PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.file_size_)>()
          , arena, from.file_size_
        }
        ,
        offset_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::StreamContentsProto,
              PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.offset_)>()
          , arena, from.offset_
        }
        ,
        audio_size_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::StreamContentsProto,
              PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.audio_size_)>()
          , arena, from.audio_size_
        }
     {}

StreamContentsProto::StreamContentsProto(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const StreamContentsProto& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, StreamContentsProto_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  StreamContentsProto* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.stream_idx_ = from._impl_.stream_idx_;

  // @@protoc_insertion_point(copy_constructor:StreamContentsProto)
}
PROTOBUF_NDEBUG_INLINE StreamContentsProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : pts_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::StreamContentsProto,
            PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.pts_)>()
         }
        ,
        file_size_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::StreamContentsProto,
            PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.file_size_)>()
         }
        ,
        offset_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::StreamContentsProto,
            PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.offset_)>()
         }
        ,
        audio_size_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::StreamContentsProto,
            PROTOBUF_FIELD_OFFSET(::StreamContentsProto, _impl_.audio_size_)>()
         }
     {}

inline void StreamContentsProto::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.stream_idx_ = {};
}
StreamContentsProto::~StreamContentsProto() {
  // @@protoc_insertion_point(destructor:StreamContentsProto)
  SharedDtor(*this);
}
inline void StreamContentsProto::SharedDtor(MessageLite& self) {
  StreamContentsProto& this_ = static_cast<StreamContentsProto&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull StreamContentsProto_class_data_ =
        StreamContentsProto::InternalGenerateClassData_(StreamContentsProto_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamContentsProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamContentsProto_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(StreamContentsProto_class_data_.tc_table);
  return StreamContentsProto_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StreamContentsProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StreamContentsProto_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&StreamContentsProto_globals_));
  return StreamContentsProto_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const StreamContentsProto::ParseTableT_
    StreamContentsProto::_table_ =
        StreamContentsProto::InternalGenerateParseTable_(StreamContentsProto_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void StreamContentsProto::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:StreamContentsProto)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.pts_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.file_size_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.offset_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _impl_.audio_size_.Clear();
    }
  }
  this_._impl_.stream_idx_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL StreamContentsProto::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const StreamContentsProto& this_ = static_cast<const StreamContentsProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL StreamContentsProto::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const StreamContentsProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:StreamContentsProto)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int32 stream_idx = 1;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_stream_idx() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<1>(
              stream, this_._internal_stream_idx(), target);
    }
  }

  // repeated int64 pts = 2 [packed = true];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    {
      int byte_size = this_._impl_._pts_cached_byte_size_.Get();
      if (byte_size > 0) {
        target = stream->WriteInt64Packed(
            2, this_._internal_pts(), byte_size, target);
      }
    }
  }

  // repeated int64 file_size = 3 [packed = true];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    {
      int byte_size = this_._impl_._file_size_cached_byte_size_.Get();
      if (byte_size > 0) {
        target = stream->WriteInt64Packed(
            3, this_._internal_file_size(), byte_size, target);
      }
    }
  }

  // repeated int64 offset = 4 [packed = true];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    {
      int byte_size = this_._impl_._offset_cached_byte_size_.Get();
      if (byte_size > 0) {
        target = stream->WriteInt64Packed(
            4, this_._internal_offset(), byte_size, target);
      }
    }
  }

  // repeated int32 audio_size = 5 [packed = true];
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    {
      int byte_size = this_._impl_._audio_size_cached_byte_size_.Get();
      if (byte_size > 0) {
        target = stream->WriteInt32Packed(
            5, this_._internal_audio_size(), byte_size, target);
      }
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:StreamContentsProto)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t StreamContentsProto::ByteSizeLong(const MessageLite& base) {
  const StreamContentsProto& this_ = static_cast<const StreamContentsProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t StreamContentsProto::ByteSizeLong() const {
  const StreamContentsProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:StreamContentsProto)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    // repeated int64 pts = 2 [packed = true];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          ::_pbi::WireFormatLite::Int64SizeWithPackedTagSize(
              this_._internal_pts(), 1,
              this_._impl_._pts_cached_byte_size_);
    }
    // repeated int64 file_size = 3 [packed = true];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size +=
          ::_pbi::WireFormatLite::Int64SizeWithPackedTagSize(
              this_._internal_file_size(), 1,
              this_._impl_._file_size_cached_byte_size_);
    }
    // repeated int64 offset = 4 [packed = true];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size +=
          ::_pbi::WireFormatLite::Int64SizeWithPackedTagSize(
              this_._internal_offset(), 1,
              this_._impl_._offset_cached_byte_size_);
    }
    // repeated int32 audio_size = 5 [packed = true];
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size +=
          ::_pbi::WireFormatLite::Int32SizeWithPackedTagSize(
              this_._internal_audio_size(), 1,
              this_._impl_._audio_size_cached_byte_size_);
    }
    // int32 stream_idx = 1;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_stream_idx() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_stream_idx());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void StreamContentsProto::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<StreamContentsProto*>(&to_msg);
  auto& from = static_cast<const StreamContentsProto&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:StreamContentsProto)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_pts()->MergeFrom(from._internal_pts());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_internal_mutable_file_size()->MergeFrom(from._internal_file_size());
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_internal_mutable_offset()->MergeFrom(from._internal_offset());
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _this->_internal_mutable_audio_size()->MergeFrom(from._internal_audio_size());
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_stream_idx() != 0) {
        _this->_impl_.stream_idx_ = from._impl_.stream_idx_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void StreamContentsProto::CopyFrom(const StreamContentsProto& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:StreamContentsProto)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void StreamContentsProto::InternalSwap(StreamContentsProto* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.pts_.InternalSwap(&other->_impl_.pts_);
  _impl_.file_size_.InternalSwap(&other->_impl_.file_size_);
  _impl_.offset_.InternalSwap(&other->_impl_.offset_);
  _impl_.audio_size_.InternalSwap(&other->_impl_.audio_size_);
  swap(_impl_.stream_idx_, other->_impl_.stream_idx_);
}

::google::protobuf::Metadata StreamContentsProto::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

FileContentsProto::FileContentsProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, FileContentsProto_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:FileContentsProto)
}
PROTOBUF_NDEBUG_INLINE FileContentsProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::FileContentsProto& from_msg)
      : _has_bits_{from._has_bits_},
        stream_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::FileContentsProto,
              PROTOBUF_FIELD_OFFSET(::FileContentsProto, _impl_.stream_)>()
          , arena, from.stream_
        }
     {}

FileContentsProto::FileContentsProto(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const FileContentsProto& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, FileContentsProto_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  FileContentsProto* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:FileContentsProto)
}
PROTOBUF_NDEBUG_INLINE FileContentsProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : stream_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::FileContentsProto,
            PROTOBUF_FIELD_OFFSET(::FileContentsProto, _impl_.stream_)>()
         }
     {}

inline void FileContentsProto::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
FileContentsProto::~FileContentsProto() {
  // @@protoc_insertion_point(destructor:FileContentsProto)
  SharedDtor(*this);
}
inline void FileContentsProto::SharedDtor(MessageLite& self) {
  FileContentsProto& this_ = static_cast<FileContentsProto&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull FileContentsProto_class_data_ =
        FileContentsProto::InternalGenerateClassData_(FileContentsProto_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
FileContentsProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&FileContentsProto_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(FileContentsProto_class_data_.tc_table);
  return FileContentsProto_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
FileContentsProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&FileContentsProto_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&FileContentsProto_globals_));
  return FileContentsProto_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const FileContentsProto::ParseTableT_
    FileContentsProto::_table_ =
        FileContentsProto::InternalGenerateParseTable_(FileContentsProto_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void FileContentsProto::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:FileContentsProto)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.stream_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL FileContentsProto::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const FileContentsProto& this_ = static_cast<const FileContentsProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL FileContentsProto::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const FileContentsProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:FileContentsProto)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .StreamContentsProto stream = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_stream_size());
         i < n; i++) {
      const auto& repfield = this_._internal_stream().Get(i);
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          1, repfield, repfield.GetCachedSize(), target,
          stream);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:FileContentsProto)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t FileContentsProto::ByteSizeLong(const MessageLite& base) {
  const FileContentsProto& this_ = static_cast<const FileContentsProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t FileContentsProto::ByteSizeLong() const {
  const FileContentsProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:FileContentsProto)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated .StreamContentsProto stream = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_stream_size();
      for (const auto& msg : this_._internal_stream()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void FileContentsProto::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<FileContentsProto*>(&to_msg);
  auto& from = static_cast<const FileContentsProto&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:FileContentsProto)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _this->_internal_mutable_stream()->InternalMergeFromWithArena(
        ::google::protobuf::MessageLite::internal_visibility(), arena,
        from._internal_stream());
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void FileContentsProto::CopyFrom(const FileContentsProto& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:FileContentsProto)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void FileContentsProto::InternalSwap(FileContentsProto* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.stream_.InternalSwap(&other->_impl_.stream_);
}

::google::protobuf::Metadata FileContentsProto::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_frame_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
