/********************************************************************************
** Form generated from reading UI file 'mainwindow.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDial>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "clickable_label.h"
#include "correlation_meter.h"
#include "glwidget.h"
#include "lrameter.h"
#include "vumeter.h"

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QAction *exit_action;
    QAction *cut_action;
    QAction *about_action;
    QAction *x264_bitrate_action;
    QAction *input_mapping_action;
    QAction *simple_audio_mode;
    QAction *multichannel_audio_mode;
    QAction *midi_mapping_action;
    QAction *manual_action;
    QAction *timecode_stream_action;
    QAction *timecode_stdout_action;
    QAction *open_analyzer_action;
    QAction *quick_cut_enable_action;
    QAction *srt_enable_action;
    QWidget *central_widget;
    QGridLayout *gridLayout;
    QVBoxLayout *vertical_layout;
    QHBoxLayout *me_displays;
    QVBoxLayout *preview_vertical_layout;
    GLWidget *me_preview;
    QLabel *label_preview;
    QVBoxLayout *verticalLayout_2;
    QPushButton *transition_btn1;
    QPushButton *transition_btn2;
    QPushButton *transition_btn3;
    QVBoxLayout *verticalLayout_5;
    GLWidget *me_live;
    QLabel *label_live;
    QVBoxLayout *audio_meters;
    CorrelationMeter *correlation_meter;
    QHBoxLayout *vu_meters;
    QVBoxLayout *vu_meter_vertical_layout;
    QHBoxLayout *horizontalLayout;
    VUMeter *vu_meter;
    ClickableLabel *peak_display;
    QVBoxLayout *lra_vertical_layout;
    QHBoxLayout *horizontalLayout_2;
    LRAMeter *lra_meter;
    QPushButton *reset_meters_button;
    QStackedWidget *audio_views;
    QWidget *compact_audio_view;
    QVBoxLayout *compact_audio_layout;
    QWidget *compact_header;
    QHBoxLayout *horizontalLayout_3;
    QLabel *compact_label;
    QToolButton *compact_prev_page;
    QToolButton *compact_next_page;
    QHBoxLayout *audiostrip;
    QScrollArea *fader_scroll;
    QWidget *fader_scroll_contents;
    QHBoxLayout *horizontalLayout_4;
    QHBoxLayout *faders;
    QSpacerItem *fader_spacer;
    QGridLayout *master_audio_strip;
    QDial *compressor_threshold_knob;
    QCheckBox *makeup_gain_auto_checkbox;
    QLabel *compressor_threshold_label;
    QLabel *gainstaging_label;
    QLabel *compressor_threshold_db_display;
    QDial *gainstaging_knob;
    QLabel *gainstaging_db_display;
    QLabel *limiter_threshold_db_display;
    QCheckBox *limiter_enabled;
    QCheckBox *gainstaging_auto_checkbox;
    QDial *limiter_threshold_knob;
    QLabel *locut_cutoff_display;
    QCheckBox *locut_enabled;
    QLabel *makeup_gain_db_display;
    QCheckBox *compressor_enabled;
    QLabel *locut_cutoff_label;
    QSpacerItem *verticalSpacer;
    QSpacerItem *verticalSpacer_2;
    QDial *makeup_gain_knob;
    QLabel *limiter_threshold_label;
    QLabel *makeup_gain_label;
    QDial *locut_cutoff_knob;
    QHBoxLayout *preview_displays;
    QSpacerItem *horizontalSpacer;
    QWidget *full_audio_view;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *full_header;
    QLabel *full_label;
    QToolButton *full_prev_page;
    QToolButton *full_next_page;
    QHBoxLayout *audiostrip_2;
    QScrollArea *bus_scroll;
    QWidget *bus_scroll_contents;
    QHBoxLayout *horizontalLayout_5;
    QHBoxLayout *buses;
    QSpacerItem *buses_spacer;
    QGridLayout *master_audio_strip_2;
    QLabel *locut_cutoff_display_2;
    QLabel *makeup_gain_db_display_2;
    QLabel *limiter_threshold_db_display_2;
    QDial *locut_cutoff_knob_2;
    QCheckBox *limiter_enabled_2;
    QDial *limiter_threshold_knob_2;
    QCheckBox *makeup_gain_auto_checkbox_2;
    QSpacerItem *verticalSpacer_4;
    QLabel *locut_cutoff_label_2;
    QSpacerItem *verticalSpacer_3;
    QDial *makeup_gain_knob_2;
    QLabel *limiter_threshold_label_2;
    QLabel *makeup_gain_label_2;
    QWidget *video_grid_view;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *video_grid_header;
    QLabel *video_grid_label;
    QToolButton *video_grid_prev_page;
    QToolButton *video_grid_next_page;
    QWidget *preview_displays_grid;
    QMenuBar *menuBar;
    QMenu *video_menu;
    QMenu *display_timecode_menu;
    QMenu *hdmi_sdi_output_device_menu;
    QMenu *hdmi_sdi_output_resolution_menu;
    QMenu *menu_Help;
    QMenu *menu_Audio;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName(QString::fromUtf8("MainWindow"));
        MainWindow->resize(1089, 664);
        exit_action = new QAction(MainWindow);
        exit_action->setObjectName(QString::fromUtf8("exit_action"));
        cut_action = new QAction(MainWindow);
        cut_action->setObjectName(QString::fromUtf8("cut_action"));
        about_action = new QAction(MainWindow);
        about_action->setObjectName(QString::fromUtf8("about_action"));
        x264_bitrate_action = new QAction(MainWindow);
        x264_bitrate_action->setObjectName(QString::fromUtf8("x264_bitrate_action"));
        input_mapping_action = new QAction(MainWindow);
        input_mapping_action->setObjectName(QString::fromUtf8("input_mapping_action"));
        simple_audio_mode = new QAction(MainWindow);
        simple_audio_mode->setObjectName(QString::fromUtf8("simple_audio_mode"));
        simple_audio_mode->setCheckable(true);
        simple_audio_mode->setChecked(true);
        multichannel_audio_mode = new QAction(MainWindow);
        multichannel_audio_mode->setObjectName(QString::fromUtf8("multichannel_audio_mode"));
        multichannel_audio_mode->setCheckable(true);
        midi_mapping_action = new QAction(MainWindow);
        midi_mapping_action->setObjectName(QString::fromUtf8("midi_mapping_action"));
        manual_action = new QAction(MainWindow);
        manual_action->setObjectName(QString::fromUtf8("manual_action"));
        timecode_stream_action = new QAction(MainWindow);
        timecode_stream_action->setObjectName(QString::fromUtf8("timecode_stream_action"));
        timecode_stream_action->setCheckable(true);
        timecode_stdout_action = new QAction(MainWindow);
        timecode_stdout_action->setObjectName(QString::fromUtf8("timecode_stdout_action"));
        timecode_stdout_action->setCheckable(true);
        open_analyzer_action = new QAction(MainWindow);
        open_analyzer_action->setObjectName(QString::fromUtf8("open_analyzer_action"));
        quick_cut_enable_action = new QAction(MainWindow);
        quick_cut_enable_action->setObjectName(QString::fromUtf8("quick_cut_enable_action"));
        quick_cut_enable_action->setCheckable(true);
        srt_enable_action = new QAction(MainWindow);
        srt_enable_action->setObjectName(QString::fromUtf8("srt_enable_action"));
        srt_enable_action->setCheckable(true);
        srt_enable_action->setChecked(true);
        central_widget = new QWidget(MainWindow);
        central_widget->setObjectName(QString::fromUtf8("central_widget"));
        central_widget->setEnabled(true);
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(central_widget->sizePolicy().hasHeightForWidth());
        central_widget->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(central_widget);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        vertical_layout = new QVBoxLayout();
        vertical_layout->setSpacing(6);
        vertical_layout->setObjectName(QString::fromUtf8("vertical_layout"));
        me_displays = new QHBoxLayout();
        me_displays->setSpacing(6);
        me_displays->setObjectName(QString::fromUtf8("me_displays"));
        preview_vertical_layout = new QVBoxLayout();
        preview_vertical_layout->setSpacing(6);
        preview_vertical_layout->setObjectName(QString::fromUtf8("preview_vertical_layout"));
        preview_vertical_layout->setContentsMargins(0, -1, -1, -1);
        me_preview = new GLWidget(central_widget);
        me_preview->setObjectName(QString::fromUtf8("me_preview"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(1);
        sizePolicy1.setHeightForWidth(me_preview->sizePolicy().hasHeightForWidth());
        me_preview->setSizePolicy(sizePolicy1);

        preview_vertical_layout->addWidget(me_preview);

        label_preview = new QLabel(central_widget);
        label_preview->setObjectName(QString::fromUtf8("label_preview"));
        label_preview->setAlignment(Qt::AlignCenter);

        preview_vertical_layout->addWidget(label_preview);


        me_displays->addLayout(preview_vertical_layout);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(25);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(-1, 0, -1, 20);
        transition_btn1 = new QPushButton(central_widget);
        transition_btn1->setObjectName(QString::fromUtf8("transition_btn1"));
        QSizePolicy sizePolicy2(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(transition_btn1->sizePolicy().hasHeightForWidth());
        transition_btn1->setSizePolicy(sizePolicy2);
        transition_btn1->setMinimumSize(QSize(115, 0));
        transition_btn1->setMaximumSize(QSize(16777215, 16777215));

        verticalLayout_2->addWidget(transition_btn1);

        transition_btn2 = new QPushButton(central_widget);
        transition_btn2->setObjectName(QString::fromUtf8("transition_btn2"));
        sizePolicy2.setHeightForWidth(transition_btn2->sizePolicy().hasHeightForWidth());
        transition_btn2->setSizePolicy(sizePolicy2);
        transition_btn2->setMinimumSize(QSize(115, 0));
        transition_btn2->setMaximumSize(QSize(16777215, 16777215));

        verticalLayout_2->addWidget(transition_btn2);

        transition_btn3 = new QPushButton(central_widget);
        transition_btn3->setObjectName(QString::fromUtf8("transition_btn3"));
        sizePolicy2.setHeightForWidth(transition_btn3->sizePolicy().hasHeightForWidth());
        transition_btn3->setSizePolicy(sizePolicy2);
        transition_btn3->setMinimumSize(QSize(115, 0));
        transition_btn3->setMaximumSize(QSize(16777215, 16777215));

        verticalLayout_2->addWidget(transition_btn3);


        me_displays->addLayout(verticalLayout_2);

        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setSpacing(6);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        verticalLayout_5->setContentsMargins(0, -1, -1, -1);
        me_live = new GLWidget(central_widget);
        me_live->setObjectName(QString::fromUtf8("me_live"));
        sizePolicy1.setHeightForWidth(me_live->sizePolicy().hasHeightForWidth());
        me_live->setSizePolicy(sizePolicy1);
        me_live->setSizeIncrement(QSize(16, 9));
        me_live->setBaseSize(QSize(16, 9));

        verticalLayout_5->addWidget(me_live);

        label_live = new QLabel(central_widget);
        label_live->setObjectName(QString::fromUtf8("label_live"));
        label_live->setAlignment(Qt::AlignCenter);

        verticalLayout_5->addWidget(label_live);


        me_displays->addLayout(verticalLayout_5);

        audio_meters = new QVBoxLayout();
        audio_meters->setSpacing(6);
        audio_meters->setObjectName(QString::fromUtf8("audio_meters"));
        audio_meters->setContentsMargins(-1, -1, 0, -1);
        correlation_meter = new CorrelationMeter(central_widget);
        correlation_meter->setObjectName(QString::fromUtf8("correlation_meter"));
        QSizePolicy sizePolicy3(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(correlation_meter->sizePolicy().hasHeightForWidth());
        correlation_meter->setSizePolicy(sizePolicy3);
        correlation_meter->setMinimumSize(QSize(0, 14));
        QPalette palette;
        QBrush brush(QColor(255, 255, 255, 255));
        brush.setStyle(Qt::SolidPattern);
        palette.setBrush(QPalette::Active, QPalette::Base, brush);
        QBrush brush1(QColor(239, 0, 4, 255));
        brush1.setStyle(Qt::SolidPattern);
        palette.setBrush(QPalette::Active, QPalette::Window, brush1);
        palette.setBrush(QPalette::Inactive, QPalette::Base, brush);
        palette.setBrush(QPalette::Inactive, QPalette::Window, brush1);
        palette.setBrush(QPalette::Disabled, QPalette::Base, brush1);
        palette.setBrush(QPalette::Disabled, QPalette::Window, brush1);
        correlation_meter->setPalette(palette);
        correlation_meter->setAutoFillBackground(true);

        audio_meters->addWidget(correlation_meter);

        vu_meters = new QHBoxLayout();
        vu_meters->setSpacing(6);
        vu_meters->setObjectName(QString::fromUtf8("vu_meters"));
        vu_meter_vertical_layout = new QVBoxLayout();
        vu_meter_vertical_layout->setSpacing(6);
        vu_meter_vertical_layout->setObjectName(QString::fromUtf8("vu_meter_vertical_layout"));
        vu_meter_vertical_layout->setContentsMargins(0, -1, -1, 4);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(-1, -1, -1, 0);
        vu_meter = new VUMeter(central_widget);
        vu_meter->setObjectName(QString::fromUtf8("vu_meter"));
        QSizePolicy sizePolicy4(QSizePolicy::Fixed, QSizePolicy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(1);
        sizePolicy4.setHeightForWidth(vu_meter->sizePolicy().hasHeightForWidth());
        vu_meter->setSizePolicy(sizePolicy4);
        vu_meter->setMinimumSize(QSize(16, 0));
        vu_meter->setSizeIncrement(QSize(1, 0));
        vu_meter->setBaseSize(QSize(0, 0));
        QPalette palette1;
        palette1.setBrush(QPalette::Active, QPalette::Base, brush);
        QBrush brush2(QColor(5, 239, 111, 255));
        brush2.setStyle(Qt::SolidPattern);
        palette1.setBrush(QPalette::Active, QPalette::Window, brush2);
        palette1.setBrush(QPalette::Inactive, QPalette::Base, brush);
        palette1.setBrush(QPalette::Inactive, QPalette::Window, brush2);
        palette1.setBrush(QPalette::Disabled, QPalette::Base, brush2);
        palette1.setBrush(QPalette::Disabled, QPalette::Window, brush2);
        vu_meter->setPalette(palette1);
        vu_meter->setAutoFillBackground(true);

        horizontalLayout->addWidget(vu_meter);


        vu_meter_vertical_layout->addLayout(horizontalLayout);

        peak_display = new ClickableLabel(central_widget);
        peak_display->setObjectName(QString::fromUtf8("peak_display"));
        peak_display->setMinimumSize(QSize(30, 0));
        peak_display->setAlignment(Qt::AlignCenter);

        vu_meter_vertical_layout->addWidget(peak_display);

        vu_meter_vertical_layout->setStretch(0, 1);

        vu_meters->addLayout(vu_meter_vertical_layout);

        lra_vertical_layout = new QVBoxLayout();
        lra_vertical_layout->setSpacing(3);
        lra_vertical_layout->setObjectName(QString::fromUtf8("lra_vertical_layout"));
        lra_vertical_layout->setContentsMargins(0, -1, -1, -1);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        lra_meter = new LRAMeter(central_widget);
        lra_meter->setObjectName(QString::fromUtf8("lra_meter"));
        QSizePolicy sizePolicy5(QSizePolicy::Fixed, QSizePolicy::Preferred);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(lra_meter->sizePolicy().hasHeightForWidth());
        lra_meter->setSizePolicy(sizePolicy5);
        lra_meter->setMinimumSize(QSize(24, 0));
        QPalette palette2;
        palette2.setBrush(QPalette::Active, QPalette::Base, brush);
        QBrush brush3(QColor(0, 239, 219, 255));
        brush3.setStyle(Qt::SolidPattern);
        palette2.setBrush(QPalette::Active, QPalette::Window, brush3);
        palette2.setBrush(QPalette::Inactive, QPalette::Base, brush);
        palette2.setBrush(QPalette::Inactive, QPalette::Window, brush3);
        palette2.setBrush(QPalette::Disabled, QPalette::Base, brush3);
        palette2.setBrush(QPalette::Disabled, QPalette::Window, brush3);
        lra_meter->setPalette(palette2);
        lra_meter->setAutoFillBackground(true);

        horizontalLayout_2->addWidget(lra_meter);


        lra_vertical_layout->addLayout(horizontalLayout_2);

        reset_meters_button = new QPushButton(central_widget);
        reset_meters_button->setObjectName(QString::fromUtf8("reset_meters_button"));
        reset_meters_button->setMaximumSize(QSize(30, 20));
        reset_meters_button->setChecked(false);

        lra_vertical_layout->addWidget(reset_meters_button);

        lra_vertical_layout->setStretch(0, 1);

        vu_meters->addLayout(lra_vertical_layout);


        audio_meters->addLayout(vu_meters);


        me_displays->addLayout(audio_meters);


        vertical_layout->addLayout(me_displays);

        audio_views = new QStackedWidget(central_widget);
        audio_views->setObjectName(QString::fromUtf8("audio_views"));
        compact_audio_view = new QWidget();
        compact_audio_view->setObjectName(QString::fromUtf8("compact_audio_view"));
        compact_audio_layout = new QVBoxLayout(compact_audio_view);
        compact_audio_layout->setSpacing(6);
        compact_audio_layout->setContentsMargins(11, 11, 11, 11);
        compact_audio_layout->setObjectName(QString::fromUtf8("compact_audio_layout"));
        compact_audio_layout->setContentsMargins(0, 0, 0, 0);
        compact_header = new QWidget(compact_audio_view);
        compact_header->setObjectName(QString::fromUtf8("compact_header"));
        QSizePolicy sizePolicy6(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy6.setHorizontalStretch(8);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(compact_header->sizePolicy().hasHeightForWidth());
        compact_header->setSizePolicy(sizePolicy6);
        horizontalLayout_3 = new QHBoxLayout(compact_header);
        horizontalLayout_3->setSpacing(0);
        horizontalLayout_3->setContentsMargins(11, 11, 11, 11);
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        horizontalLayout_3->setContentsMargins(0, 0, 0, 0);
        compact_label = new QLabel(compact_header);
        compact_label->setObjectName(QString::fromUtf8("compact_label"));
        compact_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_3->addWidget(compact_label);

        compact_prev_page = new QToolButton(compact_header);
        compact_prev_page->setObjectName(QString::fromUtf8("compact_prev_page"));
        compact_prev_page->setEnabled(true);
        compact_prev_page->setMaximumSize(QSize(15, 15));
        compact_prev_page->setAutoRaise(true);
        compact_prev_page->setArrowType(Qt::LeftArrow);

        horizontalLayout_3->addWidget(compact_prev_page);

        compact_next_page = new QToolButton(compact_header);
        compact_next_page->setObjectName(QString::fromUtf8("compact_next_page"));
        compact_next_page->setMaximumSize(QSize(15, 15));
        compact_next_page->setAutoRaise(true);
        compact_next_page->setArrowType(Qt::RightArrow);

        horizontalLayout_3->addWidget(compact_next_page);


        compact_audio_layout->addWidget(compact_header);

        audiostrip = new QHBoxLayout();
        audiostrip->setSpacing(6);
        audiostrip->setObjectName(QString::fromUtf8("audiostrip"));
        audiostrip->setContentsMargins(-1, 0, -1, -1);
        fader_scroll = new QScrollArea(compact_audio_view);
        fader_scroll->setObjectName(QString::fromUtf8("fader_scroll"));
        fader_scroll->setFrameShape(QFrame::NoFrame);
        fader_scroll->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        fader_scroll->setWidgetResizable(true);
        fader_scroll_contents = new QWidget();
        fader_scroll_contents->setObjectName(QString::fromUtf8("fader_scroll_contents"));
        fader_scroll_contents->setGeometry(QRect(0, 0, 524, 238));
        horizontalLayout_4 = new QHBoxLayout(fader_scroll_contents);
        horizontalLayout_4->setSpacing(0);
        horizontalLayout_4->setContentsMargins(11, 11, 11, 11);
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        horizontalLayout_4->setContentsMargins(0, 0, 0, 0);
        faders = new QHBoxLayout();
        faders->setSpacing(6);
        faders->setObjectName(QString::fromUtf8("faders"));
        faders->setSizeConstraint(QLayout::SetFixedSize);

        horizontalLayout_4->addLayout(faders);

        fader_spacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_4->addItem(fader_spacer);

        fader_scroll->setWidget(fader_scroll_contents);

        audiostrip->addWidget(fader_scroll);

        master_audio_strip = new QGridLayout();
        master_audio_strip->setSpacing(6);
        master_audio_strip->setObjectName(QString::fromUtf8("master_audio_strip"));
        master_audio_strip->setContentsMargins(-1, -1, -1, 0);
        compressor_threshold_knob = new QDial(compact_audio_view);
        compressor_threshold_knob->setObjectName(QString::fromUtf8("compressor_threshold_knob"));
        compressor_threshold_knob->setMinimumSize(QSize(64, 64));
        compressor_threshold_knob->setMaximumSize(QSize(16777215, 64));
        compressor_threshold_knob->setMinimum(-400);
        compressor_threshold_knob->setMaximum(0);
        compressor_threshold_knob->setValue(-260);
        compressor_threshold_knob->setNotchTarget(30.000000000000000);
        compressor_threshold_knob->setNotchesVisible(true);

        master_audio_strip->addWidget(compressor_threshold_knob, 2, 3, 1, 1);

        makeup_gain_auto_checkbox = new QCheckBox(compact_audio_view);
        makeup_gain_auto_checkbox->setObjectName(QString::fromUtf8("makeup_gain_auto_checkbox"));
        makeup_gain_auto_checkbox->setChecked(true);

        master_audio_strip->addWidget(makeup_gain_auto_checkbox, 4, 5, 1, 1);

        compressor_threshold_label = new QLabel(compact_audio_view);
        compressor_threshold_label->setObjectName(QString::fromUtf8("compressor_threshold_label"));

        master_audio_strip->addWidget(compressor_threshold_label, 1, 3, 1, 1);

        gainstaging_label = new QLabel(compact_audio_view);
        gainstaging_label->setObjectName(QString::fromUtf8("gainstaging_label"));
        gainstaging_label->setAlignment(Qt::AlignCenter);

        master_audio_strip->addWidget(gainstaging_label, 1, 2, 1, 1);

        compressor_threshold_db_display = new QLabel(compact_audio_view);
        compressor_threshold_db_display->setObjectName(QString::fromUtf8("compressor_threshold_db_display"));
        compressor_threshold_db_display->setAlignment(Qt::AlignCenter);

        master_audio_strip->addWidget(compressor_threshold_db_display, 3, 3, 1, 1);

        gainstaging_knob = new QDial(compact_audio_view);
        gainstaging_knob->setObjectName(QString::fromUtf8("gainstaging_knob"));
        gainstaging_knob->setMinimumSize(QSize(64, 64));
        gainstaging_knob->setMaximumSize(QSize(16777215, 64));
        gainstaging_knob->setMinimum(-300);
        gainstaging_knob->setMaximum(300);
        gainstaging_knob->setNotchTarget(60.000000000000000);
        gainstaging_knob->setNotchesVisible(true);

        master_audio_strip->addWidget(gainstaging_knob, 2, 2, 1, 1);

        gainstaging_db_display = new QLabel(compact_audio_view);
        gainstaging_db_display->setObjectName(QString::fromUtf8("gainstaging_db_display"));
        gainstaging_db_display->setAlignment(Qt::AlignCenter);

        master_audio_strip->addWidget(gainstaging_db_display, 3, 2, 1, 1);

        limiter_threshold_db_display = new QLabel(compact_audio_view);
        limiter_threshold_db_display->setObjectName(QString::fromUtf8("limiter_threshold_db_display"));
        limiter_threshold_db_display->setAlignment(Qt::AlignCenter);

        master_audio_strip->addWidget(limiter_threshold_db_display, 3, 4, 1, 1);

        limiter_enabled = new QCheckBox(compact_audio_view);
        limiter_enabled->setObjectName(QString::fromUtf8("limiter_enabled"));
        limiter_enabled->setChecked(true);

        master_audio_strip->addWidget(limiter_enabled, 4, 4, 1, 1);

        gainstaging_auto_checkbox = new QCheckBox(compact_audio_view);
        gainstaging_auto_checkbox->setObjectName(QString::fromUtf8("gainstaging_auto_checkbox"));
        gainstaging_auto_checkbox->setChecked(true);

        master_audio_strip->addWidget(gainstaging_auto_checkbox, 4, 2, 1, 1);

        limiter_threshold_knob = new QDial(compact_audio_view);
        limiter_threshold_knob->setObjectName(QString::fromUtf8("limiter_threshold_knob"));
        limiter_threshold_knob->setMinimumSize(QSize(64, 64));
        limiter_threshold_knob->setMaximumSize(QSize(16777215, 64));
        limiter_threshold_knob->setMinimum(-400);
        limiter_threshold_knob->setMaximum(0);
        limiter_threshold_knob->setValue(-140);
        limiter_threshold_knob->setNotchTarget(30.000000000000000);
        limiter_threshold_knob->setNotchesVisible(true);

        master_audio_strip->addWidget(limiter_threshold_knob, 2, 4, 1, 1);

        locut_cutoff_display = new QLabel(compact_audio_view);
        locut_cutoff_display->setObjectName(QString::fromUtf8("locut_cutoff_display"));
        locut_cutoff_display->setAlignment(Qt::AlignCenter);

        master_audio_strip->addWidget(locut_cutoff_display, 3, 1, 1, 1);

        locut_enabled = new QCheckBox(compact_audio_view);
        locut_enabled->setObjectName(QString::fromUtf8("locut_enabled"));
        locut_enabled->setChecked(true);

        master_audio_strip->addWidget(locut_enabled, 4, 1, 1, 1);

        makeup_gain_db_display = new QLabel(compact_audio_view);
        makeup_gain_db_display->setObjectName(QString::fromUtf8("makeup_gain_db_display"));
        makeup_gain_db_display->setAlignment(Qt::AlignCenter);

        master_audio_strip->addWidget(makeup_gain_db_display, 3, 5, 1, 1);

        compressor_enabled = new QCheckBox(compact_audio_view);
        compressor_enabled->setObjectName(QString::fromUtf8("compressor_enabled"));
        compressor_enabled->setChecked(true);

        master_audio_strip->addWidget(compressor_enabled, 4, 3, 1, 1);

        locut_cutoff_label = new QLabel(compact_audio_view);
        locut_cutoff_label->setObjectName(QString::fromUtf8("locut_cutoff_label"));

        master_audio_strip->addWidget(locut_cutoff_label, 1, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        master_audio_strip->addItem(verticalSpacer, 0, 1, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        master_audio_strip->addItem(verticalSpacer_2, 5, 1, 1, 1);

        makeup_gain_knob = new QDial(compact_audio_view);
        makeup_gain_knob->setObjectName(QString::fromUtf8("makeup_gain_knob"));
        makeup_gain_knob->setMinimumSize(QSize(64, 64));
        makeup_gain_knob->setMaximumSize(QSize(16777215, 64));
        makeup_gain_knob->setMinimum(-150);
        makeup_gain_knob->setMaximum(150);
        makeup_gain_knob->setNotchTarget(30.000000000000000);
        makeup_gain_knob->setNotchesVisible(true);

        master_audio_strip->addWidget(makeup_gain_knob, 2, 5, 1, 1);

        limiter_threshold_label = new QLabel(compact_audio_view);
        limiter_threshold_label->setObjectName(QString::fromUtf8("limiter_threshold_label"));
        limiter_threshold_label->setAlignment(Qt::AlignCenter);

        master_audio_strip->addWidget(limiter_threshold_label, 1, 4, 1, 1);

        makeup_gain_label = new QLabel(compact_audio_view);
        makeup_gain_label->setObjectName(QString::fromUtf8("makeup_gain_label"));
        makeup_gain_label->setAlignment(Qt::AlignCenter);

        master_audio_strip->addWidget(makeup_gain_label, 1, 5, 1, 1);

        locut_cutoff_knob = new QDial(compact_audio_view);
        locut_cutoff_knob->setObjectName(QString::fromUtf8("locut_cutoff_knob"));
        QSizePolicy sizePolicy7(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy7.setHorizontalStretch(0);
        sizePolicy7.setVerticalStretch(0);
        sizePolicy7.setHeightForWidth(locut_cutoff_knob->sizePolicy().hasHeightForWidth());
        locut_cutoff_knob->setSizePolicy(sizePolicy7);
        locut_cutoff_knob->setMinimumSize(QSize(64, 64));
        locut_cutoff_knob->setMaximumSize(QSize(16777215, 64));
        locut_cutoff_knob->setMaximum(60);
        locut_cutoff_knob->setValue(26);

        master_audio_strip->addWidget(locut_cutoff_knob, 2, 1, 1, 1);


        audiostrip->addLayout(master_audio_strip);


        compact_audio_layout->addLayout(audiostrip);

        preview_displays = new QHBoxLayout();
        preview_displays->setSpacing(6);
        preview_displays->setObjectName(QString::fromUtf8("preview_displays"));
        horizontalSpacer = new QSpacerItem(0, 40, QSizePolicy::Preferred, QSizePolicy::Minimum);

        preview_displays->addItem(horizontalSpacer);


        compact_audio_layout->addLayout(preview_displays);

        audio_views->addWidget(compact_audio_view);
        full_audio_view = new QWidget();
        full_audio_view->setObjectName(QString::fromUtf8("full_audio_view"));
        verticalLayout_3 = new QVBoxLayout(full_audio_view);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        full_header = new QHBoxLayout();
        full_header->setSpacing(0);
        full_header->setObjectName(QString::fromUtf8("full_header"));
        full_label = new QLabel(full_audio_view);
        full_label->setObjectName(QString::fromUtf8("full_label"));
        full_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        full_header->addWidget(full_label);

        full_prev_page = new QToolButton(full_audio_view);
        full_prev_page->setObjectName(QString::fromUtf8("full_prev_page"));
        full_prev_page->setEnabled(true);
        full_prev_page->setMaximumSize(QSize(15, 15));
        full_prev_page->setAutoRaise(true);
        full_prev_page->setArrowType(Qt::LeftArrow);

        full_header->addWidget(full_prev_page);

        full_next_page = new QToolButton(full_audio_view);
        full_next_page->setObjectName(QString::fromUtf8("full_next_page"));
        full_next_page->setMaximumSize(QSize(15, 15));
        full_next_page->setAutoRaise(true);
        full_next_page->setArrowType(Qt::RightArrow);

        full_header->addWidget(full_next_page);


        verticalLayout_3->addLayout(full_header);

        audiostrip_2 = new QHBoxLayout();
        audiostrip_2->setSpacing(6);
        audiostrip_2->setObjectName(QString::fromUtf8("audiostrip_2"));
        audiostrip_2->setContentsMargins(-1, 0, -1, -1);
        bus_scroll = new QScrollArea(full_audio_view);
        bus_scroll->setObjectName(QString::fromUtf8("bus_scroll"));
        bus_scroll->setFrameShape(QFrame::NoFrame);
        bus_scroll->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        bus_scroll->setWidgetResizable(true);
        bus_scroll_contents = new QWidget();
        bus_scroll_contents->setObjectName(QString::fromUtf8("bus_scroll_contents"));
        bus_scroll_contents->setGeometry(QRect(0, 0, 46, 20));
        horizontalLayout_5 = new QHBoxLayout(bus_scroll_contents);
        horizontalLayout_5->setSpacing(0);
        horizontalLayout_5->setContentsMargins(11, 11, 11, 11);
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        horizontalLayout_5->setContentsMargins(0, 0, 0, 0);
        buses = new QHBoxLayout();
        buses->setSpacing(6);
        buses->setObjectName(QString::fromUtf8("buses"));
        buses->setSizeConstraint(QLayout::SetFixedSize);

        horizontalLayout_5->addLayout(buses);

        buses_spacer = new QSpacerItem(723, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(buses_spacer);

        bus_scroll->setWidget(bus_scroll_contents);

        audiostrip_2->addWidget(bus_scroll);

        master_audio_strip_2 = new QGridLayout();
        master_audio_strip_2->setSpacing(6);
        master_audio_strip_2->setObjectName(QString::fromUtf8("master_audio_strip_2"));
        master_audio_strip_2->setContentsMargins(-1, -1, -1, 0);
        locut_cutoff_display_2 = new QLabel(full_audio_view);
        locut_cutoff_display_2->setObjectName(QString::fromUtf8("locut_cutoff_display_2"));
        locut_cutoff_display_2->setAlignment(Qt::AlignCenter);

        master_audio_strip_2->addWidget(locut_cutoff_display_2, 3, 1, 1, 1);

        makeup_gain_db_display_2 = new QLabel(full_audio_view);
        makeup_gain_db_display_2->setObjectName(QString::fromUtf8("makeup_gain_db_display_2"));
        makeup_gain_db_display_2->setAlignment(Qt::AlignCenter);

        master_audio_strip_2->addWidget(makeup_gain_db_display_2, 3, 3, 1, 1);

        limiter_threshold_db_display_2 = new QLabel(full_audio_view);
        limiter_threshold_db_display_2->setObjectName(QString::fromUtf8("limiter_threshold_db_display_2"));
        limiter_threshold_db_display_2->setAlignment(Qt::AlignCenter);

        master_audio_strip_2->addWidget(limiter_threshold_db_display_2, 3, 2, 1, 1);

        locut_cutoff_knob_2 = new QDial(full_audio_view);
        locut_cutoff_knob_2->setObjectName(QString::fromUtf8("locut_cutoff_knob_2"));
        sizePolicy7.setHeightForWidth(locut_cutoff_knob_2->sizePolicy().hasHeightForWidth());
        locut_cutoff_knob_2->setSizePolicy(sizePolicy7);
        locut_cutoff_knob_2->setMinimumSize(QSize(64, 64));
        locut_cutoff_knob_2->setMaximumSize(QSize(16777215, 64));
        locut_cutoff_knob_2->setMaximum(60);
        locut_cutoff_knob_2->setValue(26);

        master_audio_strip_2->addWidget(locut_cutoff_knob_2, 2, 1, 1, 1);

        limiter_enabled_2 = new QCheckBox(full_audio_view);
        limiter_enabled_2->setObjectName(QString::fromUtf8("limiter_enabled_2"));
        limiter_enabled_2->setChecked(true);

        master_audio_strip_2->addWidget(limiter_enabled_2, 4, 2, 1, 1);

        limiter_threshold_knob_2 = new QDial(full_audio_view);
        limiter_threshold_knob_2->setObjectName(QString::fromUtf8("limiter_threshold_knob_2"));
        limiter_threshold_knob_2->setMinimumSize(QSize(64, 64));
        limiter_threshold_knob_2->setMaximumSize(QSize(16777215, 64));
        limiter_threshold_knob_2->setMinimum(-400);
        limiter_threshold_knob_2->setMaximum(0);
        limiter_threshold_knob_2->setValue(-140);
        limiter_threshold_knob_2->setNotchTarget(30.000000000000000);
        limiter_threshold_knob_2->setNotchesVisible(true);

        master_audio_strip_2->addWidget(limiter_threshold_knob_2, 2, 2, 1, 1);

        makeup_gain_auto_checkbox_2 = new QCheckBox(full_audio_view);
        makeup_gain_auto_checkbox_2->setObjectName(QString::fromUtf8("makeup_gain_auto_checkbox_2"));
        makeup_gain_auto_checkbox_2->setChecked(true);

        master_audio_strip_2->addWidget(makeup_gain_auto_checkbox_2, 4, 3, 1, 1);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        master_audio_strip_2->addItem(verticalSpacer_4, 5, 1, 1, 1);

        locut_cutoff_label_2 = new QLabel(full_audio_view);
        locut_cutoff_label_2->setObjectName(QString::fromUtf8("locut_cutoff_label_2"));

        master_audio_strip_2->addWidget(locut_cutoff_label_2, 1, 1, 1, 1);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        master_audio_strip_2->addItem(verticalSpacer_3, 0, 1, 1, 1);

        makeup_gain_knob_2 = new QDial(full_audio_view);
        makeup_gain_knob_2->setObjectName(QString::fromUtf8("makeup_gain_knob_2"));
        makeup_gain_knob_2->setMinimumSize(QSize(64, 64));
        makeup_gain_knob_2->setMaximumSize(QSize(16777215, 64));
        makeup_gain_knob_2->setMinimum(-150);
        makeup_gain_knob_2->setMaximum(150);
        makeup_gain_knob_2->setNotchTarget(30.000000000000000);
        makeup_gain_knob_2->setNotchesVisible(true);

        master_audio_strip_2->addWidget(makeup_gain_knob_2, 2, 3, 1, 1);

        limiter_threshold_label_2 = new QLabel(full_audio_view);
        limiter_threshold_label_2->setObjectName(QString::fromUtf8("limiter_threshold_label_2"));
        limiter_threshold_label_2->setAlignment(Qt::AlignCenter);

        master_audio_strip_2->addWidget(limiter_threshold_label_2, 1, 2, 1, 1);

        makeup_gain_label_2 = new QLabel(full_audio_view);
        makeup_gain_label_2->setObjectName(QString::fromUtf8("makeup_gain_label_2"));
        makeup_gain_label_2->setAlignment(Qt::AlignCenter);

        master_audio_strip_2->addWidget(makeup_gain_label_2, 1, 3, 1, 1);


        audiostrip_2->addLayout(master_audio_strip_2);


        verticalLayout_3->addLayout(audiostrip_2);

        audio_views->addWidget(full_audio_view);
        video_grid_view = new QWidget();
        video_grid_view->setObjectName(QString::fromUtf8("video_grid_view"));
        verticalLayout_4 = new QVBoxLayout(video_grid_view);
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setContentsMargins(11, 11, 11, 11);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        video_grid_header = new QHBoxLayout();
        video_grid_header->setSpacing(0);
        video_grid_header->setObjectName(QString::fromUtf8("video_grid_header"));
        video_grid_label = new QLabel(video_grid_view);
        video_grid_label->setObjectName(QString::fromUtf8("video_grid_label"));
        video_grid_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        video_grid_header->addWidget(video_grid_label);

        video_grid_prev_page = new QToolButton(video_grid_view);
        video_grid_prev_page->setObjectName(QString::fromUtf8("video_grid_prev_page"));
        video_grid_prev_page->setEnabled(true);
        video_grid_prev_page->setMaximumSize(QSize(15, 15));
        video_grid_prev_page->setAutoRaise(true);
        video_grid_prev_page->setArrowType(Qt::LeftArrow);

        video_grid_header->addWidget(video_grid_prev_page);

        video_grid_next_page = new QToolButton(video_grid_view);
        video_grid_next_page->setObjectName(QString::fromUtf8("video_grid_next_page"));
        video_grid_next_page->setMaximumSize(QSize(15, 15));
        video_grid_next_page->setAutoRaise(true);
        video_grid_next_page->setArrowType(Qt::RightArrow);

        video_grid_header->addWidget(video_grid_next_page);


        verticalLayout_4->addLayout(video_grid_header);

        preview_displays_grid = new QWidget(video_grid_view);
        preview_displays_grid->setObjectName(QString::fromUtf8("preview_displays_grid"));

        verticalLayout_4->addWidget(preview_displays_grid);

        verticalLayout_4->setStretch(1, 1);
        audio_views->addWidget(video_grid_view);

        vertical_layout->addWidget(audio_views);


        gridLayout->addLayout(vertical_layout, 0, 0, 1, 1);

        MainWindow->setCentralWidget(central_widget);
        menuBar = new QMenuBar(MainWindow);
        menuBar->setObjectName(QString::fromUtf8("menuBar"));
        menuBar->setGeometry(QRect(0, 0, 1089, 20));
        video_menu = new QMenu(menuBar);
        video_menu->setObjectName(QString::fromUtf8("video_menu"));
        display_timecode_menu = new QMenu(video_menu);
        display_timecode_menu->setObjectName(QString::fromUtf8("display_timecode_menu"));
        hdmi_sdi_output_device_menu = new QMenu(video_menu);
        hdmi_sdi_output_device_menu->setObjectName(QString::fromUtf8("hdmi_sdi_output_device_menu"));
        hdmi_sdi_output_resolution_menu = new QMenu(video_menu);
        hdmi_sdi_output_resolution_menu->setObjectName(QString::fromUtf8("hdmi_sdi_output_resolution_menu"));
        menu_Help = new QMenu(menuBar);
        menu_Help->setObjectName(QString::fromUtf8("menu_Help"));
        menu_Audio = new QMenu(menuBar);
        menu_Audio->setObjectName(QString::fromUtf8("menu_Audio"));
        MainWindow->setMenuBar(menuBar);

        menuBar->addAction(video_menu->menuAction());
        menuBar->addAction(menu_Audio->menuAction());
        menuBar->addAction(menu_Help->menuAction());
        video_menu->addAction(cut_action);
        video_menu->addAction(open_analyzer_action);
        video_menu->addAction(x264_bitrate_action);
        video_menu->addAction(hdmi_sdi_output_device_menu->menuAction());
        video_menu->addAction(hdmi_sdi_output_resolution_menu->menuAction());
        video_menu->addAction(display_timecode_menu->menuAction());
        video_menu->addAction(quick_cut_enable_action);
        video_menu->addAction(srt_enable_action);
        video_menu->addAction(exit_action);
        display_timecode_menu->addAction(timecode_stream_action);
        display_timecode_menu->addAction(timecode_stdout_action);
        hdmi_sdi_output_device_menu->addSeparator();
        hdmi_sdi_output_resolution_menu->addSeparator();
        menu_Help->addAction(manual_action);
        menu_Help->addAction(about_action);
        menu_Audio->addAction(simple_audio_mode);
        menu_Audio->addAction(multichannel_audio_mode);
        menu_Audio->addSeparator();
        menu_Audio->addAction(input_mapping_action);
        menu_Audio->addAction(midi_mapping_action);

        retranslateUi(MainWindow);

        audio_views->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        MainWindow->setWindowTitle(QCoreApplication::translate("MainWindow", "Nageru", nullptr));
        exit_action->setText(QCoreApplication::translate("MainWindow", "&Exit", nullptr));
        cut_action->setText(QCoreApplication::translate("MainWindow", "&Begin new video segment", nullptr));
        about_action->setText(QCoreApplication::translate("MainWindow", "&About Nageru\342\200\246", nullptr));
        x264_bitrate_action->setText(QCoreApplication::translate("MainWindow", "Change &x264 bitrate\342\200\246", nullptr));
        input_mapping_action->setText(QCoreApplication::translate("MainWindow", "&Input mapping\342\200\246", nullptr));
        simple_audio_mode->setText(QCoreApplication::translate("MainWindow", "Simple", nullptr));
        multichannel_audio_mode->setText(QCoreApplication::translate("MainWindow", "Multichannel", nullptr));
        midi_mapping_action->setText(QCoreApplication::translate("MainWindow", "Setup MIDI controller\342\200\246", nullptr));
        manual_action->setText(QCoreApplication::translate("MainWindow", "Online &manual\342\200\246", nullptr));
        timecode_stream_action->setText(QCoreApplication::translate("MainWindow", "In &stream", nullptr));
        timecode_stdout_action->setText(QCoreApplication::translate("MainWindow", "On standard &output", nullptr));
        open_analyzer_action->setText(QCoreApplication::translate("MainWindow", "Open frame &analyzer\342\200\246", nullptr));
        quick_cut_enable_action->setText(QCoreApplication::translate("MainWindow", "Enable &quick-cut keys (Q, W, E, etc.)", nullptr));
        srt_enable_action->setText(QCoreApplication::translate("MainWindow", "Accept new SRT connections on port 9710", nullptr));
        label_preview->setText(QCoreApplication::translate("MainWindow", "Preview", nullptr));
        transition_btn1->setText(QCoreApplication::translate("MainWindow", "Cut", nullptr));
        transition_btn2->setText(QCoreApplication::translate("MainWindow", "Fade", nullptr));
        transition_btn3->setText(QCoreApplication::translate("MainWindow", "Wipe", nullptr));
        label_live->setText(QCoreApplication::translate("MainWindow", "Live", nullptr));
        peak_display->setText(QCoreApplication::translate("MainWindow", "-0.0", nullptr));
        reset_meters_button->setText(QCoreApplication::translate("MainWindow", "RST", nullptr));
        compact_label->setText(QCoreApplication::translate("MainWindow", "Compact audio view (1/3)  ", nullptr));
        compact_prev_page->setText(QCoreApplication::translate("MainWindow", "...", nullptr));
        compact_next_page->setText(QCoreApplication::translate("MainWindow", "...", nullptr));
        makeup_gain_auto_checkbox->setText(QCoreApplication::translate("MainWindow", "Auto", nullptr));
        compressor_threshold_label->setText(QCoreApplication::translate("MainWindow", "Compr. threshold", nullptr));
        gainstaging_label->setText(QCoreApplication::translate("MainWindow", "Gain staging", nullptr));
        compressor_threshold_db_display->setText(QCoreApplication::translate("MainWindow", "-26.0 dB", nullptr));
        gainstaging_db_display->setText(QCoreApplication::translate("MainWindow", "-0.0 dB", nullptr));
        limiter_threshold_db_display->setText(QCoreApplication::translate("MainWindow", "-14.0 dB", nullptr));
        limiter_enabled->setText(QCoreApplication::translate("MainWindow", "Enabled", nullptr));
        gainstaging_auto_checkbox->setText(QCoreApplication::translate("MainWindow", "Auto", nullptr));
        locut_cutoff_display->setText(QCoreApplication::translate("MainWindow", "120 Hz", nullptr));
        locut_enabled->setText(QCoreApplication::translate("MainWindow", "Enabled", nullptr));
        makeup_gain_db_display->setText(QCoreApplication::translate("MainWindow", "-0.0 dB", nullptr));
        compressor_enabled->setText(QCoreApplication::translate("MainWindow", "Enabled", nullptr));
        locut_cutoff_label->setText(QCoreApplication::translate("MainWindow", "Lo-cut (24dB/oct)", nullptr));
        limiter_threshold_label->setText(QCoreApplication::translate("MainWindow", "Limiter threshold", nullptr));
        makeup_gain_label->setText(QCoreApplication::translate("MainWindow", "Makeup gain", nullptr));
        full_label->setText(QCoreApplication::translate("MainWindow", "Full audio view (2/3)  ", nullptr));
        full_prev_page->setText(QCoreApplication::translate("MainWindow", "...", nullptr));
        full_next_page->setText(QCoreApplication::translate("MainWindow", "...", nullptr));
        locut_cutoff_display_2->setText(QCoreApplication::translate("MainWindow", "120 Hz", nullptr));
        makeup_gain_db_display_2->setText(QCoreApplication::translate("MainWindow", "-0.0 dB", nullptr));
        limiter_threshold_db_display_2->setText(QCoreApplication::translate("MainWindow", "-14.0 dB", nullptr));
        limiter_enabled_2->setText(QCoreApplication::translate("MainWindow", "Enabled", nullptr));
        makeup_gain_auto_checkbox_2->setText(QCoreApplication::translate("MainWindow", "Auto", nullptr));
        locut_cutoff_label_2->setText(QCoreApplication::translate("MainWindow", "Lo-cut (24dB/oct)", nullptr));
        limiter_threshold_label_2->setText(QCoreApplication::translate("MainWindow", "Limiter threshold", nullptr));
        makeup_gain_label_2->setText(QCoreApplication::translate("MainWindow", "Makeup gain", nullptr));
        video_grid_label->setText(QCoreApplication::translate("MainWindow", "Video grid display (3/3)  ", nullptr));
        video_grid_prev_page->setText(QCoreApplication::translate("MainWindow", "...", nullptr));
        video_grid_next_page->setText(QCoreApplication::translate("MainWindow", "...", nullptr));
        video_menu->setTitle(QCoreApplication::translate("MainWindow", "&Video", nullptr));
        display_timecode_menu->setTitle(QCoreApplication::translate("MainWindow", "Display &time code", nullptr));
        hdmi_sdi_output_device_menu->setTitle(QCoreApplication::translate("MainWindow", "HDMI/SDI output device", nullptr));
        hdmi_sdi_output_resolution_menu->setTitle(QCoreApplication::translate("MainWindow", "HDMI/SDI output resolution", nullptr));
        menu_Help->setTitle(QCoreApplication::translate("MainWindow", "&Help", nullptr));
        menu_Audio->setTitle(QCoreApplication::translate("MainWindow", "&Audio", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINDOW_H
