/********************************************************************************
** Form generated from reading UI file 'audio_expanded_view.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_AUDIO_EXPANDED_VIEW_H
#define UI_AUDIO_EXPANDED_VIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDial>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "clickable_label.h"
#include "compression_reduction_meter.h"
#include "ellipsis_label.h"
#include "nonlinear_fader.h"
#include "vumeter.h"

QT_BEGIN_NAMESPACE

class Ui_AudioExpandedView
{
public:
    QVBoxLayout *verticalLayout_6;
    EllipsisLabel *bus_desc_label;
    QHBoxLayout *main_layout;
    QVBoxLayout *settings_layout;
    QSpacerItem *verticalSpacer_2;
    QHBoxLayout *stereo_width_layout;
    QDial *stereo_width_knob;
    QLabel *stereo_width_label;
    QFrame *line_3;
    QHBoxLayout *locut_offseter;
    QCheckBox *locut_enabled;
    QHBoxLayout *treble_layout;
    QDial *treble_knob;
    QLabel *treble_label;
    QHBoxLayout *mid_layout;
    QDial *mid_knob;
    QLabel *mid_label;
    QHBoxLayout *bass_layout;
    QDial *bass_knob;
    QLabel *bass_label;
    QFrame *line;
    QHBoxLayout *gainstaging_auto_offseter;
    QCheckBox *gainstaging_auto_checkbox;
    QHBoxLayout *gainstaging_layout;
    QDial *gainstaging_knob;
    QLabel *gainstaging_db_display;
    QFrame *line_2;
    QHBoxLayout *compressor_centerer;
    QSpacerItem *horizontalSpacer;
    QCheckBox *compressor_enabled;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *compressor_controls_layout;
    QVBoxLayout *threshold_layout;
    QLabel *threshold_heading;
    QHBoxLayout *compressor_threshold_knob_centerer;
    QDial *compressor_threshold_knob;
    QLabel *compressor_threshold_db_display;
    QVBoxLayout *reduction_layout;
    QLabel *reduction_header;
    QHBoxLayout *reduction_meter_centerer;
    CompressionReductionMeter *reduction_meter;
    QSpacerItem *verticalSpacer;
    QVBoxLayout *vu_layout;
    QHBoxLayout *vu_centerer;
    VUMeter *peak_meter;
    ClickableLabel *peak_display_label;
    QVBoxLayout *fader_layout;
    QHBoxLayout *mute_centerer;
    QPushButton *mute_button;
    QHBoxLayout *fader_centerer;
    NonLinearFader *fader;
    QLabel *fader_label;

    void setupUi(QWidget *AudioExpandedView)
    {
        if (AudioExpandedView->objectName().isEmpty())
            AudioExpandedView->setObjectName(QString::fromUtf8("AudioExpandedView"));
        AudioExpandedView->resize(312, 484);
        verticalLayout_6 = new QVBoxLayout(AudioExpandedView);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        bus_desc_label = new EllipsisLabel(AudioExpandedView);
        bus_desc_label->setObjectName(QString::fromUtf8("bus_desc_label"));
        QFont font;
        font.setFamily(QString::fromUtf8("DejaVu Sans"));
        font.setBold(true);
        font.setUnderline(true);
        font.setWeight(75);
        bus_desc_label->setFont(font);
        bus_desc_label->setAlignment(Qt::AlignCenter);

        verticalLayout_6->addWidget(bus_desc_label);

        main_layout = new QHBoxLayout();
        main_layout->setObjectName(QString::fromUtf8("main_layout"));
        settings_layout = new QVBoxLayout();
        settings_layout->setObjectName(QString::fromUtf8("settings_layout"));
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        settings_layout->addItem(verticalSpacer_2);

        stereo_width_layout = new QHBoxLayout();
        stereo_width_layout->setObjectName(QString::fromUtf8("stereo_width_layout"));
        stereo_width_knob = new QDial(AudioExpandedView);
        stereo_width_knob->setObjectName(QString::fromUtf8("stereo_width_knob"));
        stereo_width_knob->setMaximumSize(QSize(31, 31));
        stereo_width_knob->setMinimum(-100);
        stereo_width_knob->setMaximum(100);
        stereo_width_knob->setValue(100);
        stereo_width_knob->setNotchTarget(50.000000000000000);
        stereo_width_knob->setNotchesVisible(true);

        stereo_width_layout->addWidget(stereo_width_knob);

        stereo_width_label = new QLabel(AudioExpandedView);
        stereo_width_label->setObjectName(QString::fromUtf8("stereo_width_label"));

        stereo_width_layout->addWidget(stereo_width_label);


        settings_layout->addLayout(stereo_width_layout);

        line_3 = new QFrame(AudioExpandedView);
        line_3->setObjectName(QString::fromUtf8("line_3"));
        line_3->setFrameShape(QFrame::HLine);
        line_3->setFrameShadow(QFrame::Sunken);

        settings_layout->addWidget(line_3);

        locut_offseter = new QHBoxLayout();
        locut_offseter->setObjectName(QString::fromUtf8("locut_offseter"));
        locut_offseter->setContentsMargins(8, -1, -1, -1);
        locut_enabled = new QCheckBox(AudioExpandedView);
        locut_enabled->setObjectName(QString::fromUtf8("locut_enabled"));

        locut_offseter->addWidget(locut_enabled);


        settings_layout->addLayout(locut_offseter);

        treble_layout = new QHBoxLayout();
        treble_layout->setObjectName(QString::fromUtf8("treble_layout"));
        treble_knob = new QDial(AudioExpandedView);
        treble_knob->setObjectName(QString::fromUtf8("treble_knob"));
        treble_knob->setMaximumSize(QSize(31, 31));
        treble_knob->setMinimum(-150);
        treble_knob->setMaximum(150);
        treble_knob->setNotchTarget(60.000000000000000);
        treble_knob->setNotchesVisible(true);

        treble_layout->addWidget(treble_knob);

        treble_label = new QLabel(AudioExpandedView);
        treble_label->setObjectName(QString::fromUtf8("treble_label"));

        treble_layout->addWidget(treble_label);


        settings_layout->addLayout(treble_layout);

        mid_layout = new QHBoxLayout();
        mid_layout->setObjectName(QString::fromUtf8("mid_layout"));
        mid_knob = new QDial(AudioExpandedView);
        mid_knob->setObjectName(QString::fromUtf8("mid_knob"));
        mid_knob->setMaximumSize(QSize(31, 31));
        mid_knob->setMinimum(-150);
        mid_knob->setMaximum(150);
        mid_knob->setNotchTarget(60.000000000000000);
        mid_knob->setNotchesVisible(true);

        mid_layout->addWidget(mid_knob);

        mid_label = new QLabel(AudioExpandedView);
        mid_label->setObjectName(QString::fromUtf8("mid_label"));

        mid_layout->addWidget(mid_label);


        settings_layout->addLayout(mid_layout);

        bass_layout = new QHBoxLayout();
        bass_layout->setObjectName(QString::fromUtf8("bass_layout"));
        bass_knob = new QDial(AudioExpandedView);
        bass_knob->setObjectName(QString::fromUtf8("bass_knob"));
        bass_knob->setMaximumSize(QSize(31, 31));
        bass_knob->setMinimum(-150);
        bass_knob->setMaximum(150);
        bass_knob->setNotchTarget(60.000000000000000);
        bass_knob->setNotchesVisible(true);

        bass_layout->addWidget(bass_knob);

        bass_label = new QLabel(AudioExpandedView);
        bass_label->setObjectName(QString::fromUtf8("bass_label"));

        bass_layout->addWidget(bass_label);


        settings_layout->addLayout(bass_layout);

        line = new QFrame(AudioExpandedView);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        settings_layout->addWidget(line);

        gainstaging_auto_offseter = new QHBoxLayout();
        gainstaging_auto_offseter->setObjectName(QString::fromUtf8("gainstaging_auto_offseter"));
        gainstaging_auto_offseter->setContentsMargins(8, -1, -1, -1);
        gainstaging_auto_checkbox = new QCheckBox(AudioExpandedView);
        gainstaging_auto_checkbox->setObjectName(QString::fromUtf8("gainstaging_auto_checkbox"));
        gainstaging_auto_checkbox->setChecked(true);

        gainstaging_auto_offseter->addWidget(gainstaging_auto_checkbox);


        settings_layout->addLayout(gainstaging_auto_offseter);

        gainstaging_layout = new QHBoxLayout();
        gainstaging_layout->setObjectName(QString::fromUtf8("gainstaging_layout"));
        gainstaging_knob = new QDial(AudioExpandedView);
        gainstaging_knob->setObjectName(QString::fromUtf8("gainstaging_knob"));
        gainstaging_knob->setMaximumSize(QSize(31, 31));
        gainstaging_knob->setMinimum(-300);
        gainstaging_knob->setMaximum(300);
        gainstaging_knob->setNotchTarget(60.000000000000000);
        gainstaging_knob->setNotchesVisible(true);

        gainstaging_layout->addWidget(gainstaging_knob);

        gainstaging_db_display = new QLabel(AudioExpandedView);
        gainstaging_db_display->setObjectName(QString::fromUtf8("gainstaging_db_display"));

        gainstaging_layout->addWidget(gainstaging_db_display);


        settings_layout->addLayout(gainstaging_layout);

        line_2 = new QFrame(AudioExpandedView);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        settings_layout->addWidget(line_2);

        compressor_centerer = new QHBoxLayout();
        compressor_centerer->setObjectName(QString::fromUtf8("compressor_centerer"));
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        compressor_centerer->addItem(horizontalSpacer);

        compressor_enabled = new QCheckBox(AudioExpandedView);
        compressor_enabled->setObjectName(QString::fromUtf8("compressor_enabled"));
        compressor_enabled->setChecked(true);

        compressor_centerer->addWidget(compressor_enabled);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        compressor_centerer->addItem(horizontalSpacer_2);


        settings_layout->addLayout(compressor_centerer);

        compressor_controls_layout = new QHBoxLayout();
        compressor_controls_layout->setObjectName(QString::fromUtf8("compressor_controls_layout"));
        threshold_layout = new QVBoxLayout();
        threshold_layout->setObjectName(QString::fromUtf8("threshold_layout"));
        threshold_heading = new QLabel(AudioExpandedView);
        threshold_heading->setObjectName(QString::fromUtf8("threshold_heading"));
        threshold_heading->setAlignment(Qt::AlignCenter);

        threshold_layout->addWidget(threshold_heading);

        compressor_threshold_knob_centerer = new QHBoxLayout();
        compressor_threshold_knob_centerer->setObjectName(QString::fromUtf8("compressor_threshold_knob_centerer"));
        compressor_threshold_knob = new QDial(AudioExpandedView);
        compressor_threshold_knob->setObjectName(QString::fromUtf8("compressor_threshold_knob"));
        compressor_threshold_knob->setMaximumSize(QSize(64, 64));
        compressor_threshold_knob->setMinimum(-400);
        compressor_threshold_knob->setMaximum(0);
        compressor_threshold_knob->setValue(-260);
        compressor_threshold_knob->setNotchTarget(30.000000000000000);
        compressor_threshold_knob->setNotchesVisible(true);

        compressor_threshold_knob_centerer->addWidget(compressor_threshold_knob);


        threshold_layout->addLayout(compressor_threshold_knob_centerer);

        compressor_threshold_db_display = new QLabel(AudioExpandedView);
        compressor_threshold_db_display->setObjectName(QString::fromUtf8("compressor_threshold_db_display"));
        compressor_threshold_db_display->setAlignment(Qt::AlignCenter);

        threshold_layout->addWidget(compressor_threshold_db_display);


        compressor_controls_layout->addLayout(threshold_layout);

        reduction_layout = new QVBoxLayout();
        reduction_layout->setSpacing(0);
        reduction_layout->setObjectName(QString::fromUtf8("reduction_layout"));
        reduction_header = new QLabel(AudioExpandedView);
        reduction_header->setObjectName(QString::fromUtf8("reduction_header"));
        reduction_header->setAlignment(Qt::AlignCenter);

        reduction_layout->addWidget(reduction_header);

        reduction_meter_centerer = new QHBoxLayout();
        reduction_meter_centerer->setObjectName(QString::fromUtf8("reduction_meter_centerer"));
        reduction_meter = new CompressionReductionMeter(AudioExpandedView);
        reduction_meter->setObjectName(QString::fromUtf8("reduction_meter"));
        reduction_meter->setMaximumSize(QSize(16777215, 16777215));

        reduction_meter_centerer->addWidget(reduction_meter);


        reduction_layout->addLayout(reduction_meter_centerer);

        reduction_layout->setStretch(1, 1);

        compressor_controls_layout->addLayout(reduction_layout);


        settings_layout->addLayout(compressor_controls_layout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        settings_layout->addItem(verticalSpacer);


        main_layout->addLayout(settings_layout);

        vu_layout = new QVBoxLayout();
        vu_layout->setObjectName(QString::fromUtf8("vu_layout"));
        vu_centerer = new QHBoxLayout();
        vu_centerer->setObjectName(QString::fromUtf8("vu_centerer"));
        peak_meter = new VUMeter(AudioExpandedView);
        peak_meter->setObjectName(QString::fromUtf8("peak_meter"));
        peak_meter->setMaximumSize(QSize(20, 16777215));

        vu_centerer->addWidget(peak_meter);


        vu_layout->addLayout(vu_centerer);

        peak_display_label = new ClickableLabel(AudioExpandedView);
        peak_display_label->setObjectName(QString::fromUtf8("peak_display_label"));
        peak_display_label->setMinimumSize(QSize(60, 0));
        peak_display_label->setAlignment(Qt::AlignCenter);

        vu_layout->addWidget(peak_display_label);

        vu_layout->setStretch(0, 1);

        main_layout->addLayout(vu_layout);

        fader_layout = new QVBoxLayout();
        fader_layout->setObjectName(QString::fromUtf8("fader_layout"));
        mute_centerer = new QHBoxLayout();
        mute_centerer->setSpacing(0);
        mute_centerer->setObjectName(QString::fromUtf8("mute_centerer"));
        mute_centerer->setContentsMargins(-1, -1, -1, 0);
        mute_button = new QPushButton(AudioExpandedView);
        mute_button->setObjectName(QString::fromUtf8("mute_button"));
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(mute_button->sizePolicy().hasHeightForWidth());
        mute_button->setSizePolicy(sizePolicy);
        mute_button->setMaximumSize(QSize(40, 22));
        QFont font1;
        font1.setPointSize(8);
        mute_button->setFont(font1);
        mute_button->setStyleSheet(QString::fromUtf8("QPushButton:checked { background: rgba(255,0,0,80); }"));
        mute_button->setCheckable(true);

        mute_centerer->addWidget(mute_button);


        fader_layout->addLayout(mute_centerer);

        fader_centerer = new QHBoxLayout();
        fader_centerer->setObjectName(QString::fromUtf8("fader_centerer"));
        fader = new NonLinearFader(AudioExpandedView);
        fader->setObjectName(QString::fromUtf8("fader"));
        fader->setMaximum(1000);
        fader->setSingleStep(10);
        fader->setPageStep(100);
        fader->setOrientation(Qt::Vertical);

        fader_centerer->addWidget(fader);


        fader_layout->addLayout(fader_centerer);

        fader_label = new QLabel(AudioExpandedView);
        fader_label->setObjectName(QString::fromUtf8("fader_label"));
        fader_label->setMinimumSize(QSize(60, 0));
        fader_label->setAlignment(Qt::AlignCenter);

        fader_layout->addWidget(fader_label);

        fader_layout->setStretch(1, 1);

        main_layout->addLayout(fader_layout);


        verticalLayout_6->addLayout(main_layout);


        retranslateUi(AudioExpandedView);

        QMetaObject::connectSlotsByName(AudioExpandedView);
    } // setupUi

    void retranslateUi(QWidget *AudioExpandedView)
    {
        AudioExpandedView->setWindowTitle(QCoreApplication::translate("AudioExpandedView", "AudioExpandedView", nullptr));
        bus_desc_label->setText(QCoreApplication::translate("AudioExpandedView", "Bus name", nullptr));
        stereo_width_label->setText(QCoreApplication::translate("AudioExpandedView", "Stereo: 100%", nullptr));
        locut_enabled->setText(QCoreApplication::translate("AudioExpandedView", "Lo-cut", nullptr));
        treble_label->setText(QCoreApplication::translate("AudioExpandedView", "Treble: +0.0 dB", nullptr));
        mid_label->setText(QCoreApplication::translate("AudioExpandedView", "Mid: +0.0 dB", nullptr));
        bass_label->setText(QCoreApplication::translate("AudioExpandedView", "Bass: +0.0 dB", nullptr));
        gainstaging_auto_checkbox->setText(QCoreApplication::translate("AudioExpandedView", "Auto gain staging", nullptr));
        gainstaging_db_display->setText(QCoreApplication::translate("AudioExpandedView", "Gain: +0.0 dB", nullptr));
        compressor_enabled->setText(QCoreApplication::translate("AudioExpandedView", "Compressor", nullptr));
        threshold_heading->setText(QCoreApplication::translate("AudioExpandedView", "Threshold", nullptr));
        compressor_threshold_db_display->setText(QCoreApplication::translate("AudioExpandedView", "-10.0 dB", nullptr));
        reduction_header->setText(QCoreApplication::translate("AudioExpandedView", "Reduction", nullptr));
        peak_display_label->setText(QCoreApplication::translate("AudioExpandedView", "-40.0", nullptr));
        mute_button->setText(QCoreApplication::translate("AudioExpandedView", "Mute", nullptr));
        fader_label->setText(QCoreApplication::translate("AudioExpandedView", "+0.0 dB", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AudioExpandedView: public Ui_AudioExpandedView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_AUDIO_EXPANDED_VIEW_H
