/********************************************************************************
** Form generated from reading UI file 'mainwindow.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "jpeg_frame_view.h"

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QAction *exit_action;
    QAction *manual_action;
    QAction *about_action;
    QAction *export_cliplist_clip_multitrack_action;
    QAction *export_playlist_clip_interpolated_action;
    QAction *undo_action;
    QAction *redo_action;
    QAction *quality_0_action;
    QAction *quality_1_action;
    QAction *quality_2_action;
    QAction *quality_3_action;
    QAction *quality_4_action;
    QAction *in_padding_0_action;
    QAction *in_padding_1_action;
    QAction *in_padding_2_action;
    QAction *in_padding_5_action;
    QAction *midi_mapping_action;
    QAction *out_padding_0_action;
    QAction *out_padding_1_action;
    QAction *out_padding_2_action;
    QAction *out_padding_5_action;
    QWidget *centralwidget;
    QGridLayout *gridLayout;
    QSplitter *splitter;
    QWidget *horizontalLayoutWidget;
    QVBoxLayout *clip_and_play_lists;
    QTableView *clip_list;
    QHBoxLayout *clip_list_buttons;
    QPushButton *queue_btn;
    QPushButton *preview_btn;
    QPushButton *cue_in_btn;
    QPushButton *cue_out_btn;
    QSpacerItem *horizontalSpacer_2;
    QTableView *playlist;
    QHBoxLayout *playlist_buttons;
    QPushButton *playlist_duplicate_btn;
    QPushButton *playlist_remove_btn;
    QPushButton *playlist_move_up_btn;
    QPushButton *playlist_move_down_btn;
    QSlider *speed_slider;
    QPushButton *speed_lock_btn;
    QPushButton *stop_btn;
    QPushButton *play_btn;
    QPushButton *next_btn;
    QWidget *verticalLayoutWidget_4;
    QVBoxLayout *video_displays;
    QHBoxLayout *preview_and_live_panes;
    QVBoxLayout *preview_pane;
    QFrame *preview_frame;
    QGridLayout *gridLayout_3;
    JPEGFrameView *preview_display;
    QHBoxLayout *preview_layout;
    QLabel *preview_label;
    QVBoxLayout *live_pane;
    QFrame *live_frame;
    QGridLayout *gridLayout_2;
    JPEGFrameView *live_display;
    QLabel *live_label;
    QVBoxLayout *input_pane;
    QGridLayout *input_displays;
    QLabel *label;
    QMenuBar *menuBar;
    QMenu *menuFile;
    QMenu *menu_Export;
    QMenu *interpolation_menu;
    QMenu *in_padding_menu;
    QMenu *out_padding_menu;
    QMenu *menu_Help;
    QMenu *menu_Edit;
    QMenu *view_menu;
    QMenu *hide_camera_menu;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName(QString::fromUtf8("MainWindow"));
        MainWindow->resize(1061, 600);
        exit_action = new QAction(MainWindow);
        exit_action->setObjectName(QString::fromUtf8("exit_action"));
        manual_action = new QAction(MainWindow);
        manual_action->setObjectName(QString::fromUtf8("manual_action"));
        about_action = new QAction(MainWindow);
        about_action->setObjectName(QString::fromUtf8("about_action"));
        export_cliplist_clip_multitrack_action = new QAction(MainWindow);
        export_cliplist_clip_multitrack_action->setObjectName(QString::fromUtf8("export_cliplist_clip_multitrack_action"));
        export_playlist_clip_interpolated_action = new QAction(MainWindow);
        export_playlist_clip_interpolated_action->setObjectName(QString::fromUtf8("export_playlist_clip_interpolated_action"));
        undo_action = new QAction(MainWindow);
        undo_action->setObjectName(QString::fromUtf8("undo_action"));
        redo_action = new QAction(MainWindow);
        redo_action->setObjectName(QString::fromUtf8("redo_action"));
        quality_0_action = new QAction(MainWindow);
        quality_0_action->setObjectName(QString::fromUtf8("quality_0_action"));
        quality_0_action->setCheckable(true);
        quality_1_action = new QAction(MainWindow);
        quality_1_action->setObjectName(QString::fromUtf8("quality_1_action"));
        quality_1_action->setCheckable(true);
        quality_2_action = new QAction(MainWindow);
        quality_2_action->setObjectName(QString::fromUtf8("quality_2_action"));
        quality_2_action->setCheckable(true);
        quality_3_action = new QAction(MainWindow);
        quality_3_action->setObjectName(QString::fromUtf8("quality_3_action"));
        quality_3_action->setCheckable(true);
        quality_4_action = new QAction(MainWindow);
        quality_4_action->setObjectName(QString::fromUtf8("quality_4_action"));
        quality_4_action->setCheckable(true);
        in_padding_0_action = new QAction(MainWindow);
        in_padding_0_action->setObjectName(QString::fromUtf8("in_padding_0_action"));
        in_padding_0_action->setCheckable(true);
        in_padding_1_action = new QAction(MainWindow);
        in_padding_1_action->setObjectName(QString::fromUtf8("in_padding_1_action"));
        in_padding_1_action->setCheckable(true);
        in_padding_2_action = new QAction(MainWindow);
        in_padding_2_action->setObjectName(QString::fromUtf8("in_padding_2_action"));
        in_padding_2_action->setCheckable(true);
        in_padding_5_action = new QAction(MainWindow);
        in_padding_5_action->setObjectName(QString::fromUtf8("in_padding_5_action"));
        in_padding_5_action->setCheckable(true);
        midi_mapping_action = new QAction(MainWindow);
        midi_mapping_action->setObjectName(QString::fromUtf8("midi_mapping_action"));
        out_padding_0_action = new QAction(MainWindow);
        out_padding_0_action->setObjectName(QString::fromUtf8("out_padding_0_action"));
        out_padding_0_action->setCheckable(true);
        out_padding_1_action = new QAction(MainWindow);
        out_padding_1_action->setObjectName(QString::fromUtf8("out_padding_1_action"));
        out_padding_1_action->setCheckable(true);
        out_padding_2_action = new QAction(MainWindow);
        out_padding_2_action->setObjectName(QString::fromUtf8("out_padding_2_action"));
        out_padding_2_action->setCheckable(true);
        out_padding_5_action = new QAction(MainWindow);
        out_padding_5_action->setObjectName(QString::fromUtf8("out_padding_5_action"));
        out_padding_5_action->setCheckable(true);
        centralwidget = new QWidget(MainWindow);
        centralwidget->setObjectName(QString::fromUtf8("centralwidget"));
        gridLayout = new QGridLayout(centralwidget);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        splitter = new QSplitter(centralwidget);
        splitter->setObjectName(QString::fromUtf8("splitter"));
        splitter->setOrientation(Qt::Horizontal);
        horizontalLayoutWidget = new QWidget(splitter);
        horizontalLayoutWidget->setObjectName(QString::fromUtf8("horizontalLayoutWidget"));
        clip_and_play_lists = new QVBoxLayout(horizontalLayoutWidget);
        clip_and_play_lists->setObjectName(QString::fromUtf8("clip_and_play_lists"));
        clip_and_play_lists->setContentsMargins(0, 0, 0, 0);
        clip_list = new QTableView(horizontalLayoutWidget);
        clip_list->setObjectName(QString::fromUtf8("clip_list"));

        clip_and_play_lists->addWidget(clip_list);

        clip_list_buttons = new QHBoxLayout();
        clip_list_buttons->setObjectName(QString::fromUtf8("clip_list_buttons"));
        queue_btn = new QPushButton(horizontalLayoutWidget);
        queue_btn->setObjectName(QString::fromUtf8("queue_btn"));
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        queue_btn->setIcon(icon);

        clip_list_buttons->addWidget(queue_btn);

        preview_btn = new QPushButton(horizontalLayoutWidget);
        preview_btn->setObjectName(QString::fromUtf8("preview_btn"));
        QIcon icon1;
        iconThemeName = QString::fromUtf8("media-playback-start");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        preview_btn->setIcon(icon1);

        clip_list_buttons->addWidget(preview_btn);

        cue_in_btn = new QPushButton(horizontalLayoutWidget);
        cue_in_btn->setObjectName(QString::fromUtf8("cue_in_btn"));

        clip_list_buttons->addWidget(cue_in_btn);

        cue_out_btn = new QPushButton(horizontalLayoutWidget);
        cue_out_btn->setObjectName(QString::fromUtf8("cue_out_btn"));

        clip_list_buttons->addWidget(cue_out_btn);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        clip_list_buttons->addItem(horizontalSpacer_2);


        clip_and_play_lists->addLayout(clip_list_buttons);

        playlist = new QTableView(horizontalLayoutWidget);
        playlist->setObjectName(QString::fromUtf8("playlist"));
        playlist->setSelectionMode(QAbstractItemView::ContiguousSelection);
        playlist->setSelectionBehavior(QAbstractItemView::SelectRows);

        clip_and_play_lists->addWidget(playlist);

        playlist_buttons = new QHBoxLayout();
        playlist_buttons->setObjectName(QString::fromUtf8("playlist_buttons"));
        playlist_duplicate_btn = new QPushButton(horizontalLayoutWidget);
        playlist_duplicate_btn->setObjectName(QString::fromUtf8("playlist_duplicate_btn"));
        playlist_duplicate_btn->setIcon(icon);

        playlist_buttons->addWidget(playlist_duplicate_btn);

        playlist_remove_btn = new QPushButton(horizontalLayoutWidget);
        playlist_remove_btn->setObjectName(QString::fromUtf8("playlist_remove_btn"));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(playlist_remove_btn->sizePolicy().hasHeightForWidth());
        playlist_remove_btn->setSizePolicy(sizePolicy);
        QIcon icon2;
        iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        playlist_remove_btn->setIcon(icon2);

        playlist_buttons->addWidget(playlist_remove_btn);

        playlist_move_up_btn = new QPushButton(horizontalLayoutWidget);
        playlist_move_up_btn->setObjectName(QString::fromUtf8("playlist_move_up_btn"));
        QIcon icon3;
        iconThemeName = QString::fromUtf8("go-up");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        playlist_move_up_btn->setIcon(icon3);

        playlist_buttons->addWidget(playlist_move_up_btn);

        playlist_move_down_btn = new QPushButton(horizontalLayoutWidget);
        playlist_move_down_btn->setObjectName(QString::fromUtf8("playlist_move_down_btn"));
        QIcon icon4;
        iconThemeName = QString::fromUtf8("go-down");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        playlist_move_down_btn->setIcon(icon4);

        playlist_buttons->addWidget(playlist_move_down_btn);

        speed_slider = new QSlider(horizontalLayoutWidget);
        speed_slider->setObjectName(QString::fromUtf8("speed_slider"));
        speed_slider->setEnabled(false);
        speed_slider->setMinimum(10);
        speed_slider->setMaximum(200);
        speed_slider->setValue(100);
        speed_slider->setOrientation(Qt::Horizontal);

        playlist_buttons->addWidget(speed_slider);

        speed_lock_btn = new QPushButton(horizontalLayoutWidget);
        speed_lock_btn->setObjectName(QString::fromUtf8("speed_lock_btn"));
        sizePolicy.setHeightForWidth(speed_lock_btn->sizePolicy().hasHeightForWidth());
        speed_lock_btn->setSizePolicy(sizePolicy);
        speed_lock_btn->setMinimumSize(QSize(72, 0));
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/lock.svg"), QSize(), QIcon::Normal, QIcon::On);
        speed_lock_btn->setIcon(icon5);
        speed_lock_btn->setCheckable(true);
        speed_lock_btn->setChecked(true);

        playlist_buttons->addWidget(speed_lock_btn);

        stop_btn = new QPushButton(horizontalLayoutWidget);
        stop_btn->setObjectName(QString::fromUtf8("stop_btn"));
        QIcon icon6;
        iconThemeName = QString::fromUtf8("media-playback-stop");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon6 = QIcon::fromTheme(iconThemeName);
        } else {
            icon6.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        stop_btn->setIcon(icon6);

        playlist_buttons->addWidget(stop_btn);

        play_btn = new QPushButton(horizontalLayoutWidget);
        play_btn->setObjectName(QString::fromUtf8("play_btn"));
        play_btn->setIcon(icon1);

        playlist_buttons->addWidget(play_btn);

        next_btn = new QPushButton(horizontalLayoutWidget);
        next_btn->setObjectName(QString::fromUtf8("next_btn"));
        QIcon icon7;
        iconThemeName = QString::fromUtf8("media-skip-forward");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon7 = QIcon::fromTheme(iconThemeName);
        } else {
            icon7.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        next_btn->setIcon(icon7);

        playlist_buttons->addWidget(next_btn);


        clip_and_play_lists->addLayout(playlist_buttons);

        splitter->addWidget(horizontalLayoutWidget);
        verticalLayoutWidget_4 = new QWidget(splitter);
        verticalLayoutWidget_4->setObjectName(QString::fromUtf8("verticalLayoutWidget_4"));
        video_displays = new QVBoxLayout(verticalLayoutWidget_4);
        video_displays->setObjectName(QString::fromUtf8("video_displays"));
        video_displays->setContentsMargins(0, 0, 0, 0);
        preview_and_live_panes = new QHBoxLayout();
        preview_and_live_panes->setSpacing(3);
        preview_and_live_panes->setObjectName(QString::fromUtf8("preview_and_live_panes"));
        preview_pane = new QVBoxLayout();
        preview_pane->setObjectName(QString::fromUtf8("preview_pane"));
        preview_frame = new QFrame(verticalLayoutWidget_4);
        preview_frame->setObjectName(QString::fromUtf8("preview_frame"));
        preview_frame->setAutoFillBackground(true);
        preview_frame->setFrameShape(QFrame::NoFrame);
        preview_frame->setFrameShadow(QFrame::Plain);
        preview_frame->setLineWidth(0);
        gridLayout_3 = new QGridLayout(preview_frame);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        gridLayout_3->setContentsMargins(3, 3, 3, 3);
        preview_display = new JPEGFrameView(preview_frame);
        preview_display->setObjectName(QString::fromUtf8("preview_display"));

        gridLayout_3->addWidget(preview_display, 0, 0, 1, 1);


        preview_pane->addWidget(preview_frame);

        preview_layout = new QHBoxLayout();
        preview_layout->setObjectName(QString::fromUtf8("preview_layout"));
        preview_label = new QLabel(verticalLayoutWidget_4);
        preview_label->setObjectName(QString::fromUtf8("preview_label"));
        preview_label->setAlignment(Qt::AlignCenter);

        preview_layout->addWidget(preview_label);


        preview_pane->addLayout(preview_layout);

        preview_pane->setStretch(0, 1);

        preview_and_live_panes->addLayout(preview_pane);

        live_pane = new QVBoxLayout();
        live_pane->setObjectName(QString::fromUtf8("live_pane"));
        live_frame = new QFrame(verticalLayoutWidget_4);
        live_frame->setObjectName(QString::fromUtf8("live_frame"));
        live_frame->setAutoFillBackground(true);
        live_frame->setFrameShape(QFrame::NoFrame);
        live_frame->setFrameShadow(QFrame::Plain);
        live_frame->setLineWidth(0);
        gridLayout_2 = new QGridLayout(live_frame);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(3, 3, 3, 3);
        live_display = new JPEGFrameView(live_frame);
        live_display->setObjectName(QString::fromUtf8("live_display"));

        gridLayout_2->addWidget(live_display, 0, 0, 1, 1);


        live_pane->addWidget(live_frame);

        live_label = new QLabel(verticalLayoutWidget_4);
        live_label->setObjectName(QString::fromUtf8("live_label"));
        live_label->setAlignment(Qt::AlignCenter);

        live_pane->addWidget(live_label);

        live_pane->setStretch(0, 1);

        preview_and_live_panes->addLayout(live_pane);


        video_displays->addLayout(preview_and_live_panes);

        input_pane = new QVBoxLayout();
        input_pane->setObjectName(QString::fromUtf8("input_pane"));
        input_displays = new QGridLayout();
        input_displays->setSpacing(0);
        input_displays->setObjectName(QString::fromUtf8("input_displays"));

        input_pane->addLayout(input_displays);

        label = new QLabel(verticalLayoutWidget_4);
        label->setObjectName(QString::fromUtf8("label"));
        label->setAlignment(Qt::AlignCenter);

        input_pane->addWidget(label);

        input_pane->setStretch(0, 1);

        video_displays->addLayout(input_pane);

        video_displays->setStretch(0, 1);
        video_displays->setStretch(1, 2);
        splitter->addWidget(verticalLayoutWidget_4);

        gridLayout->addWidget(splitter, 0, 0, 1, 1);

        MainWindow->setCentralWidget(centralwidget);
        menuBar = new QMenuBar(MainWindow);
        menuBar->setObjectName(QString::fromUtf8("menuBar"));
        menuBar->setGeometry(QRect(0, 0, 1061, 22));
        menuFile = new QMenu(menuBar);
        menuFile->setObjectName(QString::fromUtf8("menuFile"));
        menu_Export = new QMenu(menuFile);
        menu_Export->setObjectName(QString::fromUtf8("menu_Export"));
        interpolation_menu = new QMenu(menuFile);
        interpolation_menu->setObjectName(QString::fromUtf8("interpolation_menu"));
        in_padding_menu = new QMenu(menuFile);
        in_padding_menu->setObjectName(QString::fromUtf8("in_padding_menu"));
        out_padding_menu = new QMenu(menuFile);
        out_padding_menu->setObjectName(QString::fromUtf8("out_padding_menu"));
        menu_Help = new QMenu(menuBar);
        menu_Help->setObjectName(QString::fromUtf8("menu_Help"));
        menu_Edit = new QMenu(menuBar);
        menu_Edit->setObjectName(QString::fromUtf8("menu_Edit"));
        view_menu = new QMenu(menuBar);
        view_menu->setObjectName(QString::fromUtf8("view_menu"));
        hide_camera_menu = new QMenu(view_menu);
        hide_camera_menu->setObjectName(QString::fromUtf8("hide_camera_menu"));
        MainWindow->setMenuBar(menuBar);

        menuBar->addAction(menuFile->menuAction());
        menuBar->addAction(menu_Edit->menuAction());
        menuBar->addAction(view_menu->menuAction());
        menuBar->addAction(menu_Help->menuAction());
        menuFile->addAction(interpolation_menu->menuAction());
        menuFile->addAction(in_padding_menu->menuAction());
        menuFile->addAction(out_padding_menu->menuAction());
        menuFile->addAction(menu_Export->menuAction());
        menuFile->addAction(midi_mapping_action);
        menuFile->addAction(exit_action);
        menuFile->addSeparator();
        menu_Export->addAction(export_cliplist_clip_multitrack_action);
        menu_Export->addAction(export_playlist_clip_interpolated_action);
        interpolation_menu->addAction(quality_0_action);
        interpolation_menu->addAction(quality_1_action);
        interpolation_menu->addAction(quality_2_action);
        interpolation_menu->addAction(quality_3_action);
        interpolation_menu->addAction(quality_4_action);
        in_padding_menu->addAction(in_padding_0_action);
        in_padding_menu->addAction(in_padding_1_action);
        in_padding_menu->addAction(in_padding_2_action);
        in_padding_menu->addAction(in_padding_5_action);
        out_padding_menu->addAction(out_padding_0_action);
        out_padding_menu->addAction(out_padding_1_action);
        out_padding_menu->addAction(out_padding_2_action);
        out_padding_menu->addAction(out_padding_5_action);
        menu_Help->addAction(manual_action);
        menu_Help->addAction(about_action);
        menu_Edit->addAction(undo_action);
        menu_Edit->addAction(redo_action);
        view_menu->addAction(hide_camera_menu->menuAction());

        retranslateUi(MainWindow);

        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        MainWindow->setWindowTitle(QCoreApplication::translate("MainWindow", "Futatabi", nullptr));
        exit_action->setText(QCoreApplication::translate("MainWindow", "E&xit", nullptr));
        manual_action->setText(QCoreApplication::translate("MainWindow", "Online &manual\342\200\246", nullptr));
        about_action->setText(QCoreApplication::translate("MainWindow", "&About Futatabi\342\200\246", nullptr));
        export_cliplist_clip_multitrack_action->setText(QCoreApplication::translate("MainWindow", "Selected clip list clip as raw &multitrack\342\200\246", nullptr));
        export_playlist_clip_interpolated_action->setText(QCoreApplication::translate("MainWindow", "Selected playlist clip(s) as &interpolated single track\342\200\246", nullptr));
        undo_action->setText(QCoreApplication::translate("MainWindow", "&Undo", nullptr));
#if QT_CONFIG(shortcut)
        undo_action->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Z", nullptr));
#endif // QT_CONFIG(shortcut)
        redo_action->setText(QCoreApplication::translate("MainWindow", "&Redo", nullptr));
#if QT_CONFIG(shortcut)
        redo_action->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Y", nullptr));
#endif // QT_CONFIG(shortcut)
        quality_0_action->setText(QCoreApplication::translate("MainWindow", "No interpolation (&0)", nullptr));
        quality_1_action->setText(QCoreApplication::translate("MainWindow", "Fastest (&1)", nullptr));
        quality_2_action->setText(QCoreApplication::translate("MainWindow", "Default (&2) (realtime 720p on fast embedded GPUs)", nullptr));
        quality_3_action->setText(QCoreApplication::translate("MainWindow", "Good (&3) (realtime 720p on GTX 970 or so)", nullptr));
        quality_4_action->setText(QCoreApplication::translate("MainWindow", "Best (&4) (not realtime on any current GPU)", nullptr));
        in_padding_0_action->setText(QCoreApplication::translate("MainWindow", "&0 seconds", nullptr));
        in_padding_1_action->setText(QCoreApplication::translate("MainWindow", "&1 second", nullptr));
        in_padding_2_action->setText(QCoreApplication::translate("MainWindow", "&2 seconds", nullptr));
        in_padding_5_action->setText(QCoreApplication::translate("MainWindow", "&5 seconds", nullptr));
        midi_mapping_action->setText(QCoreApplication::translate("MainWindow", "Setup MIDI controller\342\200\246", nullptr));
        out_padding_0_action->setText(QCoreApplication::translate("MainWindow", "&0 seconds", nullptr));
        out_padding_1_action->setText(QCoreApplication::translate("MainWindow", "&1 seconds", nullptr));
        out_padding_2_action->setText(QCoreApplication::translate("MainWindow", "&2 seconds", nullptr));
        out_padding_5_action->setText(QCoreApplication::translate("MainWindow", "&5 seconds", nullptr));
        queue_btn->setText(QCoreApplication::translate("MainWindow", "Queue (&Q)", nullptr));
        preview_btn->setText(QCoreApplication::translate("MainWindow", "Preview (&W)", nullptr));
        cue_in_btn->setText(QCoreApplication::translate("MainWindow", "Cue in (&A)", nullptr));
        cue_out_btn->setText(QCoreApplication::translate("MainWindow", "Cue out (&S)", nullptr));
        playlist_duplicate_btn->setText(QCoreApplication::translate("MainWindow", "Duplicate", nullptr));
        playlist_remove_btn->setText(QCoreApplication::translate("MainWindow", "Remove", nullptr));
        playlist_move_up_btn->setText(QCoreApplication::translate("MainWindow", "Move up", nullptr));
        playlist_move_down_btn->setText(QCoreApplication::translate("MainWindow", "Move down", nullptr));
        speed_lock_btn->setText(QCoreApplication::translate("MainWindow", " 100%", nullptr));
        stop_btn->setText(QCoreApplication::translate("MainWindow", "Stop", nullptr));
        play_btn->setText(QCoreApplication::translate("MainWindow", "Play (space)", nullptr));
        next_btn->setText(QCoreApplication::translate("MainWindow", "Next (N)", nullptr));
        preview_label->setText(QCoreApplication::translate("MainWindow", "Preview output", nullptr));
        live_label->setText(QCoreApplication::translate("MainWindow", "Current output (paused)", nullptr));
        label->setText(QCoreApplication::translate("MainWindow", "Current inputs", nullptr));
        menuFile->setTitle(QCoreApplication::translate("MainWindow", "&Video", nullptr));
        menu_Export->setTitle(QCoreApplication::translate("MainWindow", "&Export", nullptr));
        interpolation_menu->setTitle(QCoreApplication::translate("MainWindow", "Interpolation &quality", nullptr));
        in_padding_menu->setTitle(QCoreApplication::translate("MainWindow", "Cue &in point padding", nullptr));
        out_padding_menu->setTitle(QCoreApplication::translate("MainWindow", "Cue &out point padding", nullptr));
        menu_Help->setTitle(QCoreApplication::translate("MainWindow", "&Help", nullptr));
        menu_Edit->setTitle(QCoreApplication::translate("MainWindow", "&Edit", nullptr));
        view_menu->setTitle(QCoreApplication::translate("MainWindow", "V&iew", nullptr));
        hide_camera_menu->setTitle(QCoreApplication::translate("MainWindow", "&Hide camera", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINDOW_H
