/********************************************************************************
** Form generated from reading UI file 'input_mapping.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INPUT_MAPPING_H
#define UI_INPUT_MAPPING_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_InputMappingDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QTableWidget *table;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *add_button;
    QPushButton *remove_button;
    QPushButton *up_button;
    QPushButton *down_button;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *save_button;
    QPushButton *load_button;
    QSpacerItem *horizontalSpacer;
    QDialogButtonBox *ok_cancel_buttons;

    void setupUi(QDialog *InputMappingDialog)
    {
        if (InputMappingDialog->objectName().isEmpty())
            InputMappingDialog->setObjectName(QString::fromUtf8("InputMappingDialog"));
        InputMappingDialog->resize(879, 583);
        verticalLayout_2 = new QVBoxLayout(InputMappingDialog);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        table = new QTableWidget(InputMappingDialog);
        if (table->columnCount() < 4)
            table->setColumnCount(4);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        table->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        table->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        table->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        table->setHorizontalHeaderItem(3, __qtablewidgetitem3);
        table->setObjectName(QString::fromUtf8("table"));

        verticalLayout_2->addWidget(table);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        add_button = new QPushButton(InputMappingDialog);
        add_button->setObjectName(QString::fromUtf8("add_button"));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(add_button->sizePolicy().hasHeightForWidth());
        add_button->setSizePolicy(sizePolicy);
        add_button->setMaximumSize(QSize(30, 30));
        QIcon icon;
        QString iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon = QIcon::fromTheme(iconThemeName);
        } else {
            icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        add_button->setIcon(icon);

        horizontalLayout_2->addWidget(add_button);

        remove_button = new QPushButton(InputMappingDialog);
        remove_button->setObjectName(QString::fromUtf8("remove_button"));
        sizePolicy.setHeightForWidth(remove_button->sizePolicy().hasHeightForWidth());
        remove_button->setSizePolicy(sizePolicy);
        remove_button->setMaximumSize(QSize(30, 30));
        QIcon icon1;
        iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        remove_button->setIcon(icon1);

        horizontalLayout_2->addWidget(remove_button);

        up_button = new QPushButton(InputMappingDialog);
        up_button->setObjectName(QString::fromUtf8("up_button"));
        up_button->setMaximumSize(QSize(30, 30));
        QIcon icon2;
        iconThemeName = QString::fromUtf8("go-up");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        up_button->setIcon(icon2);

        horizontalLayout_2->addWidget(up_button);

        down_button = new QPushButton(InputMappingDialog);
        down_button->setObjectName(QString::fromUtf8("down_button"));
        down_button->setMaximumSize(QSize(30, 30));
        QIcon icon3;
        iconThemeName = QString::fromUtf8("go-down");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        down_button->setIcon(icon3);

        horizontalLayout_2->addWidget(down_button);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        save_button = new QPushButton(InputMappingDialog);
        save_button->setObjectName(QString::fromUtf8("save_button"));

        horizontalLayout_2->addWidget(save_button);

        load_button = new QPushButton(InputMappingDialog);
        load_button->setObjectName(QString::fromUtf8("load_button"));

        horizontalLayout_2->addWidget(load_button);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        ok_cancel_buttons = new QDialogButtonBox(InputMappingDialog);
        ok_cancel_buttons->setObjectName(QString::fromUtf8("ok_cancel_buttons"));
        ok_cancel_buttons->setOrientation(Qt::Horizontal);
        ok_cancel_buttons->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        horizontalLayout_2->addWidget(ok_cancel_buttons);

        horizontalLayout_2->setStretch(4, 1);
        horizontalLayout_2->setStretch(7, 1);

        verticalLayout_2->addLayout(horizontalLayout_2);


        retranslateUi(InputMappingDialog);

        QMetaObject::connectSlotsByName(InputMappingDialog);
    } // setupUi

    void retranslateUi(QDialog *InputMappingDialog)
    {
        InputMappingDialog->setWindowTitle(QCoreApplication::translate("InputMappingDialog", "Input mapping", nullptr));
        QTableWidgetItem *___qtablewidgetitem = table->horizontalHeaderItem(1);
        ___qtablewidgetitem->setText(QCoreApplication::translate("InputMappingDialog", "Device", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = table->horizontalHeaderItem(2);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("InputMappingDialog", "Left input", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = table->horizontalHeaderItem(3);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("InputMappingDialog", "Right input", nullptr));
        add_button->setText(QString());
        remove_button->setText(QString());
        up_button->setText(QString());
        down_button->setText(QString());
        save_button->setText(QCoreApplication::translate("InputMappingDialog", "&Save\342\200\246", nullptr));
        load_button->setText(QCoreApplication::translate("InputMappingDialog", "&Load\342\200\246", nullptr));
    } // retranslateUi

};

namespace Ui {
    class InputMappingDialog: public Ui_InputMappingDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_INPUT_MAPPING_H
