/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../nageru-2.3.2/nageru/mainwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MainWindow_t {
    QByteArrayData data[101];
    char stringdata0[1736];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MainWindow_t qt_meta_stringdata_MainWindow = {
    {
QT_MOC_LITERAL(0, 0, 10), // "MainWindow"
QT_MOC_LITERAL(1, 11, 13), // "cut_triggered"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 22), // "x264_bitrate_triggered"
QT_MOC_LITERAL(4, 49, 14), // "exit_triggered"
QT_MOC_LITERAL(5, 64, 16), // "manual_triggered"
QT_MOC_LITERAL(6, 81, 15), // "about_triggered"
QT_MOC_LITERAL(7, 97, 23), // "open_analyzer_triggered"
QT_MOC_LITERAL(8, 121, 27), // "simple_audio_mode_triggered"
QT_MOC_LITERAL(9, 149, 33), // "multichannel_audio_mode_trigg..."
QT_MOC_LITERAL(10, 183, 23), // "input_mapping_triggered"
QT_MOC_LITERAL(11, 207, 22), // "midi_mapping_triggered"
QT_MOC_LITERAL(12, 230, 25), // "timecode_stream_triggered"
QT_MOC_LITERAL(13, 256, 25), // "timecode_stdout_triggered"
QT_MOC_LITERAL(14, 282, 18), // "transition_clicked"
QT_MOC_LITERAL(15, 301, 17), // "transition_number"
QT_MOC_LITERAL(16, 319, 15), // "channel_clicked"
QT_MOC_LITERAL(17, 335, 14), // "channel_number"
QT_MOC_LITERAL(18, 350, 19), // "quick_cut_activated"
QT_MOC_LITERAL(19, 370, 17), // "wb_button_clicked"
QT_MOC_LITERAL(20, 388, 18), // "audio_view_changed"
QT_MOC_LITERAL(21, 407, 10), // "audio_view"
QT_MOC_LITERAL(22, 418, 20), // "set_transition_names"
QT_MOC_LITERAL(23, 439, 24), // "std::vector<std::string>"
QT_MOC_LITERAL(24, 464, 16), // "transition_names"
QT_MOC_LITERAL(25, 481, 19), // "update_channel_name"
QT_MOC_LITERAL(26, 501, 13), // "Mixer::Output"
QT_MOC_LITERAL(27, 515, 6), // "output"
QT_MOC_LITERAL(28, 522, 11), // "std::string"
QT_MOC_LITERAL(29, 534, 4), // "name"
QT_MOC_LITERAL(30, 539, 20), // "update_channel_color"
QT_MOC_LITERAL(31, 560, 5), // "color"
QT_MOC_LITERAL(32, 566, 25), // "gain_staging_knob_changed"
QT_MOC_LITERAL(33, 592, 9), // "bus_index"
QT_MOC_LITERAL(34, 602, 5), // "value"
QT_MOC_LITERAL(35, 608, 30), // "final_makeup_gain_knob_changed"
QT_MOC_LITERAL(36, 639, 19), // "cutoff_knob_changed"
QT_MOC_LITERAL(37, 659, 25), // "stereo_width_knob_changed"
QT_MOC_LITERAL(38, 685, 15), // "eq_knob_changed"
QT_MOC_LITERAL(39, 701, 6), // "EQBand"
QT_MOC_LITERAL(40, 708, 4), // "band"
QT_MOC_LITERAL(41, 713, 30), // "limiter_threshold_knob_changed"
QT_MOC_LITERAL(42, 744, 33), // "compressor_threshold_knob_cha..."
QT_MOC_LITERAL(43, 778, 18), // "mini_fader_changed"
QT_MOC_LITERAL(44, 797, 3), // "bus"
QT_MOC_LITERAL(45, 801, 9), // "db_volume"
QT_MOC_LITERAL(46, 811, 19), // "mute_button_toggled"
QT_MOC_LITERAL(47, 831, 7), // "checked"
QT_MOC_LITERAL(48, 839, 27), // "reset_meters_button_clicked"
QT_MOC_LITERAL(49, 867, 8), // "relayout"
QT_MOC_LITERAL(50, 876, 9), // "set_locut"
QT_MOC_LITERAL(51, 886, 21), // "set_limiter_threshold"
QT_MOC_LITERAL(52, 908, 15), // "set_makeup_gain"
QT_MOC_LITERAL(53, 924, 16), // "set_stereo_width"
QT_MOC_LITERAL(54, 941, 7), // "bus_idx"
QT_MOC_LITERAL(55, 949, 10), // "set_treble"
QT_MOC_LITERAL(56, 960, 7), // "set_mid"
QT_MOC_LITERAL(57, 968, 8), // "set_bass"
QT_MOC_LITERAL(58, 977, 15), // "set_eq_absolute"
QT_MOC_LITERAL(59, 993, 7), // "eq_band"
QT_MOC_LITERAL(60, 1001, 8), // "value_db"
QT_MOC_LITERAL(61, 1010, 8), // "set_gain"
QT_MOC_LITERAL(62, 1019, 24), // "set_compressor_threshold"
QT_MOC_LITERAL(63, 1044, 9), // "set_fader"
QT_MOC_LITERAL(64, 1054, 18), // "set_fader_absolute"
QT_MOC_LITERAL(65, 1073, 11), // "toggle_mute"
QT_MOC_LITERAL(66, 1085, 12), // "toggle_locut"
QT_MOC_LITERAL(67, 1098, 24), // "toggle_auto_gain_staging"
QT_MOC_LITERAL(68, 1123, 17), // "toggle_compressor"
QT_MOC_LITERAL(69, 1141, 10), // "clear_peak"
QT_MOC_LITERAL(70, 1152, 14), // "toggle_limiter"
QT_MOC_LITERAL(71, 1167, 23), // "toggle_auto_makeup_gain"
QT_MOC_LITERAL(72, 1191, 20), // "switch_video_channel"
QT_MOC_LITERAL(73, 1212, 16), // "apply_transition"
QT_MOC_LITERAL(74, 1229, 15), // "prev_audio_view"
QT_MOC_LITERAL(75, 1245, 15), // "next_audio_view"
QT_MOC_LITERAL(76, 1261, 17), // "begin_new_segment"
QT_MOC_LITERAL(77, 1279, 4), // "exit"
QT_MOC_LITERAL(78, 1284, 20), // "clear_all_highlights"
QT_MOC_LITERAL(79, 1305, 15), // "highlight_locut"
QT_MOC_LITERAL(80, 1321, 9), // "highlight"
QT_MOC_LITERAL(81, 1331, 27), // "highlight_limiter_threshold"
QT_MOC_LITERAL(82, 1359, 21), // "highlight_makeup_gain"
QT_MOC_LITERAL(83, 1381, 22), // "highlight_stereo_width"
QT_MOC_LITERAL(84, 1404, 16), // "highlight_treble"
QT_MOC_LITERAL(85, 1421, 13), // "highlight_mid"
QT_MOC_LITERAL(86, 1435, 14), // "highlight_bass"
QT_MOC_LITERAL(87, 1450, 14), // "highlight_gain"
QT_MOC_LITERAL(88, 1465, 30), // "highlight_compressor_threshold"
QT_MOC_LITERAL(89, 1496, 15), // "highlight_fader"
QT_MOC_LITERAL(90, 1512, 14), // "highlight_mute"
QT_MOC_LITERAL(91, 1527, 22), // "highlight_toggle_locut"
QT_MOC_LITERAL(92, 1550, 34), // "highlight_toggle_auto_gain_st..."
QT_MOC_LITERAL(93, 1585, 27), // "highlight_toggle_compressor"
QT_MOC_LITERAL(94, 1613, 20), // "highlight_clear_peak"
QT_MOC_LITERAL(95, 1634, 24), // "highlight_toggle_limiter"
QT_MOC_LITERAL(96, 1659, 33), // "highlight_toggle_auto_makeup_..."
QT_MOC_LITERAL(97, 1693, 18), // "controller_changed"
QT_MOC_LITERAL(98, 1712, 10), // "controller"
QT_MOC_LITERAL(99, 1723, 7), // "note_on"
QT_MOC_LITERAL(100, 1731, 4) // "note"

    },
    "MainWindow\0cut_triggered\0\0"
    "x264_bitrate_triggered\0exit_triggered\0"
    "manual_triggered\0about_triggered\0"
    "open_analyzer_triggered\0"
    "simple_audio_mode_triggered\0"
    "multichannel_audio_mode_triggered\0"
    "input_mapping_triggered\0midi_mapping_triggered\0"
    "timecode_stream_triggered\0"
    "timecode_stdout_triggered\0transition_clicked\0"
    "transition_number\0channel_clicked\0"
    "channel_number\0quick_cut_activated\0"
    "wb_button_clicked\0audio_view_changed\0"
    "audio_view\0set_transition_names\0"
    "std::vector<std::string>\0transition_names\0"
    "update_channel_name\0Mixer::Output\0"
    "output\0std::string\0name\0update_channel_color\0"
    "color\0gain_staging_knob_changed\0"
    "bus_index\0value\0final_makeup_gain_knob_changed\0"
    "cutoff_knob_changed\0stereo_width_knob_changed\0"
    "eq_knob_changed\0EQBand\0band\0"
    "limiter_threshold_knob_changed\0"
    "compressor_threshold_knob_changed\0"
    "mini_fader_changed\0bus\0db_volume\0"
    "mute_button_toggled\0checked\0"
    "reset_meters_button_clicked\0relayout\0"
    "set_locut\0set_limiter_threshold\0"
    "set_makeup_gain\0set_stereo_width\0"
    "bus_idx\0set_treble\0set_mid\0set_bass\0"
    "set_eq_absolute\0eq_band\0value_db\0"
    "set_gain\0set_compressor_threshold\0"
    "set_fader\0set_fader_absolute\0toggle_mute\0"
    "toggle_locut\0toggle_auto_gain_staging\0"
    "toggle_compressor\0clear_peak\0"
    "toggle_limiter\0toggle_auto_makeup_gain\0"
    "switch_video_channel\0apply_transition\0"
    "prev_audio_view\0next_audio_view\0"
    "begin_new_segment\0exit\0clear_all_highlights\0"
    "highlight_locut\0highlight\0"
    "highlight_limiter_threshold\0"
    "highlight_makeup_gain\0highlight_stereo_width\0"
    "highlight_treble\0highlight_mid\0"
    "highlight_bass\0highlight_gain\0"
    "highlight_compressor_threshold\0"
    "highlight_fader\0highlight_mute\0"
    "highlight_toggle_locut\0"
    "highlight_toggle_auto_gain_staging\0"
    "highlight_toggle_compressor\0"
    "highlight_clear_peak\0highlight_toggle_limiter\0"
    "highlight_toggle_auto_makeup_gain\0"
    "controller_changed\0controller\0note_on\0"
    "note"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MainWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      76,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  394,    2, 0x0a /* Public */,
       3,    0,  395,    2, 0x0a /* Public */,
       4,    0,  396,    2, 0x0a /* Public */,
       5,    0,  397,    2, 0x0a /* Public */,
       6,    0,  398,    2, 0x0a /* Public */,
       7,    0,  399,    2, 0x0a /* Public */,
       8,    0,  400,    2, 0x0a /* Public */,
       9,    0,  401,    2, 0x0a /* Public */,
      10,    0,  402,    2, 0x0a /* Public */,
      11,    0,  403,    2, 0x0a /* Public */,
      12,    0,  404,    2, 0x0a /* Public */,
      13,    0,  405,    2, 0x0a /* Public */,
      14,    1,  406,    2, 0x0a /* Public */,
      16,    1,  409,    2, 0x0a /* Public */,
      18,    1,  412,    2, 0x0a /* Public */,
      19,    1,  415,    2, 0x0a /* Public */,
      20,    1,  418,    2, 0x0a /* Public */,
      22,    1,  421,    2, 0x0a /* Public */,
      25,    2,  424,    2, 0x0a /* Public */,
      30,    2,  429,    2, 0x0a /* Public */,
      32,    2,  434,    2, 0x0a /* Public */,
      35,    1,  439,    2, 0x0a /* Public */,
      36,    1,  442,    2, 0x0a /* Public */,
      37,    2,  445,    2, 0x0a /* Public */,
      38,    3,  450,    2, 0x0a /* Public */,
      41,    1,  457,    2, 0x0a /* Public */,
      42,    2,  460,    2, 0x0a /* Public */,
      43,    2,  465,    2, 0x0a /* Public */,
      46,    2,  470,    2, 0x0a /* Public */,
      48,    0,  475,    2, 0x0a /* Public */,
      49,    0,  476,    2, 0x0a /* Public */,
      50,    1,  477,    2, 0x0a /* Public */,
      51,    1,  480,    2, 0x0a /* Public */,
      52,    1,  483,    2, 0x0a /* Public */,
      53,    2,  486,    2, 0x0a /* Public */,
      55,    2,  491,    2, 0x0a /* Public */,
      56,    2,  496,    2, 0x0a /* Public */,
      57,    2,  501,    2, 0x0a /* Public */,
      58,    3,  506,    2, 0x0a /* Public */,
      61,    2,  513,    2, 0x0a /* Public */,
      62,    2,  518,    2, 0x0a /* Public */,
      63,    2,  523,    2, 0x0a /* Public */,
      64,    2,  528,    2, 0x0a /* Public */,
      65,    1,  533,    2, 0x0a /* Public */,
      66,    1,  536,    2, 0x0a /* Public */,
      67,    1,  539,    2, 0x0a /* Public */,
      68,    1,  542,    2, 0x0a /* Public */,
      69,    1,  545,    2, 0x0a /* Public */,
      70,    0,  548,    2, 0x0a /* Public */,
      71,    0,  549,    2, 0x0a /* Public */,
      72,    1,  550,    2, 0x0a /* Public */,
      73,    1,  553,    2, 0x0a /* Public */,
      74,    0,  556,    2, 0x0a /* Public */,
      75,    0,  557,    2, 0x0a /* Public */,
      76,    0,  558,    2, 0x0a /* Public */,
      77,    0,  559,    2, 0x0a /* Public */,
      78,    0,  560,    2, 0x0a /* Public */,
      79,    1,  561,    2, 0x0a /* Public */,
      81,    1,  564,    2, 0x0a /* Public */,
      82,    1,  567,    2, 0x0a /* Public */,
      83,    2,  570,    2, 0x0a /* Public */,
      84,    2,  575,    2, 0x0a /* Public */,
      85,    2,  580,    2, 0x0a /* Public */,
      86,    2,  585,    2, 0x0a /* Public */,
      87,    2,  590,    2, 0x0a /* Public */,
      88,    2,  595,    2, 0x0a /* Public */,
      89,    2,  600,    2, 0x0a /* Public */,
      90,    2,  605,    2, 0x0a /* Public */,
      91,    2,  610,    2, 0x0a /* Public */,
      92,    2,  615,    2, 0x0a /* Public */,
      93,    2,  620,    2, 0x0a /* Public */,
      94,    2,  625,    2, 0x0a /* Public */,
      95,    1,  630,    2, 0x0a /* Public */,
      96,    1,  633,    2, 0x0a /* Public */,
      97,    1,  636,    2, 0x0a /* Public */,
      99,    1,  639,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Void, 0x80000000 | 23,   24,
    QMetaType::Void, 0x80000000 | 26, 0x80000000 | 28,   27,   29,
    QMetaType::Void, 0x80000000 | 26, 0x80000000 | 28,   27,   31,
    QMetaType::Void, QMetaType::UInt, QMetaType::Int,   33,   34,
    QMetaType::Void, QMetaType::Int,   34,
    QMetaType::Void, QMetaType::Int,   34,
    QMetaType::Void, QMetaType::UInt, QMetaType::Int,   33,   34,
    QMetaType::Void, QMetaType::UInt, 0x80000000 | 39, QMetaType::Int,   33,   40,   34,
    QMetaType::Void, QMetaType::Int,   34,
    QMetaType::Void, QMetaType::UInt, QMetaType::Int,   33,   34,
    QMetaType::Void, QMetaType::Int, QMetaType::Double,   44,   45,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,   44,   47,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Float,   34,
    QMetaType::Void, QMetaType::Float,   34,
    QMetaType::Void, QMetaType::Float,   34,
    QMetaType::Void, QMetaType::UInt, QMetaType::Float,   54,   34,
    QMetaType::Void, QMetaType::UInt, QMetaType::Float,   54,   34,
    QMetaType::Void, QMetaType::UInt, QMetaType::Float,   54,   34,
    QMetaType::Void, QMetaType::UInt, QMetaType::Float,   54,   34,
    QMetaType::Void, QMetaType::UInt, 0x80000000 | 39, QMetaType::Float,   54,   59,   60,
    QMetaType::Void, QMetaType::UInt, QMetaType::Float,   54,   34,
    QMetaType::Void, QMetaType::UInt, QMetaType::Float,   54,   34,
    QMetaType::Void, QMetaType::UInt, QMetaType::Float,   54,   34,
    QMetaType::Void, QMetaType::UInt, QMetaType::Float,   54,   60,
    QMetaType::Void, QMetaType::UInt,   54,
    QMetaType::Void, QMetaType::UInt,   54,
    QMetaType::Void, QMetaType::UInt,   54,
    QMetaType::Void, QMetaType::UInt,   54,
    QMetaType::Void, QMetaType::UInt,   54,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   80,
    QMetaType::Void, QMetaType::Bool,   80,
    QMetaType::Void, QMetaType::Bool,   80,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,   54,   80,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,   54,   80,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,   54,   80,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,   54,   80,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,   54,   80,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,   54,   80,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,   54,   80,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,   54,   80,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,   54,   80,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,   54,   80,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,   54,   80,
    QMetaType::Void, QMetaType::UInt, QMetaType::Bool,   54,   80,
    QMetaType::Void, QMetaType::Bool,   80,
    QMetaType::Void, QMetaType::Bool,   80,
    QMetaType::Void, QMetaType::UInt,   98,
    QMetaType::Void, QMetaType::UInt,  100,

       0        // eod
};

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MainWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cut_triggered(); break;
        case 1: _t->x264_bitrate_triggered(); break;
        case 2: _t->exit_triggered(); break;
        case 3: _t->manual_triggered(); break;
        case 4: _t->about_triggered(); break;
        case 5: _t->open_analyzer_triggered(); break;
        case 6: _t->simple_audio_mode_triggered(); break;
        case 7: _t->multichannel_audio_mode_triggered(); break;
        case 8: _t->input_mapping_triggered(); break;
        case 9: _t->midi_mapping_triggered(); break;
        case 10: _t->timecode_stream_triggered(); break;
        case 11: _t->timecode_stdout_triggered(); break;
        case 12: _t->transition_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->channel_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->quick_cut_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->wb_button_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->audio_view_changed((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->set_transition_names((*reinterpret_cast< std::vector<std::string>(*)>(_a[1]))); break;
        case 18: _t->update_channel_name((*reinterpret_cast< Mixer::Output(*)>(_a[1])),(*reinterpret_cast< const std::string(*)>(_a[2]))); break;
        case 19: _t->update_channel_color((*reinterpret_cast< Mixer::Output(*)>(_a[1])),(*reinterpret_cast< const std::string(*)>(_a[2]))); break;
        case 20: _t->gain_staging_knob_changed((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 21: _t->final_makeup_gain_knob_changed((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 22: _t->cutoff_knob_changed((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 23: _t->stereo_width_knob_changed((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 24: _t->eq_knob_changed((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< EQBand(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 25: _t->limiter_threshold_knob_changed((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 26: _t->compressor_threshold_knob_changed((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 27: _t->mini_fader_changed((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 28: _t->mute_button_toggled((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 29: _t->reset_meters_button_clicked(); break;
        case 30: _t->relayout(); break;
        case 31: _t->set_locut((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 32: _t->set_limiter_threshold((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 33: _t->set_makeup_gain((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 34: _t->set_stereo_width((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 35: _t->set_treble((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 36: _t->set_mid((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 37: _t->set_bass((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 38: _t->set_eq_absolute((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< EQBand(*)>(_a[2])),(*reinterpret_cast< float(*)>(_a[3]))); break;
        case 39: _t->set_gain((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 40: _t->set_compressor_threshold((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 41: _t->set_fader((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 42: _t->set_fader_absolute((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 43: _t->toggle_mute((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 44: _t->toggle_locut((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 45: _t->toggle_auto_gain_staging((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 46: _t->toggle_compressor((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 47: _t->clear_peak((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 48: _t->toggle_limiter(); break;
        case 49: _t->toggle_auto_makeup_gain(); break;
        case 50: _t->switch_video_channel((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 51: _t->apply_transition((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 52: _t->prev_audio_view(); break;
        case 53: _t->next_audio_view(); break;
        case 54: _t->begin_new_segment(); break;
        case 55: _t->exit(); break;
        case 56: _t->clear_all_highlights(); break;
        case 57: _t->highlight_locut((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 58: _t->highlight_limiter_threshold((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 59: _t->highlight_makeup_gain((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 60: _t->highlight_stereo_width((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 61: _t->highlight_treble((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 62: _t->highlight_mid((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 63: _t->highlight_bass((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 64: _t->highlight_gain((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 65: _t->highlight_compressor_threshold((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 66: _t->highlight_fader((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 67: _t->highlight_mute((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 68: _t->highlight_toggle_locut((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 69: _t->highlight_toggle_auto_gain_staging((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 70: _t->highlight_toggle_compressor((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 71: _t->highlight_clear_peak((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 72: _t->highlight_toggle_limiter((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 73: _t->highlight_toggle_auto_makeup_gain((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 74: _t->controller_changed((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 75: _t->note_on((*reinterpret_cast< uint(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_MainWindow.data,
    qt_meta_data_MainWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MainWindow.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ControllerReceiver"))
        return static_cast< ControllerReceiver*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 76)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 76;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 76)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 76;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
