/********************************************************************************
** Form generated from reading UI file 'display.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DISPLAY_H
#define UI_DISPLAY_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "glwidget.h"

QT_BEGIN_NAMESPACE

class Ui_Display
{
public:
    QVBoxLayout *main_vertical_layout;
    QFrame *frame;
    QGridLayout *gridLayout;
    GLWidget *display;
    QHBoxLayout *title_bar;
    QLabel *label;
    QPushButton *wb_button;

    void setupUi(QWidget *Display)
    {
        if (Display->objectName().isEmpty())
            Display->setObjectName(QString::fromUtf8("Display"));
        Display->resize(606, 433);
        main_vertical_layout = new QVBoxLayout(Display);
        main_vertical_layout->setObjectName(QString::fromUtf8("main_vertical_layout"));
        main_vertical_layout->setContentsMargins(0, 0, 0, 0);
        frame = new QFrame(Display);
        frame->setObjectName(QString::fromUtf8("frame"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(frame->sizePolicy().hasHeightForWidth());
        frame->setSizePolicy(sizePolicy);
        frame->setAutoFillBackground(true);
        frame->setFrameShape(QFrame::Box);
        frame->setFrameShadow(QFrame::Plain);
        frame->setLineWidth(0);
        gridLayout = new QGridLayout(frame);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(3, 3, 3, 3);
        display = new GLWidget(frame);
        display->setObjectName(QString::fromUtf8("display"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(display->sizePolicy().hasHeightForWidth());
        display->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(display, 0, 0, 1, 1);


        main_vertical_layout->addWidget(frame);

        title_bar = new QHBoxLayout();
        title_bar->setObjectName(QString::fromUtf8("title_bar"));
        label = new QLabel(Display);
        label->setObjectName(QString::fromUtf8("label"));
        QSizePolicy sizePolicy2(QSizePolicy::Ignored, QSizePolicy::Preferred);
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy2);
        label->setMinimumSize(QSize(0, 24));
        label->setAlignment(Qt::AlignCenter);

        title_bar->addWidget(label);

        wb_button = new QPushButton(Display);
        wb_button->setObjectName(QString::fromUtf8("wb_button"));

        title_bar->addWidget(wb_button);


        main_vertical_layout->addLayout(title_bar);


        retranslateUi(Display);

        QMetaObject::connectSlotsByName(Display);
    } // setupUi

    void retranslateUi(QWidget *Display)
    {
        Display->setWindowTitle(QCoreApplication::translate("Display", "Form", nullptr));
        label->setText(QCoreApplication::translate("Display", "TextLabel", nullptr));
        wb_button->setText(QCoreApplication::translate("Display", "Set WB", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Display: public Ui_Display {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DISPLAY_H
